﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="AxisAlignedBoundingBoxBruteForce.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Collision.
//
//   starLiGHT.Collision is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Collision is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Collision. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
// </license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 3216                  $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2011-01-11 11:05:37 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.Collision.BroadPhase
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using starLiGHT.Collections;
    #endregion

    public class AxisAlignedBoundingBoxBruteForce : AbstractBroadPhase, IBroadPhase
    {
        public override bool Test(ICollidable3D a, ICollidable3D b)
        {
            if (a == null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == null)
            {
                throw new ArgumentNullException("b");
            }

            return a.BoundingBox.Intersects(b.BoundingBox);
        }

        public override bool Test(ICollidable2D a, ICollidable2D b)
        {
            throw new NotSupportedException("AxisAlignedBoundingBoxBruteForce is 3D only");
        }

        public override void CleanupPairs(CollisionManager manager)
        {
            if (manager == null)
            {
                throw new ArgumentNullException("manager");
            }

            foreach (UserPair<ICollidable> pair in manager.PairManager.Pairs)
            {
                ICollidable3D a = pair.object0 as ICollidable3D;
                ICollidable3D b = pair.object1 as ICollidable3D;

                if (!a.BoundingBox.Intersects(b.BoundingBox))
                {
                    manager.PairManager.RemovePair(pair.ID0, pair.ID1);
                }
            }
        }

        public override bool CheckCompatibility(ICollidable c)
        {
            try
            {
                if (c is ICollidable3D)
                {
                    return ((ICollidable3D)c).BoundingBox != null;
                }
            }
            catch
            {
            }

            return false;
        }
    }
}
