﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="BoundingCircleBruteForce.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Collision.
//
//   starLiGHT.Collision is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Collision is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Collision. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
// </license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 3216                  $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2011-01-11 11:05:37 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.Collision.BroadPhase
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using Microsoft.Xna.Framework;
    using starLiGHT.Collections;
    #endregion

    public class BoundingCircleBruteForce : AbstractBroadPhase, IBroadPhase
    {
        public override bool Test(ICollidable2D a, ICollidable2D b)
        {
            if (a == null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == null)
            {
                throw new ArgumentNullException("b");
            }

            CircleF circleA = a.BoundingCircle;
            CircleF circleB = b.BoundingCircle;
            Transform2D transformA = a.Transform;
            Transform2D transformB = b.Transform;

            circleA.Center = Vector2.Transform(circleA.Center, transformA.TransformationMatrix);
            circleA.Radius *= transformA.Scale.HasValue ? (transformA.Scale.Value.X + transformA.Scale.Value.Y) * 0.5f : 1.0f;
            
            circleB.Center = Vector2.Transform(circleB.Center, transformB.TransformationMatrix);
            circleB.Radius *= transformB.Scale.HasValue ? (transformB.Scale.Value.X + transformB.Scale.Value.Y) * 0.5f : 1.0f;

            return circleA.Intersects(circleB);
        }

        public override bool Test(ICollidable3D a, ICollidable3D b)
        {
            throw new NotSupportedException("BoundingCircleBruteForce is 2D only");
        }

        public override bool CheckCompatibility(ICollidable c)
        {
            try
            {
                if (c is ICollidable2D)
                {
                    return !((ICollidable2D)c).BoundingCircle.IsEmpty;
                }
            }
            catch
            {
            }

            return false;
        }
    }
}
