﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="ICollidable2D.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2007, 2008, 2009, 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Collision.
//
//   starLiGHT.Collision is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Collision is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Collision. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
// </license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 3345                  $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2011-01-13 16:01:08 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.Collision
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Graphics;
    #endregion

    public interface ICollidable2D : ICollidable
    {
        event EventHandler Moved;

        /// <summary>
        /// Gets the rectangular bounding volume of the collidable object
        /// </summary>
        /// <remarks>
        /// The BoundingRectangle is defined in object space
        /// </remarks>
        RectangleF LocalBoundingRectangle { get; }

        RectangleF WorldBoundingRectangle { get; }

        /// <summary>
        /// Gets the circular bounding volume of the collidable object
        /// </summary>
        /// <remarks>
        /// The BoundingCircle is defined in object space
        /// </remarks>
        CircleF BoundingCircle { get; }

        /// <summary>
        /// Gets the polygonal bounding volume of the collidable object
        /// </summary>
        /// <remarks>
        /// The Hull2D is defined in object space
        /// </remarks>
        Hull2D Hull { get; }

        /// <summary>
        /// Gets the transformation used to transform object space to world space
        /// </summary>
        Transform2D Transform { get; }

        /// <summary>
        /// Gets the texture of the collidable
        /// </summary>
        Texture2D Texture { get; }

        Vector2 Velocity { get; set; }

        void LookAt(Vector2 target);
    }
}
     