﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="XnaViewportBox.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2011 
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.GUI.
//
//   It is released under the Microsoft Public License (Ms-PL).
//
//   You should have received a copy of the Microsoft Public License
//   along with starLiGHT.GUI. If not, see http://sle.codeplex.com/license.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.GUI are available on request.
// </license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 9659                  $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2011-05-04 12:33:30 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.GUI.Widgets
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Content;
    using Microsoft.Xna.Framework.Graphics;
    using Microsoft.Xna.Framework.Input;
    #endregion

    public class XnaViewportBox : Widget, IUpdateableWidget
    {
        #region Constructors
        public XnaViewportBox(GuiManager manager, string name)
            : base(manager, name)
        {
        }

        public XnaViewportBox(GuiManager manager, string name, string layout)
            : base(manager, name)
        {
        }
        #endregion

        public override bool CanFocus
        {
            get
            {
                return false;
            }
        }

        public GraphicsDevice GraphicsDevice
        {
            get
            {
                return Manager.GraphicsDevice;
            }
        }

        public ContentManager Content
        {
            get
            {
                return Manager.Content;
            }
        }


        public Viewport Viewport
        {
            get
            {
                ////TODO: this should be only created when the DisplayRectangle changes...
                Viewport vp = new Viewport();
                vp.X = (int)DisplayRectangle.Left;
                vp.Y = (int)DisplayRectangle.Top;
                vp.Width = (int)DisplayRectangle.Width;
                vp.Height = (int)DisplayRectangle.Height;

                return vp;
            }
        }

        public override void Draw(GameTime gameTime, IGuiRenderer renderer)
        {
            base.Draw(gameTime, renderer);

            Viewport viewportBackup = Manager.GraphicsDevice.Viewport;
            Manager.GraphicsDevice.Viewport = Viewport;

            renderer.ScissorPush(DisplayRectangle);
            renderer.DrawFilledRectangle(DisplayRectangle, BackColor);
            this.Draw(gameTime);
            renderer.ScissorPop();

            Manager.GraphicsDevice.Viewport = viewportBackup;
        }

        public virtual void Draw(GameTime gameTime)
        {
        }

        public virtual void Update(GameTime gameTime)
        {
        }
    }
}
