﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

#endregion

#region COPYRIGHT

/*
    Copyright (c) 2007, 2008, 2009, 2010 
        Roland Rosenkranz (Glatzemann@email.de)
*/

#endregion

#region LICENSE

/*
    This file is part of starLiGHT.Input.

    starLiGHT.Input is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    starLiGHT.Input is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with starLiGHT.Input.  If not, see <http://www.gnu.org/licenses/>.

    
    ADDITIONAL (commercial) LICENSES for starLiGHT.Input are available on request.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1496                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-11 09:38:05 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Input
{
    public enum ActionState
    {
        NONE,
        UP,
        DOWN,
    }

    public abstract class InputAction
    {
        string name;
        LogicalPlayerIndex logicalPlayer;

        public InputAction(string name)
        {
            this.name = name;
        }

        public InputAction(string name, LogicalPlayerIndex logicalPlayer)
            : this(name)
        {
            this.logicalPlayer = logicalPlayer;
        }

        public string Name
        {
            get { return name; }
        }

        public LogicalPlayerIndex LogicalPlayer
        {
            get { return logicalPlayer; }
            set { logicalPlayer = value; }
        }

        public abstract void Update(GameTime gameTime);
        public abstract ActionState GetState();
        public abstract ActionState GetLastState();
        public abstract Vector2 Position { get; }
        public abstract Vector2 LastPosition { get; }

        public virtual bool IsActionDown()
        {
            return GetState() == ActionState.DOWN;
        }

        public virtual bool IsActionHeld()
        {
            return GetState() == ActionState.DOWN && GetLastState() == ActionState.DOWN;
        }

        public virtual bool IsActionUp()
        {
            return GetState() == ActionState.UP;
        }

        public virtual bool IsActionPressed()
        {
            return GetState() == ActionState.DOWN && GetLastState() == ActionState.UP;
        }

        public virtual bool IsActionReleased()
        {
            return GetState() == ActionState.UP && GetLastState() == ActionState.DOWN;
        }
    }
}
