﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

#endregion

#region COPYRIGHT

/*
    Copyright (c) 2007, 2008, 2009, 2010 
        Roland Rosenkranz (Glatzemann@email.de)
*/

#endregion

#region LICENSE

/*
    This file is part of starLiGHT.Input.

    starLiGHT.Input is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    starLiGHT.Input is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with starLiGHT.Input.  If not, see <http://www.gnu.org/licenses/>.

    
    ADDITIONAL (commercial) LICENSES for starLiGHT.Input are available on request.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1496                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-11 09:38:05 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Input
{
    public class TriggerAction : InputAction
    {
        #region Member Variables
        List<IDigitalControl> controls;
        ActionState lastState = ActionState.NONE;
        ActionState actState = ActionState.UP;

        #endregion

        #region Constructors
        public TriggerAction(string name)
            : this(name, null)
        {
        }

        public TriggerAction(string name, IDigitalControl control)
            : base(name)
        {
            this.controls = new List<IDigitalControl>();
            if (control != null)
                this.controls.Add(control);
        }

        #endregion

        public override void Update(GameTime gameTime)
        {
            bool oneReleased = false;
            bool onePressed = false;

            lastState = actState;

            foreach (IDigitalControl control in controls)
            {
                onePressed |= control.Pressed;
                oneReleased |= control.Released;
            }

            if (onePressed)
                actState = ActionState.DOWN;
            else if (oneReleased)
                actState = ActionState.UP;
        }

        public override ActionState GetState()
        {
            return actState;
        }

        public override ActionState GetLastState()
        {
            return lastState;
        }

        public void AddDigitalControl(IDigitalControl digitalControl)
        {
            controls.Add(digitalControl);
        }

        public IEnumerable<IDigitalControl> DigitalControls
        {
            get 
            {
                foreach (IDigitalControl control in controls)
                {
                    yield return control;
                }
            }
            set 
            { 
                controls.Clear();
                controls.AddRange(value);
            }
        }

        public override Vector2 Position
        {
            get { return Vector2.Zero; }
        }

        public override Vector2 LastPosition
        {
            get { return Vector2.Zero; }
        }
    }
}
