﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

#endregion

#region COPYRIGHT

/*
    Copyright (c) 2007, 2008, 2009, 2010 
        Roland Rosenkranz (Glatzemann@email.de)
*/

#endregion

#region LICENSE

/*
    This file is part of starLiGHT.Input.

    starLiGHT.Input is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    starLiGHT.Input is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with starLiGHT.Input.  If not, see <http://www.gnu.org/licenses/>.

    
    ADDITIONAL (commercial) LICENSES for starLiGHT.Input are available on request.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1496                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-11 09:38:05 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Input
{
#if XBOX360
    public class ChatpadDevice : IControlDevice
    {
        KeyboardState lastState;
        KeyboardState actState;
        Dictionary<Keys, KeyControl> keyStates;
        GamepadDevice parentGamePad;

        public ChatpadDevice(GamepadDevice parentGamePad)
        {
            this.parentGamePad = parentGamePad;
            keyStates = new Dictionary<Keys, KeyControl>();
        }

        /// <summary>
        /// Because there's no possibility to recognize a chatpad this property returns true if it is possible to
        /// connect a chatpad to the controller (XBox360 Controller). Reading keys from a not existing chatpad returns
        /// always no pressed keys.
        /// </summary>
        public bool IsConnected
        {
            get { return parentGamePad.HasChatpad; }
        }

        public DeviceType DeviceType
        {
            get { return DeviceType.CHATPAD; }
        }

        public bool HasPlayerIndex
        {
            get { return true; }
        }

        public PlayerIndex PlayerIndex
        {
            get { return parentGamePad.PlayerIndex; }
        }

        public void Update(GameTime gameTime)
        {
            lastState = actState;
            actState = Keyboard.GetState();

            foreach (KeyValuePair<Keys, KeyControl> kvp in keyStates)
            {
                kvp.Value.Update(actState[kvp.Key]);
            }
        }

        public IDigitalControl GetKey(Keys key)
        {
            if (!keyStates.ContainsKey(key))
            {
                keyStates.Add(key, new KeyControl());
            }

            return keyStates[key];
        }
    }
#endif
}
