﻿#region Using Statement
using System;
using System.Collections.Generic;
using System.Text;

#endregion

#region COPYRIGHT

/*
    Copyright (c) 2007, 2008, 2009, 2010 
        Roland Rosenkranz (Glatzemann@email.de)
*/

#endregion

#region LICENSE

/*
    This file is part of starLiGHT.Input.

    starLiGHT.Input is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    starLiGHT.Input is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with starLiGHT.Input.  If not, see <http://www.gnu.org/licenses/>.

    
    ADDITIONAL (commercial) LICENSES for starLiGHT.Input are available on request.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1630                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-11-10 14:20:28 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Input
{
    /// <summary>
    /// Enumeration for describing the different input device types available. This is used
    /// by the DeviceManager e.g.
    /// </summary>
    public enum DeviceType
    {
        /// <summary>
        /// Pointing device that functions by detecting two-dimensional
        /// motion relative to its supporting surface.<br/>
        /// <br/>
        /// This device is available on PC only.
        /// </summary>
        MOUSE,
        /// <summary>
        /// Input device for typing texts with many keys.<br/>
        /// <br/>
        /// This device is available on PC only.
        /// </summary>
        KEYBOARD,
        /// <summary>
        /// XBox360 controller like devices with digital and analog buttons, switches and sticks 
        /// for controlling games.<br/>
        /// <br/>
        /// This device is available on PC and XBox.
        /// </summary>
        GAMEPAD,
        /// <summary>
        /// Keyboard like input device connected to the XBox360 controller.<br/>
        /// <br/>
        /// This device is available on XBox360 only.
        /// </summary>
        CHATPAD,
        /// <summary>
        /// Touch sensitive display which is able to detect up to eigth touches at once with pressure.<br/>
        /// <br/>
        /// This device is available on Zune and Windows 7 Phone.
        /// </summary>
        TOUCHPAD,
        /// <summary>
        /// This input devices recognizes motion in three dimensional space on all axis. It is usually
        /// realized by an accelerometer(s) and/or gyroscope(s).<br/>
        /// <br/>
        /// This device is available on Zune and Windows 7 Phone.
        /// </summary>
        MOTION_SENSING,
        /// <summary>
        /// This input devices is used for the hardware buttons on windows phone.
        /// <br/>
        /// This device is available only on Windows 7 Phone.
        /// </summary>
        PHONE,
    }
}
