﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

#endregion

#region COPYRIGHT

/*
    Copyright (c) 2007, 2008, 2009, 2010 
        Roland Rosenkranz (Glatzemann@email.de)
*/

#endregion

#region LICENSE

/*
    This file is part of starLiGHT.Input.

    starLiGHT.Input is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    starLiGHT.Input is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with starLiGHT.Input.  If not, see <http://www.gnu.org/licenses/>.

    
    ADDITIONAL (commercial) LICENSES for starLiGHT.Input are available on request.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1496                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-11 09:38:05 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Input
{
#if WINDOWS    
    public class MouseDevice : IControlDevice
    {
        MouseState lastMouseState;
        MouseState mouseState;

        ButtonControl leftButton;
        ButtonControl rightButton;
        ButtonControl middleButton;
        ButtonControl xButton1;
        ButtonControl xButton2;
        ScreenPositionControl screenPosition;
        AnalogSingleAxisControl mouseWheel;

        public MouseDevice()
        {
            mouseState = Mouse.GetState();

            leftButton = new ButtonControl();
            rightButton = new ButtonControl();
            middleButton = new ButtonControl();
            xButton1 = new ButtonControl();
            xButton2 = new ButtonControl();
            screenPosition = new ScreenPositionControl();
            mouseWheel = new AnalogSingleAxisControl();
        }

        public bool IsConnected
        {
            get { return true; }
        }

        public DeviceType DeviceType
        {
            get { return DeviceType.MOUSE; }
        }

        public bool HasPlayerIndex
        {
            get { return false; }
        }

        public PlayerIndex PlayerIndex
        {
            get { return PlayerIndex.One; }
        }

        public void Update(GameTime gameTime)
        {
            lastMouseState = mouseState;
            mouseState = Mouse.GetState();

            leftButton.Update(mouseState.LeftButton);
            rightButton.Update(mouseState.RightButton);
            middleButton.Update(mouseState.MiddleButton);
            xButton1.Update(mouseState.XButton1);
            xButton2.Update(mouseState.XButton2);

            screenPosition.Update(mouseState.X, mouseState.Y);
            mouseWheel.Update(mouseState.ScrollWheelValue);
        }

        public ButtonControl LeftButton
        {
            get { return leftButton; }
        }

        public ButtonControl MiddleButton
        {
            get { return middleButton; }
        }

        public ButtonControl RightButton
        {
            get { return rightButton; }
        }

        public ButtonControl XButton1
        {
            get { return xButton1; }
        }

        public ButtonControl XButton2
        {
            get { return xButton2; }
        }

        public ScreenPositionControl ScreenPosition
        {
            get { return screenPosition; }
        }

        public AnalogSingleAxisControl MouseWheel
        {
            get { return mouseWheel; }
        }
    }

#endif
}
