﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="Modifier.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2008, 2009, 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Particles.
//
//   starLiGHT.Particles is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Particles is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Particles. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Particles are available on request.
// </license>
// <original_license>
//  * FLINT PARTICLE SYSTEM
//  * .....................
//  * 
//  * Author: Richard Lord
//  * Copyright (c) Richard Lord 2008-2009
//  * http://flintparticles.org
//  * 
//  * 
//  * Licence Agreement
//  * 
//  * Permission is hereby granted, free of charge, to any person obtaining a copy
//  * of this software and associated documentation files (the "Software"), to deal
//  * in the Software without restriction, including without limitation the rights
//  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  * copies of the Software, and to permit persons to whom the Software is
//  * furnished to do so, subject to the following conditions:
//  * 
//  * The above copyright notice and this permission notice shall be included in
//  * all copies or substantial portions of the Software.
//  * 
//  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  * THE SOFTWARE.
// </original_license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 620                   $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2010-12-07 15:23:58 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.ParticleSystem.Modifier
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using starLiGHT.ParticleSystem.Common;

    #endregion

    /// <summary>
    /// The Modifier interface must be implemented by all particle modifiers.
    ///  
    /// <p>An Modifier is a class that is used to continuously modify an aspect 
    /// of a particle by updating the particle every frame. Modifiers may, for 
    /// example, move the particle or modify its velocity.</p>
    ///  
    /// <p>Modifiers are directly associated with emitters and act on all
    /// particles created or added to that emitter. Modifiers are applied to 
    /// all particles created by an emitter by using the emitter's addModifier 
    /// method. Modifiers are removed from the emitter by using the emitter's
    /// removeModifier method.<p>
    /// 
    /// <p>The key method in the Modifier interface is the update method.
    /// This is called every frame, for every particle and is where the
    /// modifiers alters the particle's properties.</p>
    /// </summary>
    public interface IModifier : IBehaviour
    {
        /// <summary>
        /// The update method is used by the emitter to apply the modifier
        /// to every particle. It is the key feature of the modifiers and is
        /// used to update the state of every particle every frame. This method 
        /// is called within the emitter's update loop for every particle 
        /// and need not be called by the user.
        /// <p>Because the method is called for every particle, every frame it is
        /// a key area for optimization of the code. When creating a custom modifier
        /// it is usually worth making this method as efficient as possible.</p>        
        /// </summary>
        /// <param name="emitter">The Emitter that created the particle.</param>
        /// <param name="particle">The particle to be updated.</param>
        /// <param name="time">The duration of the frame - used for time based updates.</param>
        void Update(Emitter emitter, Particle particle, float time);
    }
}
