﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="Particle.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2008, 2009, 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Particles.
//
//   starLiGHT.Particles is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Particles is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Particles. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Particles are available on request.
// </license>
// <original_license>
//  * FLINT PARTICLE SYSTEM
//  * .....................
//  * 
//  * Author: Richard Lord
//  * Copyright (c) Richard Lord 2008-2009
//  * http://flintparticles.org
//  * 
//  * 
//  * Licence Agreement
//  * 
//  * Permission is hereby granted, free of charge, to any person obtaining a copy
//  * of this software and associated documentation files (the "Software"), to deal
//  * in the Software without restriction, including without limitation the rights
//  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  * copies of the Software, and to permit persons to whom the Software is
//  * furnished to do so, subject to the following conditions:
//  * 
//  * The above copyright notice and this permission notice shall be included in
//  * all copies or substantial portions of the Software.
//  * 
//  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  * THE SOFTWARE.
// </original_license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 620                   $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2010-12-07 15:23:58 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.ParticleSystem.Common
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Graphics;

    #endregion

    /// <summary>
    /// The Particle class is a set of public properties shared by all particles.
    /// It is deliberately lightweight, with only one method. The Initializers
    /// and Actions modify these properties directly. This means that the same
    /// particles can be used in many different emitters, allowing Particle 
    /// objects to be reused.
    /// Particles are usually created by the ParticleCreator class. This class
    /// just simplifies the reuse of Particle objects which speeds up the
    /// application. 
    /// </summary>
    public class Particle
    {
        public Color color = Color.White;

        public float scale = 1.0f;
        public float mass = 1.0f;
        public float collisionRadius = 1.0f;
        public float lifetime;
        public float age;
        public float energy = 1.0f;
        public bool isDead = false;

        private Texture2D texture;
        private Color _previousColor;

        public Particle()
        {
        }

        public virtual void initialize()
        {
            color = Color.White;
            scale = 1f;
            mass = 1f;
            collisionRadius = 1f;
            lifetime = 0f;
            age = 0f;
            energy = 1f;
            isDead = false;
        }

        /// <summary>
        /// Alpha value is in the range of 0..255
        /// </summary>
        public byte Alpha
        {
            get { return color.A; }
        }

        protected Particle cloneInto(Particle p)
        {
            p.color = color;
            p.scale = scale;
            p.mass = mass;
            p.collisionRadius = collisionRadius;
            p.lifetime = lifetime;
            p.age = age;
            p.energy = energy;
            p.isDead = isDead;

            ////////TODO: implement the rest

            return p;
        }

        public virtual Particle clone()
        {
            return clone(null);
        }

        public virtual Particle clone(IParticleFactory factory)
        {
            Particle p;
            if (factory != null)
            {
                p = factory.createParticle();
            }
            else
            {
                p = new Particle();
            }

            return cloneInto(p);
        }

        public Texture2D Texture
        {
            get { return texture; }
            set { texture = value; }
        }
    }
}
