﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="PulseGenerator.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2008, 2009, 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Particles.
//
//   starLiGHT.Particles is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Particles is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Particles. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Particles are available on request.
// </license>
// <original_license>
//  * FLINT PARTICLE SYSTEM
//  * .....................
//  * 
//  * Author: Richard Lord
//  * Copyright (c) Richard Lord 2008-2009
//  * http://flintparticles.org
//  * 
//  * 
//  * Licence Agreement
//  * 
//  * Permission is hereby granted, free of charge, to any person obtaining a copy
//  * of this software and associated documentation files (the "Software"), to deal
//  * in the Software without restriction, including without limitation the rights
//  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  * copies of the Software, and to permit persons to whom the Software is
//  * furnished to do so, subject to the following conditions:
//  * 
//  * The above copyright notice and this permission notice shall be included in
//  * all copies or substantial portions of the Software.
//  * 
//  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  * THE SOFTWARE.
// </original_license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 620                   $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2010-12-07 15:23:58 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.ParticleSystem.PulseGenerators
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using starLiGHT.ParticleSystem.Common;

    #endregion

    /// <summary>
    /// The PulseGenerator interface must be implemented by all PulseGenerators.
    /// <para>
    /// A PulseGenerator is a class that tells an emitter how many particles to
    /// emit at any time. The two methods control the rate of emission of particles 
    /// when the emitter starts and every frame thereafter.
    /// </para>
    /// <para>
    /// A PulseGenerator is directly associated with an emitter. A PulseGenerator is set for 
    /// an emitter by assigning it to the emitter's PulseGenerator property.
    /// </para>
    /// </summary>
    public interface IPulseGenerator
    {
        /// <summary>
        /// The startEmitter method is called when the emitter starts.
        /// </summary>
        /// <remarks>
        /// This method is private and called by the emitter's start method.
        /// </remarks>
        /// <returns>
        /// The number of particles the emitter should emit when it starts.
        /// </returns>
        uint startEmitter(Emitter emitter);

        /// <summary>
        /// The updateEmitter method is called every frame after the emitter has been started.
        /// </summary>
        /// <remarks>
        /// This method is private and called by the emitter's update loop.
        /// </remarks>
        /// <param name="time">The time in seconds since the previous call of this method.</param>
        /// <returns>The number of particles the emitter should emit at this time.</returns>
        uint updateEmitter(Emitter emitter, float time);

        /// <summary>
        /// Stops the PulseGenerator instructing the emitter to emit particles
        /// </summary>
        void stop();

        /// <summary>
        /// Resumes the PulseGenerator after a stop.
        /// </summary>
        void resume();

        /// <summary>
        /// Indicates, that the PulseGenerator has raised all it's pulses and no more 
        /// particles need to be emitted.
        /// </summary>
        bool Complete { get; }
    }
}
