﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="UpdateOnFrame.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2008, 2009, 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Particles.
//
//   starLiGHT.Particles is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Particles is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Particles. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Particles are available on request.
// </license>
// <original_license>
//  * FLINT PARTICLE SYSTEM
//  * .....................
//  * 
//  * Author: Richard Lord
//  * Copyright (c) Richard Lord 2008-2009
//  * http://flintparticles.org
//  * 
//  * 
//  * Licence Agreement
//  * 
//  * Permission is hereby granted, free of charge, to any person obtaining a copy
//  * of this software and associated documentation files (the "Software"), to deal
//  * in the Software without restriction, including without limitation the rights
//  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  * copies of the Software, and to permit persons to whom the Software is
//  * furnished to do so, subject to the following conditions:
//  * 
//  * The above copyright notice and this permission notice shall be included in
//  * all copies or substantial portions of the Software.
//  * 
//  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  * THE SOFTWARE.
// </original_license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 620                   $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2010-12-07 15:23:58 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.ParticleSystem.Common
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;

    #endregion

    /// <summary>
    /// The UpdateOnFrame activity is used to call a frameUpdate method of an object 
    /// that implements the FrameUpdatable interface. The frameUpdate method is called
    /// once every frame.
    /// <p>This activity is most often used to update an action once every frame.
    /// Because the update method of an action is called once for every particle, 
    /// every frame, there is no obvious point for an action to implement code
    /// that should be called once only every frame, irrespective of the number
    /// of particles. If the action implements FrameUpdatable and adds an 
    /// UpdateOnFrame activity to the emitter in its addedToEmitter method, the
    /// frameUpdate method will be called once every frame.</p>
    /// <p>See the Explosion Action for an example of an action that uses this 
    /// activity.</p>
    /// </summary>
    public class UpdateOnFrame : ActivityBase
    {
        private IFrameUpdatable action;

        /// <summary>
        /// The constructor creates an UpdateOnFrame activity.
        /// </summary>
        /// <param name="frameUpdatable">The object that shouldbe updated every frame.</param>
        public UpdateOnFrame(IFrameUpdatable frameUpdatable)
        {
            action = frameUpdatable;
        }

        /// <summary>
        /// Calls the frameUpdate method of the FrameUpdatable object associated
        /// with this activity.
        /// <p>This method is called by the emitter and need not be called by the 
        /// user</p>
        /// </summary>
        /// <param name="emitter">The Emitter.</param>
        /// <param name="time">The duration of the frame - used for time based updates.</param>
        public override void update(Emitter emitter, float time)
        {
            action.frameUpdate(emitter, time);
        }
    }
}
