﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="TwoWay.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2008, 2009, 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Particles.
//
//   starLiGHT.Particles is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Particles is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Particles. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Particles are available on request.
// </license>
// <original_license>
//  * FLINT PARTICLE SYSTEM
//  * .....................
//  * 
//  * Author: Richard Lord
//  * Copyright (c) Richard Lord 2008-2009
//  * http://flintparticles.org
//  * 
//  * 
//  * Licence Agreement
//  * 
//  * Permission is hereby granted, free of charge, to any person obtaining a copy
//  * of this software and associated documentation files (the "Software"), to deal
//  * in the Software without restriction, including without limitation the rights
//  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  * copies of the Software, and to permit persons to whom the Software is
//  * furnished to do so, subject to the following conditions:
//  * 
//  * The above copyright notice and this permission notice shall be included in
//  * all copies or substantial portions of the Software.
//  * 
//  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  * THE SOFTWARE.
// </original_license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 620                   $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2010-12-07 15:23:58 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.ParticleSystem.Common.EnergyEasing
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;

    #endregion

    /// <summary>
    /// A set of easing equations for modifying the particle energy such that it starts 
    /// and ends at zero and peaks at one half way through the particle's lifetime.
    /// </summary>
    public class TwoWay
    {
        /// <summary>
        /// Gives a linear increase and decrease in energy either side of the centre point.
        /// </summary>
        public static float linear(float age, float lifetime)
        {
            if ((age = 2 * age / lifetime) <= 1)
            {
                return age;
            }

            return 2 - age;
        }

        /// <summary>
        /// Energy increases and then decreases as if following the top half of a circle.
        /// </summary>
        public static float circular(float age, float lifetime)
        {
            age = 1 - (2 * age / lifetime);
            return (float)Math.Sqrt(1 - age * age);
        }

        /// <summary>
        /// Energy follows the first half of a sine wave.
        /// </summary>
        public static float sine(float age, float lifetime)
        {
            return (float)Math.Sin(Math.PI * age / lifetime);
        }

        /// <summary>
        /// Eases towards the middle using a quadratic curve.
        /// </summary>
        public static float quadratic(float age, float lifetime)
        {
            age = 1 - (2 * age / lifetime);
            return -(age * age - 1);
        }

        /// <summary>
        /// Eases towards the middle using a cubic curve.
        /// </summary>
        public static float cubic(float age, float lifetime)
        {
            age = 1 - (2 * age / lifetime);
            if (age < 0) age = -age;
            return -(age * age * age - 1);
        }

        /// <summary>
        /// Eases towards the middle using a quartic curve.
        /// </summary>
        public static float quartic(float age, float lifetime)
        {
            age = 1 - (2 * age / lifetime);
            return -(age * age * age * age - 1);
        }

        /// <summary>
        /// Eases towards the middle using a quintic curve.
        /// </summary>
        public static float quintic(float age, float lifetime)
        {
            age = 1 - (2 * age / lifetime);
            if (age < 0) age = -age;
            return -(age * age * age * age * age - 1);
        }
    }
}
