﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="RandomTexture.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2008, 2009, 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Particles.
//
//   starLiGHT.Particles is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Particles is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Particles. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Particles are available on request.
// </license>
// <original_license>
//  * FLINT PARTICLE SYSTEM
//  * .....................
//  * 
//  * Author: Richard Lord
//  * Copyright (c) Richard Lord 2008-2009
//  * http://flintparticles.org
//  * 
//  * 
//  * Licence Agreement
//  * 
//  * Permission is hereby granted, free of charge, to any person obtaining a copy
//  * of this software and associated documentation files (the "Software"), to deal
//  * in the Software without restriction, including without limitation the rights
//  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  * copies of the Software, and to permit persons to whom the Software is
//  * furnished to do so, subject to the following conditions:
//  * 
//  * The above copyright notice and this permission notice shall be included in
//  * all copies or substantial portions of the Software.
//  * 
//  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  * THE SOFTWARE.
// </original_license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 620                   $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2010-12-07 15:23:58 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.ParticleSystem.Initializer
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using Microsoft.Xna.Framework.Graphics;
    using starLiGHT.Collections;
    using starLiGHT.ParticleSystem.Common;

    #endregion

    public class RandomTexture : InitializerBase
    {
        #region Member variables
        private WeightedList<Texture2D> _textures;
        private WeightedList<string> _textureFileNames;

        #endregion

        #region Constructors
        /// <summary>
        /// The constructor creates a RandomTexture initializer to be used by an emitter.
        /// </summary>
        public RandomTexture()
        {
            _textures = new WeightedList<Texture2D>();
        }

        /// <summary>
        /// The constructor creates a RandomTexture initializer to be used by an emitter.
        /// </summary>
        public RandomTexture(List<WeightValuePair<Texture2D>> weightValuePairs)
            : this()
        {
            if (weightValuePairs == null) return;

            foreach (WeightValuePair<Texture2D> c in weightValuePairs)
            {
                _textures.Add(c);
            }
        }

        /// <summary>
        /// The constructor creates a RandomTexture initializer to be used by an emitter.
        /// </summary>
        public RandomTexture(List<Texture2D> weightValuePairs)
            : this()
        {
            if (weightValuePairs == null) return;

            foreach (Texture2D t in weightValuePairs)
            {
                _textures.Add(1.0f, t);
            }
        }

        #endregion

        /// <summary>
        /// Adds a new texture to the weighted list of textures.
        /// </summary>
        /// <param name="tex">The texture to be added. It has a default weight of 1.0f.</param>
        public void addTexture(Texture2D tex)
        {
            _textures.Add(1.0f, tex);
        }

        /// <summary>
        /// Adds a new texture to the weighted list of textures.
        /// </summary>
        /// <param name="tex">The texture to be added.</param>
        /// <param name="weight">The weight that this texture should have.</param>
        public void addTexture(Texture2D tex, float weight)
        {
            _textures.Add(weight, tex);
        }

        public WeightedList<Texture2D> Textures
        {
            get { return _textures; }
        }

        public void addTextureReference(string fileName, float weight)
        {
            if (_textureFileNames == null) _textureFileNames = new WeightedList<string>();
            _textureFileNames.Add(weight, fileName);
        }

        public WeightedList<string> TextureReferences
        {
            get { return _textureFileNames; }
        }

        /// <summary>
        /// The initialize method is used by the emitter to apply the initialization
        /// to every particle. It is the key feature of the initializers and is
        /// used to initialize the state of every particle. This method 
        /// is called within the emitter's createParticle method for every particle 
        /// and need not be called by the user.
        /// </summary>
        /// <param name="emitter">The Emitter that created the particle.</param>
        /// <param name="particle">The particle to be initialized.</param>
        /// <remarks>This is an internal method that is called by the emitter.</remarks>
        public override void initialize(Emitter emitter, ref Particle particle)
        {
            particle.Texture = _textures.RandomValue;
        }        
    }
}
