﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="DeathRegion.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2008, 2009, 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Particles.
//
//   starLiGHT.Particles is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Particles is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Particles. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Particles are available on request.
// </license>
// <original_license>
//  * FLINT PARTICLE SYSTEM
//  * .....................
//  * 
//  * Author: Richard Lord
//  * Copyright (c) Richard Lord 2008-2009
//  * http://flintparticles.org
//  * 
//  * 
//  * Licence Agreement
//  * 
//  * Permission is hereby granted, free of charge, to any person obtaining a copy
//  * of this software and associated documentation files (the "Software"), to deal
//  * in the Software without restriction, including without limitation the rights
//  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  * copies of the Software, and to permit persons to whom the Software is
//  * furnished to do so, subject to the following conditions:
//  * 
//  * The above copyright notice and this permission notice shall be included in
//  * all copies or substantial portions of the Software.
//  * 
//  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  * THE SOFTWARE.
// </original_license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 7773                  $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2011-03-28 02:04:04 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.ParticleSystem.Modifier.twoD
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using starLiGHT.ParticleSystem.Common;
    using starLiGHT.ParticleSystem.twoD;

    #endregion

    /// <summary>
    /// The DeathRegion modifier marks the particle as dead if it is inside or outside 
    /// a specific region.
    /// </summary>
    /// <remarks>
    /// This modifier is using a default priority of -20, so that it executes after all movement 
    /// has occured.
    /// </remarks>
    public class DeathRegion : Modifier2DBase
    {
        #region Member Variables
        private IRegion2D _region;
        private bool _invertRegion;
        private Particle2D p;
        private bool inside;

        #endregion

        #region Constructors
        /// <summary>
        /// The constructor creates a DeathRegion to be used by an emitter.
        /// </summary>
        public DeathRegion()
            : this(null, false)
        {
        }

        /// <summary>
        /// The constructor creates a DeathRegion to be used by an emitter.
        /// </summary>
        /// <param name="region">
        /// The region to use. Any class of the Region namespace can be used.
        /// </param>
        public DeathRegion(IRegion2D region)
            : this(region, false)
        {
        }

        /// <summary>
        /// The constructor creates a DeathRegion to be used by an emitter.
        /// </summary>
        /// <param name="region">
        /// The region to use. Any class of the Region namespace can be used.
        /// </param>
        /// <param name="regionIsSafe">
        /// If true, the region is treated as the safe area and particles outside the 
        /// region are killed immediately. If false, particles inside the region are 
        /// killed.
        /// </param>
        public DeathRegion(IRegion2D region, bool regionIsSafe)
        {
            Priority = -20;
            _region = region;
            _invertRegion = regionIsSafe;
        }

        #endregion

        /// <summary>
        /// The region that is used by this modifier to decide whether a particle 
        /// has to be killed or not.
        /// </summary>
        public IRegion2D Region
        {
            get { return _region; }
            set { _region = value; }
        }

        /// <summary>
        /// If true, the region is treated as the safe area and particles outside the 
        /// region are killed immediately. If false, particles inside the region are 
        /// killed.
        /// </summary>
        public bool RegionIsSafe
        {
            get { return _invertRegion; }
            set { _invertRegion = value; }
        }

        /// <summary>
        /// Applies the acceleration to a particle for the specified time period.
        /// </summary>
        /// <remarks>
        /// This method is called by the emitter and need not be called by the user.
        /// </remarks>
        /// <param name="emitter">The Emitter that created the particle.</param>
        /// <param name="particle">The particle to update.</param>
        /// <param name="time">The time since the last update.</param>
        public override void Update(Emitter2D emitter, Particle2D particle, float time)
        {
            inside = _region.contains(particle.position);
            particle.isDead = (_invertRegion && !inside) || (!_invertRegion && inside);
        }
    }
}
