﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="DeathSpeed.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2008, 2009, 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Particles.
//
//   starLiGHT.Particles is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Particles is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Particles. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Particles are available on request.
// </license>
// <original_license>
//  * FLINT PARTICLE SYSTEM
//  * .....................
//  * 
//  * Author: Richard Lord
//  * Copyright (c) Richard Lord 2008-2009
//  * http://flintparticles.org
//  * 
//  * 
//  * Licence Agreement
//  * 
//  * Permission is hereby granted, free of charge, to any person obtaining a copy
//  * of this software and associated documentation files (the "Software"), to deal
//  * in the Software without restriction, including without limitation the rights
//  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  * copies of the Software, and to permit persons to whom the Software is
//  * furnished to do so, subject to the following conditions:
//  * 
//  * The above copyright notice and this permission notice shall be included in
//  * all copies or substantial portions of the Software.
//  * 
//  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  * THE SOFTWARE.
// </original_license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 7773                  $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2011-03-28 02:04:04 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.ParticleSystem.Modifier.twoD
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using starLiGHT.ParticleSystem.Common;
    using starLiGHT.ParticleSystem.twoD;

    #endregion

    /// <summary>
    /// The DeathSpeed Modifier action marks the particle as dead if it is travelling faster 
    /// or slower than the specified speed.
    /// </summary>
    public class DeathSpeed : Modifier2DBase
    {
        #region Member Variables
        private float _limit;
        private float _limitSq;
        private bool _isMinimum;

        #endregion

        #region Constructors

        /// <summary>
        /// The constructor createas a DeathSpeed modifier to be used by an emitter.
        /// </summary>
        public DeathSpeed()
            : this(float.MaxValue, false)
        {
        }

        /// <summary>
        /// The constructor createas a DeathSpeed modifier to be used by an emitter.
        /// </summary>
        /// <param name="speed">
        /// The speed at which the modifier should take effect.
        /// </param>
        public DeathSpeed(float speed)
            : this(speed, false)
        {
        }

        /// <summary>
        /// The constructor createas a DeathSpeed modifier to be used by an emitter.
        /// </summary>
        /// <param name="speed">
        /// The speed at which the modifier should take effect.
        /// </param>
        /// <param name="isMinimum">
        /// Indicates wheter this is a upper speed limit (false) or a minimum speed (true)
        /// </param>
        public DeathSpeed(float speed, bool isMinimum)
        {
            _limit = speed;
            _limitSq = _limit * _limit;
            _isMinimum = isMinimum;
        }

        #endregion

        /// <summary>
        /// The speed limit which is used to decide if the particle needs to die.
        /// </summary>
        public float Limit
        {
            get 
            { 
                return _limit; 
            }

            set 
            { 
                _limit = value; 
                _limitSq = _limit * _limit; 
            }
        }

        /// <summary>
        /// Indicates whether the speed is a minimum speed (true) or a speed limit (false).
        /// </summary>
        public bool IsMinimum
        {
            get { return _isMinimum; }
            set { _isMinimum = value; }
        }

        /// <summary>
        /// Checks the particle's speed and marks it as dead if it is moving faster 
        /// than the speed limit, if this is a maximum speed, or slower if this is a 
        /// minimum speed.
        /// </summary>
        /// <remarks>
        /// This method is called by the emitter and need not be called by the user.
        /// </remarks>
        /// <param name="emitter">The Emitter that created the particle.</param>
        /// <param name="particle">The particle to update.</param>
        /// <param name="time">The time since the last update.</param>
        public override void Update(Emitter2D emitter, Particle2D particle, float time)
        {
            float speedSq = particle.velocity.X * particle.velocity.X + particle.velocity.Y * particle.velocity.Y;
            particle.isDead = (_isMinimum && speedSq < _limitSq) || (!_isMinimum && speedSq > _limitSq);
        }
    }
}
