﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="Friction.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2008, 2009, 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Particles.
//
//   starLiGHT.Particles is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Particles is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Particles. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Particles are available on request.
// </license>
// <original_license>
//  * FLINT PARTICLE SYSTEM
//  * .....................
//  * 
//  * Author: Richard Lord
//  * Copyright (c) Richard Lord 2008-2009
//  * http://flintparticles.org
//  * 
//  * 
//  * Licence Agreement
//  * 
//  * Permission is hereby granted, free of charge, to any person obtaining a copy
//  * of this software and associated documentation files (the "Software"), to deal
//  * in the Software without restriction, including without limitation the rights
//  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  * copies of the Software, and to permit persons to whom the Software is
//  * furnished to do so, subject to the following conditions:
//  * 
//  * The above copyright notice and this permission notice shall be included in
//  * all copies or substantial portions of the Software.
//  * 
//  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  * THE SOFTWARE.
// </original_license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 7773                  $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2011-03-28 02:04:04 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.ParticleSystem.Modifier.twoD
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using starLiGHT.ParticleSystem.Common;
    using starLiGHT.ParticleSystem.twoD;

    #endregion

    /// <summary>
    /// The Friction modifier applies friction to the particle to slow it down when 
    /// it's moving. The frictional force is constant, irrespective of how fast the 
    /// particle is moving. For forces proportional to the particle's velocity, use 
    /// one of the drag effect: LinearDrag or QuadraticDrag.
    /// </summary>
    public class Friction : Modifier2DBase
    {
        #region Member Variables
        private float _friction;

        #endregion

        #region Constructors
        /// <summary>
        /// The constructor creates a Friction modifier to be used by an emitter.
        /// </summary>
        public Friction()
            : this(0)
        {
        }

        /// <summary>
        /// The constructor creates a Friction modifier to be used by an emitter.
        /// </summary>
        /// <param name="number">
        /// </param>
        public Friction(float number)
        {
            _friction = number;
        }

        #endregion

        /// <summary>
        /// The amount of friction. A higher number produces a stringer frictional force.
        /// </summary>
        public float FrictionValue
        {
            get { return _friction; }
            set { _friction = value; }
        }

        /// <summary>
        /// Calculates the effect of the friction on the particle over the period of time 
        /// passed since the last update and adjusts the particle's velocity accodingly.
        /// </summary>
        /// <remarks>
        /// This method is called by the emitter and need not be called by the user.
        /// </remarks>
        /// <param name="emitter">The Emitter that created the particle.</param>
        /// <param name="particle">The particle to update.</param>
        /// <param name="time">The time since the last update.</param>
        public override void Update(Emitter2D emitter, Particle2D particle, float time)
        {
            float len2 = particle.velocity.X * particle.velocity.X + particle.velocity.Y * particle.velocity.Y;
            if (len2 == 0)
            {
                return;
            }

            float scale = 1 - (_friction * time) / (Math.Sign(len2) * particle.mass);
            if (scale < 0)
            {
                particle.velocity.X = 0;
                particle.velocity.Y = 0;
            }
            else
            {
                particle.velocity.X = particle.velocity.X * scale;
                particle.velocity.Y = particle.velocity.Y * scale;
            }
        }
    }
}
