﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="Jet.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2008, 2009, 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Particles.
//
//   starLiGHT.Particles is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Particles is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Particles. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Particles are available on request.
// </license>
// <original_license>
//  * FLINT PARTICLE SYSTEM
//  * .....................
//  * 
//  * Author: Richard Lord
//  * Copyright (c) Richard Lord 2008-2009
//  * http://flintparticles.org
//  * 
//  * 
//  * Licence Agreement
//  * 
//  * Permission is hereby granted, free of charge, to any person obtaining a copy
//  * of this software and associated documentation files (the "Software"), to deal
//  * in the Software without restriction, including without limitation the rights
//  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  * copies of the Software, and to permit persons to whom the Software is
//  * furnished to do so, subject to the following conditions:
//  * 
//  * The above copyright notice and this permission notice shall be included in
//  * all copies or substantial portions of the Software.
//  * 
//  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  * THE SOFTWARE.
// </original_license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 7773                  $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2011-03-28 02:04:04 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.ParticleSystem.Modifier.twoD
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using starLiGHT.ParticleSystem.Common;
    using starLiGHT.ParticleSystem.Modifier.twoD;
    using starLiGHT.ParticleSystem.twoD;

    #endregion

    /// <summary>
    /// The Jet modifier applies an acceleration to particles only if they are in 
    /// the specified region.
    /// </summary>
    public class Jet : Modifier2DBase
    {
        #region Member Variables
        private float _x;
        private float _y;
        private IRegion2D _region;
        private bool _invert;

        #endregion

        #region Constructors
        public Jet()
            : this(0, 0, null, false)
        {
        }

        public Jet(float accelerationX, float accelerationY)
            : this(accelerationX, accelerationY, null, false)
        {
        }

        public Jet(float accelerationX, float accelerationY, IRegion2D region)
            : this(accelerationX, accelerationY, region, false)
        {
        }

        public Jet(float accelerationX, float accelerationY, IRegion2D region, bool invertRegion)
        {
            _x = accelerationX;
            _y = accelerationY;
            _region = region;
            _invert = invertRegion;
        }
        #endregion

        #region Properties

        public float AccelerationX
        {
            get { return _x; }
            set { _x = value; }
        }

        public float AccelerationY
        {
            get { return _y; }
            set { _y = value; }
        }

        public IRegion2D Region
        {
            get { return _region; }
            set { _region = value; }
        }

        public bool InvertRegion
        {
            get { return _invert; }
            set { _invert = value; }
        }

        #endregion

        public override void Update(Emitter2D emitter, Particle2D particle, float time)
        {
            if (_region.contains(particle.position))
            {
                if (!_invert)
                {
                    particle.velocity.X += _x * time;
                    particle.velocity.Y += _y * time;
                }
            }
            else
            {
                if (_invert)
                {
                    particle.velocity.X += _x * time;
                    particle.velocity.Y += _y * time;
                }
            }
        }
    }
}
