﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="TweenTowardsPoint.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2008, 2009, 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Particles.
//
//   starLiGHT.Particles is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Particles is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Particles. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Particles are available on request.
// </license>
// <original_license>
//  * FLINT PARTICLE SYSTEM
//  * .....................
//  * 
//  * Author: Richard Lord
//  * Copyright (c) Richard Lord 2008-2009
//  * http://flintparticles.org
//  * 
//  * 
//  * Licence Agreement
//  * 
//  * Permission is hereby granted, free of charge, to any person obtaining a copy
//  * of this software and associated documentation files (the "Software"), to deal
//  * in the Software without restriction, including without limitation the rights
//  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  * copies of the Software, and to permit persons to whom the Software is
//  * furnished to do so, subject to the following conditions:
//  * 
//  * The above copyright notice and this permission notice shall be included in
//  * all copies or substantial portions of the Software.
//  * 
//  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  * THE SOFTWARE.
// </original_license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 7773                  $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2011-03-28 02:04:04 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.ParticleSystem.Modifier.twoD
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using Microsoft.Xna.Framework;
    using starLiGHT.ParticleSystem.Common;
    using starLiGHT.ParticleSystem.twoD;

    #endregion

    /// <summary>
    /// The TurnTowardsPoint modifier causes the particle to constantly adjust its 
    /// direction so that it travels towards a particular point.
    /// </summary>
    public class TurnTowardsPoint : Modifier2DBase
    {
        private float _x;
        private float _y;
        private float _power;

        public TurnTowardsPoint()
            : this(0, 0, 0)
        {
        }

        public TurnTowardsPoint(Vector2 point)
            : this(point.X, point.Y, 0)
        {
        }

        public TurnTowardsPoint(Vector2 point, float power)
            : this(point.X, point.Y, power)
        {
        }

        public TurnTowardsPoint(float x, float y)
            : this(x, y, 0.0f)
        {
        }

        public TurnTowardsPoint(float x, float y, float power)
        {
            _x = x;
            _y = y;
            _power = power;
        }

        public float X
        {
            get { return _x; }
            set { _x = value; }
        }

        public float Y
        {
            get { return _y; }
            set { _y = value; }
        }

        public float Power
        {
            get { return _power; }
            set { _power = value; }
        }

        public override void Update(Emitter2D emitter, Particle2D particle, float time)
        {
            float velLength = (float)Math.Sqrt(particle.velocity.X * particle.velocity.X + particle.velocity.Y * particle.velocity.Y);
            float dx = particle.velocity.X / velLength;
            float dy = particle.velocity.Y / velLength;
            float acc = Power * time;
            float targetX = _x - particle.position.X;
            float targetY = _y - particle.position.Y;
            float len = (float)Math.Sqrt(targetX * targetX + targetY * targetY);
            if (len == 0.0f)
            {
                return;
            }

            targetX /= len;
            targetY /= len;
            float dot = targetX * dx + targetY * dy;
            float perpX = targetX - dx * dot;
            float perpY = targetY - dy * dot;
            float factor = acc / (float)Math.Sqrt(perpX * perpX + perpY * perpY);
            particle.velocity.X += perpX * factor;
            particle.velocity.Y += perpY * factor;
            factor = velLength / (float)Math.Sqrt(particle.velocity.X * particle.velocity.X + particle.velocity.Y * particle.velocity.Y);
            particle.velocity.X *= factor;
            particle.velocity.Y *= factor;
        }
    }
}
