﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="DiscSectorRegion.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2008, 2009, 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Particles.
//
//   starLiGHT.Particles is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Particles is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Particles. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Particles are available on request.
// </license>
// <original_license>
//  * FLINT PARTICLE SYSTEM
//  * .....................
//  * 
//  * Author: Richard Lord
//  * Copyright (c) Richard Lord 2008-2009
//  * http://flintparticles.org
//  * 
//  * 
//  * Licence Agreement
//  * 
//  * Permission is hereby granted, free of charge, to any person obtaining a copy
//  * of this software and associated documentation files (the "Software"), to deal
//  * in the Software without restriction, including without limitation the rights
//  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  * copies of the Software, and to permit persons to whom the Software is
//  * furnished to do so, subject to the following conditions:
//  * 
//  * The above copyright notice and this permission notice shall be included in
//  * all copies or substantial portions of the Software.
//  * 
//  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  * THE SOFTWARE.
// </original_license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 7773                  $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2011-03-28 02:04:04 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.ParticleSystem.twoD
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using Microsoft.Xna.Framework;
    using starLiGHT.ParticleSystem.Common;

    #endregion

    public class DiscSectorRegion : IRegion2D
    {
        private Vector2 _center;
        private float _innerRadius;
        private float _outerRadius;
        private float _innerSq;
        private float _outerSq;
        private float _minAngle;
        private float _maxAngle;
        private float _minAllowed;
        private System.Random random = new System.Random();

        public DiscSectorRegion()
            : this(Vector2.Zero, 0, 0, 0, 0)
        {
        }

        public DiscSectorRegion(Vector2 center, float outerRadius, float innerRadius, float minAngle, float maxAngle)
        {
            if (outerRadius < innerRadius)
            {
                throw new Exception("The outerRadius (" + outerRadius + ") can't be smaller than the innerRadius (" + innerRadius + ") in your DiscRegion. N.B. the outerRadius is the second argument in the constructor and the innerRadius is the third argument.");
            }

            this._center = center;
            this._innerRadius = innerRadius;
            this._outerRadius = outerRadius;
            this._innerSq = this._innerRadius * this._innerRadius;
            this._outerSq = this._outerRadius * this._outerRadius;
            this._minAngle = minAngle;
            this._maxAngle = maxAngle;
            if (_maxAngle != 0.0f)
            {
                while (_maxAngle > MathHelper.TwoPi)
                {
                    _maxAngle -= MathHelper.TwoPi;
                }

                while (_maxAngle < 0)
                {
                    _maxAngle += MathHelper.TwoPi;
                }

                _minAngle = _maxAngle - MathHelper.TwoPi;
                if (_minAngle != 0.0f)
                {
                    if (minAngle == maxAngle)
                    {
                        _minAngle = _maxAngle;
                    }
                    else
                    {
                        _minAngle = clamp(_minAngle);
                    }
                }
            }
        }

        private float clamp(float angle)
        {
            if (_maxAngle != 0.0f)
            {
                while (angle > _maxAngle)
                {
                    angle -= MathHelper.TwoPi;
                }

                while (angle < _minAllowed)
                {
                    angle += MathHelper.TwoPi;
                }
            }

            return angle;
        }

        public Vector2 Center
        {
            get { return this._center; }
            set { this._center = value; }
        }

        public float InnerRadius
        {
            get 
            {
                return this._innerRadius; 
            }

            set
            {
                this._innerRadius = value;
                this._innerSq = this._innerRadius * this._innerRadius;
            }
        }

        public float OuterRadius
        {
            get 
            {
                return this._outerRadius; 
            }

            set
            {
                this._outerRadius = value;
                this._outerSq = this._outerRadius * this._outerRadius;
            }
        }

        public float MinAngle
        {
            get { return this._minAngle; }
            set { this._minAngle = clamp(value); }
        }

        public float MaxAngle
        {
            get 
            {
                return this._maxAngle; 
            }

            set
            {
                this._maxAngle = value;
                while (this._maxAngle > MathHelper.TwoPi)
                {
                    this._maxAngle -= MathHelper.TwoPi;
                }

                while (this._maxAngle < 0)
                {
                    this._maxAngle += MathHelper.TwoPi;
                }

                this._minAllowed = this._maxAngle - MathHelper.TwoPi;
                this._minAngle = clamp(this._minAngle);
            }
        }

        public bool contains(Vector2 pos)
        {
            pos -= this._center;
            float distSq = pos.X * pos.X + pos.Y * pos.Y;
            if (distSq > this._outerSq || distSq < this._innerSq)
            {
                return false;
            }

            float angle = (float)Math.Atan2(pos.Y, pos.X);
            angle = clamp(angle);
            return angle >= this._minAngle;
        }

        public Vector2 getLocation()
        {
            float rand = (float)random.NextDouble();
            Vector2 point = polar2cartesian(Vector2.Zero, this._innerRadius + (1 - rand * rand) * (this._outerRadius - this._innerRadius), random.NextFloat(this._minAngle, this._maxAngle));

            point += this._center;

            return point;
        }

        public float getArea()
        {
            return (float)(Math.PI * this._outerSq - Math.PI * this._innerSq);
        }

        private Vector2 polar2cartesian(Vector2 v, float dist, float bearing)
        {
            v.X = (float)(v.X + dist * Math.Sin(bearing));
            v.Y = (float)(v.Y + dist * Math.Cos(bearing));

            return v;
        }
    }
}
