﻿#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="RotateVelocity.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2008, 2009, 2010, 2011
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Particles.
//
//   starLiGHT.Particles is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Particles is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Particles. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Particles are available on request.
// </license>
// <original_license>
//  * FLINT PARTICLE SYSTEM
//  * .....................
//  * 
//  * Author: Richard Lord
//  * Copyright (c) Richard Lord 2008-2009
//  * http://flintparticles.org
//  * 
//  * 
//  * Licence Agreement
//  * 
//  * Permission is hereby granted, free of charge, to any person obtaining a copy
//  * of this software and associated documentation files (the "Software"), to deal
//  * in the Software without restriction, including without limitation the rights
//  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  * copies of the Software, and to permit persons to whom the Software is
//  * furnished to do so, subject to the following conditions:
//  * 
//  * The above copyright notice and this permission notice shall be included in
//  * all copies or substantial portions of the Software.
//  * 
//  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  * THE SOFTWARE.
// </original_license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 7773                  $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2011-03-28 02:04:04 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.ParticleSystem.Initializer.TwoD
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using starLiGHT.ParticleSystem.Common;
    using starLiGHT.ParticleSystem.Initializer;
    using starLiGHT.ParticleSystem.twoD;

    #endregion

    /// <summary>
    /// The RotateVelocity Initializer sets the angular velocity of the particle. It is 
    /// usually combined with the Rotation modifier to rotate the particle using this 
    /// angular velocity.
    /// </summary>
    /// <remarks>
    /// The angular velocity of particles initialized by this class will be a random value 
    /// between the minimum and maximum values set. If no maximum value is set, the minimum 
    /// value is used with no variation.
    /// </remarks>
    public class RotateVelocity : InitializerBase
    {
        #region Member Variables
        private float _min;
        private float _max;

        #endregion

        #region Constructors
        /// <summary>
        /// The constructor creates a RotateVelocity Initializer to be used by an emitter.
        /// </summary>
        public RotateVelocity()
            : this(0.0f, float.NaN)
        {
        }

        /// <summary>
        /// The constructor creates a RotateVelocity Initializer to be used by an emitter.
        /// </summary>
        public RotateVelocity(float minAngle, float maxAngle)
        {
            _min = minAngle;
            _max = maxAngle;
        }

        /// <summary>
        /// The constructor creates a RotateVelocity Initializer to be used by an emitter.
        /// </summary>
        public RotateVelocity(float angle)
        {
            _min = _max = angle;
        }

        #endregion

        public float MinAngularVelocity
        {
            get { return _min; }
            set { _min = value; }
        }

        public float MaxAngularVelocity
        {
            get { return _max; }
            set { _max = value; }
        }

        public float AngularVelocity
        {
            get { return _min == _max ? _min : (_max + _min) * 0.5f; }
            set { _max = _min = value; }
        }

        /// <summary>
        /// The initialize method is used by the emitter to apply the initialization
        /// to every particle. It is the key feature of the initializers and is
        /// used to initialize the state of every particle. This method 
        /// is called within the emitter's createParticle met6hod for every particle 
        /// and need not be called by the user.
        /// </summary>
        /// <param name="emitter">The Emitter that created the particle.</param>
        /// <param name="particle">The particle to be initialized.</param>
        /// <remarks>This is an internal method that is called by the emitter.</remarks>
        public override void initialize(Emitter emitter, ref Particle particle)
        {
            Particle2D p = (Particle2D)particle;
            if (float.IsNaN(_max))
            {
                p.angularVelocity = _min;
            }
            else
            {
                p.angularVelocity = emitter.Random.NextFloat(_min, _max);
            }
        }
    }
}
