﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using starLiGHT.Physic.TwoD;
using Microsoft.Xna.Framework;

#endregion

#region COPYRIGHT

/*
    Copyright (c) 2008, 2009, 2010 
        Roland Rosenkranz (Glatzemann@email.de)
*/

#endregion

#region LICENSE

/*
    This file is part of starLiGHT.Collision.

    starLiGHT.Collision is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    starLiGHT.Collision is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with starLiGHT.Collision.  If not, see <http://www.gnu.org/licenses/>.

    
    ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1486                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-08 15:55:47 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Physic
{
    public class RevoluteJoint2D : Joint2D
    {
        #region Constructors
        /// <summary>
        /// Creates a new RevoluteJoint. The initial revolute angle is zero. The 
        /// anchor point is at the first body's center of mass.
        /// </summary>
        /// <param name="body1">"Left" body to connect by the distance joint.</param>
        /// <param name="body2">"Right" body to connect by the distance joint.</param>
        public RevoluteJoint2D(RigidBody2D body1, RigidBody2D body2)
        {
            RevoluteJointDef jointDef = new RevoluteJointDef();
            jointDef.Initialize(body1.Body, body2.Body, body1.Position);
            jointDef.collideConnected = false;

            base.Joint = PhysicSystem2D.Instance.World.CreateJoint(jointDef);

            PhysicSystem2D.Instance.managedJoints.Add(this);
        }

        /// <summary>
        /// Creates a new RevoluteJoint. The initial revolute angle is zero.
        /// </summary>
        /// <param name="body1">"Left" body to connect by the distance joint.</param>
        /// <param name="body2">"Right" body to connect by the distance joint.</param>
        /// <param name="worldAnchor1">Anchor on "left" body in world coordinates.</param>
        /// <param name="worldAnchor2">Anchor on "right" body in world coordinates.</param>
        public RevoluteJoint2D(RigidBody2D body1, RigidBody2D body2, Vector2 worldAnchor)
        {
            RevoluteJointDef jointDef = new RevoluteJointDef();
            jointDef.Initialize(body1.Body, body2.Body, worldAnchor);
            jointDef.collideConnected = false;

            base.Joint = PhysicSystem2D.Instance.World.CreateJoint(jointDef);

            PhysicSystem2D.Instance.managedJoints.Add(this);
        }

        #endregion

        /// <summary>
        /// The internal joint of the physic system.
        /// </summary>
        public new RevoluteJoint Joint
        {
            get { return base.Joint as RevoluteJoint; }
        }
    }
}
