﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

#endregion

#region COPYRIGHT

/*
    Copyright (c) 2008, 2009, 2010 
        Roland Rosenkranz (Glatzemann@email.de)
*/

#endregion

#region LICENSE

/*
    This file is part of starLiGHT.Collision.

    starLiGHT.Collision is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    starLiGHT.Collision is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with starLiGHT.Collision.  If not, see <http://www.gnu.org/licenses/>.

    
    ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

namespace starLiGHT.Physic.TwoD
{
    public class SKPoly6 : SmoothingKernel
    {
        public SKPoly6(float kernelSize)
            : base(kernelSize)
        { }

        internal override void calculateFactor()
        {
            double kernelRad9 = System.Math.Pow(m_kernelSize, 9.0f);
            m_factor = (float)(315.0f / (64.0f * Math.PI * kernelRad9));
        }

        internal override float calculate(Vector2 distance)
        {
            float lenSq = distance.LengthSquared();
            if (lenSq > base.m_kernelSizeSq)
            {
                return 0.0f;
            }

            if (lenSq < PhysicSettings.FLT_EPSILON)
            {
                lenSq = PhysicSettings.FLT_EPSILON;
            }

            float diffSq = m_kernelSizeSq - lenSq;
            return m_factor * diffSq * diffSq * diffSq;
        }

        internal override Vector2 calculateGradient(Vector2 distance)
        {
            float lenSq = distance.LengthSquared();
            if (lenSq > m_kernelSizeSq)
            {
                return Vector2.Zero;
            }

            if (lenSq < PhysicSettings.FLT_EPSILON)
            {
                lenSq = PhysicSettings.FLT_EPSILON;
            }

            float diffSq = m_kernelSizeSq - lenSq;
            float f = -m_factor * 6.0f * diffSq * diffSq;
            return distance * f;
        }

        internal override float calculateLaplacian(Vector2 distance)
        {
            throw new NotImplementedException();
        }
    }
}
