﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

#endregion

#region COPYRIGHT

/*
    Copyright (c) 2008, 2009, 2010 
        Roland Rosenkranz (Glatzemann@email.de)
*/

#endregion

#region LICENSE

/*
    This file is part of starLiGHT.Collision.

    starLiGHT.Collision is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    starLiGHT.Collision is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with starLiGHT.Collision.  If not, see <http://www.gnu.org/licenses/>.

    
    ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

namespace starLiGHT.Physic.TwoD
{
    public class SKSpiky : SmoothingKernel
    {
        public SKSpiky(float kernelSize)
            : base(kernelSize)
        { }

        internal override void calculateFactor()
        {
            double kernelRad6 = System.Math.Pow(m_kernelSize, 6.0f);
            m_factor = (float)(15.0f / (Math.PI * kernelRad6));
        }

        internal override float calculate(Vector2 distance)
        {
            float lenSq = distance.LengthSquared();
            if (lenSq > base.m_kernelSizeSq)
            {
                return 0.0f;
            }

            if (lenSq < PhysicSettings.FLT_EPSILON)
            {
                lenSq = PhysicSettings.FLT_EPSILON;
            }

            float f = m_kernelSize - (float)System.Math.Sqrt(lenSq);
            return m_factor * f * f * f;
        }

        internal override Vector2 calculateGradient(Vector2 distance)
        {
            float lenSq = distance.LengthSquared();
            if (lenSq > m_kernelSizeSq)
            {
                return Vector2.Zero;
            }

            if (lenSq < PhysicSettings.FLT_EPSILON)
            {
                lenSq = PhysicSettings.FLT_EPSILON;
            }

            float len =(float)System.Math.Sqrt(lenSq);
            float f = -m_factor * 3.0f * (m_kernelSize - len) * (m_kernelSize - len) / len;
            return distance * f;
        }

        internal override float calculateLaplacian(Vector2 distance)
        {
            throw new NotImplementedException();
        }
    }
}
