#region COPYRIGHT
//--------------------------------------------------------------------------------
// <copyright file="MouseCameraController.cs" company="starLiGHT Entertainment Studios">
//    Copyright (c) 2007, 2008, 2009, 2010
//       Roland Rosenkranz (Glatzemann@email.de)
// </copyright>
// <license>
//   This file is part of starLiGHT.Render.
//
//   starLiGHT.Render is free software: you can redistribute it and/or modify
//   it under the terms of the GNU Lesser General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   starLiGHT.Core is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public License
//   along with starLiGHT.Render. If not, see http://www.gnu.org/licenses/.
//    
//   ADDITIONAL (commercial) LICENSES for starLiGHT.Render are available on request.
// </license>
// <version>
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 620                   $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2010-12-07 15:23:58 #$:  Date of last commit     *
// ************************************************************
// </version>
//--------------------------------------------------------------------------------
#endregion

namespace starLiGHT.Camera.TwoD
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Text;
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Input;

    #endregion

    public class MouseCameraController : ICameraController
    {
        private MouseState lastState;
        private float zoomSpeed = 0.01f;

        public string Name
        {
            get { return "Mouse"; }
        }

        public void Update(GameTime gameTime, Camera2D camera)
        {
            MouseState state = Mouse.GetState();

            if (state.X != this.lastState.X || state.Y != this.lastState.Y)
            {
                camera.Anchor = new Vector2(state.X, state.Y) - (camera.ViewportSize * 0.5f);
            }

            if (state.ScrollWheelValue > this.lastState.ScrollWheelValue)
            {
                camera.Zoom += ((state.ScrollWheelValue - this.lastState.ScrollWheelValue) / 120.0f) * this.zoomSpeed;
            }

            if (state.ScrollWheelValue < this.lastState.ScrollWheelValue)
            {
                camera.Zoom -= ((this.lastState.ScrollWheelValue - state.ScrollWheelValue) / 120.0f) * this.zoomSpeed;
            }

            this.lastState = state;
        }
    }
}
