﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using starLiGHT.Collision;
using starLiGHT.Collision.BroadPhase;
using starLiGHT.Collision.NarrowPhase;
using starLiGHT.Components.SandboxManager;
using starLiGHT.Collections;

namespace starLiGHT.Sandbox.Collision.Tests
{
    [SandboxTest("Asteroids")]
    class Asteroids : CollisionSystemTest
    {
        #region Member Variables
        private float shootRepeatSpeed = 75f; //Milliseconds
        private float shootFloatingSpeed = 10f; //Pixel per Update
        private float shootLifetime = 1f; //s

        private Microsoft.Xna.Framework.Game game;
        private bool debugDraw = true;
        private FastList<HullObject> shoots;
        private FastList<float[]> shootsHelperValues;
        private int shootdemage = 1;
        private Texture2D shootTexture;
        private Vector2 shootOrigin;

        private Texture2D enemysTexture;
        private FastList<HullObject> enemys;
        private int enemylife=1;
        private float enemySpeed = 5f; //Pixel per Update
        private float enemyRespawnrate = 800f; //Milliseconds
        private float enemyLastSpawn = 0f;
        private Vector2 enemysOrigin;
        private Random randomSpawnPoint = new Random();

        private HullObject player1;
        private float playerspeed = 3;
        private float lastFire = 75f; //Milliseconds

        private Texture2D crossHair;

        private MouseState newMouseState = new MouseState();

        private int screenWidth = 0;
        private int screenHeight = 0;
        #endregion

        public override void Initialize(Game1 game)
        {
            this.game = game;

            screenWidth = game.GraphicsDevice.PresentationParameters.BackBufferWidth;
            screenHeight = game.GraphicsDevice.PresentationParameters.BackBufferHeight;

            CollisionManager = new starLiGHT.Collision.CollisionManager(BroadPhaseType.AxisAlignedRectangleBruteForce,
                                                                        NarrowPhaseType.PerPixel,
                                                                        false, false, true);
            CollisionManager.Collision += new EventHandler<CollisionEventArgs>(CollisionManager_Collision);

            shoots = new FastList<HullObject>();
            shootsHelperValues = new FastList<float[]>();

            // add the Player
            Texture2D player1Texturepreload = game.Content.Load<Texture2D>("Asteroids/Player1");
            player1 = new HullObject(player1Texturepreload.CenterOrigin(), new RectangleF(screenWidth / 2f, screenHeight / 2f, player1Texturepreload.Width, player1Texturepreload.Height).Corners);
            player1.Tag = "Player";
            player1.Texture = game.Content.Load<Texture2D>("Asteroids/Player1");
            CollisionManager.AddCollidable(player1);

            crossHair = game.Content.Load<Texture2D>("Asteroids/CrossHair");

            shootTexture = game.Content.Load<Texture2D>("Asteroids/Shoot");
            shootOrigin = shootTexture.CenterOrigin();

            //preload EnemyTexture
            enemysTexture = game.Content.Load<Texture2D>("Asteroids/Enemy");
            enemysOrigin = enemysTexture.CenterOrigin();

            enemys = new FastList<HullObject>();
        }
        void CollisionManager_Collision(object sender, CollisionEventArgs e)
        {

            if (e.Manifold.NarrowPhaseCollision)
            {
                ICollidable2D a;
                ICollidable2D b;

                if (e.Manifold.Object0.Tag.ToString() == "Shoot")
                {
                    a = e.Manifold.Object0 as ICollidable2D;
                    b = e.Manifold.Object1 as ICollidable2D;
                }
                else if (e.Manifold.Object1.Tag.ToString() == "Shoot")
                {
                    a = e.Manifold.Object1 as ICollidable2D;
                    b = e.Manifold.Object0 as ICollidable2D;
                }
                else
                {
                    return;
                }

                if (b.Tag.ToString().Contains("enemy"))
                {
                    b.Tag = "enemy" + (Convert.ToInt32(b.Tag.ToString().Substring(5)) - shootdemage).ToString();
                    a.Tag = "HIT";
                }
            }

        }
        public override void Update(GameTime gameTime, KeyboardState actState, KeyboardState lastState)
        {
            newMouseState = Mouse.GetState();
            player1.LookAt(newMouseState.GetPosition());

            if (newMouseState.LeftButton == ButtonState.Pressed && lastFire >= shootRepeatSpeed)
            {
                shoots.Add(new HullObject(shootOrigin, new RectangleF(player1.Position.X, player1.Position.Y, (float)shootTexture.Width, (float)shootTexture.Height).Corners));
                HullObject lastShoot = shoots.Last();
                lastShoot.Texture = shootTexture;
                lastShoot.Orientation = player1.Orientation;
                lastShoot.Tag = "Shoot";

                float[] HelperValues = {shootLifetime,(float)Math.Cos(lastShoot.Orientation),(float)Math.Sin(lastShoot.Orientation) };
                shootsHelperValues.Add(HelperValues);

                CollisionManager.AddCollidable(lastShoot);
                lastFire = 0f;
            }

            if (shoots.Count != shootsHelperValues.Count)
            {
                throw new Exception("shoots and shootsHelperValues has not the same Count");
            }

            if (actState.IsKeyDown(Keys.W))
            {
                player1.Position -= Vector2.UnitY * playerspeed;
            }
            if (actState.IsKeyDown(Keys.S))
            {
                player1.Position += Vector2.UnitY * playerspeed;
            }
            if (actState.IsKeyDown(Keys.A))
            {
                player1.Position -= Vector2.UnitX * playerspeed;
            }
            if (actState.IsKeyDown(Keys.D))
            {
                player1.Position += Vector2.UnitX * playerspeed;
            }

            if (shoots.Count > 0)
            {
                for (int shootsCount = 0; shootsCount < shoots.Count && shootsCount < shootsHelperValues.Count; shootsCount++)
                {
                    if (shootsHelperValues[shootsCount][0] > 0)
                    {
                        if (shoots[shootsCount].Tag.ToString() != "Shoot")
                        {
                            CollisionManager.RemoveCollidable(shoots[shootsCount]);
                            shootsHelperValues.RemoveAt(shootsCount);
                            shoots.RemoveAt(shootsCount);
                            shootsCount--;
                        }
                        else
                        {
                            shoots[shootsCount].Position += new Vector2(shootsHelperValues[shootsCount][1] * shootFloatingSpeed, shootsHelperValues[shootsCount][2] * shootFloatingSpeed);
                            shootsHelperValues[shootsCount][0] -= (float)gameTime.ElapsedGameTime.Milliseconds / 1000;
                        }
                    }
                    else
                    {
                        CollisionManager.RemoveCollidable(shoots[shootsCount]);
                        shootsHelperValues.RemoveAt(shootsCount);
                        shoots.RemoveAt(shootsCount);
                        shootsCount--;
                    }
                }
            }

            if (enemys.Count > 0)
            {
                for (int enemysCount = 0; enemysCount < enemys.Count; enemysCount++)
                {
                    enemys[enemysCount].LookAt(player1.Position);
                    enemys[enemysCount].Position += new Vector2((float)Math.Cos(enemys[enemysCount].Orientation), (float)Math.Sin(enemys[enemysCount].Orientation) * enemySpeed);
                    if (Convert.ToInt32(enemys[enemysCount].Tag.ToString().Substring(5)) <= 0)
                    {
                        CollisionManager.RemoveCollidable(enemys[enemysCount]);
                        enemys.RemoveAt(enemysCount);
                        enemysCount--;
                    }
                }
            }

            if (enemyLastSpawn >= enemyRespawnrate)
            {
                enemys.Add(new HullObject(enemysOrigin, new RectangleF(randomSpawnPoint.NextFloat(enemysTexture.Width, screenWidth - enemysTexture.Width), randomSpawnPoint.NextFloat(enemysTexture.Height, screenHeight - enemysTexture.Height), (float)enemysTexture.Width, (float)enemysTexture.Height).Corners));
                HullObject lastenemy = enemys.Last();
                lastenemy.Texture = enemysTexture;
                lastenemy.LookAt(player1.Position);
                lastenemy.Tag = "enemy"+enemylife.ToString();

                CollisionManager.AddCollidable(lastenemy);
                enemyLastSpawn = 0;
            }

            if (actState.IsKeyDown(Keys.F1) && lastState.IsKeyUp(Keys.F1))
            {
                debugDraw = !debugDraw;
            }

            lastFire += (lastFire >= shootRepeatSpeed ? 0 : gameTime.ElapsedGameTime.Milliseconds);
            enemyLastSpawn += (enemyLastSpawn >= enemyRespawnrate ? 0 : gameTime.ElapsedGameTime.Milliseconds);
            base.Update(gameTime, actState, lastState);
        }

        public override void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            game.GraphicsDevice.Clear(Color.Black);
            spriteBatch.Begin();
            for (int shootsCount = 0; shootsCount < shoots.Count && shootsCount < shootsHelperValues.Count; shootsCount++)
            {
                spriteBatch.Draw(shoots[shootsCount].Texture, shoots[shootsCount].Position, null, Color.White, shoots[shootsCount].Orientation, shoots[shootsCount].Origin, shoots[shootsCount].Scale, SpriteEffects.None, 1);
            }
            for (int enemysCount = 0; enemysCount < enemys.Count; enemysCount++)
            {
                spriteBatch.Draw(enemys[enemysCount].Texture, enemys[enemysCount].Position, null, Color.White, enemys[enemysCount].Orientation, enemys[enemysCount].Origin, enemys[enemysCount].Scale, SpriteEffects.None, 1);
            }
            spriteBatch.Draw(player1.Texture, player1.Position, null, Color.White, player1.Orientation, player1.Origin, player1.Scale, SpriteEffects.None, 1);
            spriteBatch.Draw(crossHair, new Vector2((float)newMouseState.X, (float)newMouseState.Y),null,Color.White,0,crossHair.CenterOrigin(),1, SpriteEffects.None,0);

            if (debugDraw)
            {
                CollisionManager.DrawDebug(spriteBatch);
            }

            spriteBatch.End();
        }
    }
}
