﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using starLiGHT.Components.SandboxManager;
using starLiGHT.Collision;
using starLiGHT.Collision.BroadPhase;
using starLiGHT.Collision.NarrowPhase;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using starLiGHT.Collections;

#endregion

namespace starLiGHT.Sandbox.Collision.Tests
{

    [SandboxTest("Doodle Jump")]
    public sealed class DoodleJump : CollisionSystemTest
    {
        #region Member Variables
        private Microsoft.Xna.Framework.Game game;
        private bool debugDraw = true;
        private HullObject doodle;
        private FastList<HullObject> jumpPads;
        private float actHeight = 360f;
        private Vector2 acceleration;
        private Random rnd;

        #endregion

        public override void Initialize(Game1 game)
        {
            this.game = game;
            rnd = new Random();

            CollisionManager = new starLiGHT.Collision.CollisionManager(BroadPhaseType.AxisAlignedRectangleBruteForce,
                                                                        NarrowPhaseType.SeparatingAxisTheorem,
                                                                        false, false, true);
            CollisionManager.Collision += new EventHandler<CollisionEventArgs>(CollisionManager_Collision);

            jumpPads = new FastList<HullObject>(25);

            HullObject jumpPad = new HullObject(new RectangleF(0f, 0f, 64f, 16f).Corners);
            jumpPad.Position = new Vector2(640f - 32f, 640f - 32f);
            jumpPads.Add(jumpPad);
            CollisionManager.AddCollidable(jumpPad);

            doodle = new HullObject(new RectangleF(0f, 0f, 32f, 32f).Corners);
            doodle.Position = new Vector2(640f - 16f, 360f - 16f);
            doodle.Tag = "Doodle";
            CollisionManager.AddCollidable(doodle);
        }

        void CollisionManager_Collision(object sender, CollisionEventArgs e)
        {
            if (e.Manifold.NarrowPhaseCollision)
            {
                ICollidable2D jumpPadCollidable = null;
                ICollidable2D doodleCollidable = null;

                if (e.Pair.object0.Tag != null && e.Pair.object0.Tag.ToString() == "Doodle")
                {
                    doodleCollidable = e.Pair.object0 as HullObject;
                    jumpPadCollidable = e.Pair.object1 as HullObject;
                }
                else if (e.Pair.object1.Tag != null && e.Pair.object1.Tag.ToString() == "Doodle")
                {
                    jumpPadCollidable = e.Pair.object0 as HullObject;
                    doodleCollidable = e.Pair.object1 as HullObject;
                }

                if (jumpPadCollidable != null && doodleCollidable != null)
                {
                    doodle.Transform.Position -= e.Manifold.Separation.Value;
                    doodle.Velocity -= e.Manifold.Separation.Value;
                    doodle.Velocity += new Vector2(0f, doodle.Velocity.Y * -2f);
                }
            }
        }

        public override void Update(GameTime gameTime, KeyboardState actState, KeyboardState lastState)
        {
            if (actState.IsKeyDown(Keys.D) && lastState.IsKeyUp(Keys.D))
            {
                debugDraw = !debugDraw;
            }

            // create new JumpPads if necessary
            UpdateJumpPads();

            // integrate gravity and other velocities
            acceleration.Y = 40f;
            float deltaTime = (float)gameTime.ElapsedGameTime.TotalSeconds;
            float deltaTimeSquaredDividedByTwo = (deltaTime * deltaTime) * 0.5f;
            Vector2 pos = doodle.Position;
            pos.X = Mouse.GetState().X;
            pos.X += (doodle.Velocity.X * deltaTime) + (acceleration.X * deltaTimeSquaredDividedByTwo);
            pos.Y += (doodle.Velocity.Y * deltaTime) + (acceleration.Y * deltaTimeSquaredDividedByTwo);
            doodle.Velocity += acceleration * deltaTime;
            doodle.Position = pos;

            // calculate the new height
            actHeight = -doodle.Position.Y;

            base.Update(gameTime, actState, lastState);
        }

        public override void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            spriteBatch.Begin();

            String text = "Height: " + actHeight.ToString();
            Vector2 pos = new Vector2(1100f, 0f);
            spriteBatch.DrawString(((Game1)game).descriptionFont, text, pos, Color.White);
            
            if (debugDraw)
            {
                CollisionManager.DrawDebug(spriteBatch);
            }

            spriteBatch.End();
        }

        private void UpdateJumpPads()
        {
            //TODO: dirty hack because of issue #128
            CollisionManager.RemoveCollidable(doodle);

            float lastHeight = jumpPads.Count > 0 ? jumpPads[0].Position.Y : 360f;

            while (jumpPads.Count < 5)
            {
                float x = rnd.NextFloat(0f, 1216f);
                lastHeight = rnd.NextFloat(lastHeight - 120f, lastHeight - 240f);

                HullObject ho = new HullObject(new RectangleF(x, lastHeight, 64f, 16f).Corners);
                jumpPads.Add(ho);
                CollisionManager.AddCollidable(ho);
            }

            //TODO: dirty hack because of issue #128
            CollisionManager.AddCollidable(doodle);
        }
    }
}
