﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using starLiGHT.Components.SandboxManager;
using starLiGHT.Collision;
using starLiGHT.Collision.BroadPhase;
using starLiGHT.Collision.NarrowPhase;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

#endregion

namespace starLiGHT.Sandbox.Collision
{
    [SandboxTest("Scrolling Cave (Horizontal Shooter)")]
    public sealed class HorizontalShooter : CollisionSystemTest
    {
        TextureObject[] caveTop;
        TextureObject[] caveBottom;

        Boolean debugDraw;

        TextureObject ufo;

        public override void Initialize(Game1 game)
        {
            CollisionManager = new starLiGHT.Collision.CollisionManager(BroadPhaseType.AxisAlignedRectangleBruteForce,
                                                                        NarrowPhaseType.PerPixel,
                                                                        true, false, false);

            caveTop = new TextureObject[3];
            caveBottom = new TextureObject[3];

            for (int i = 0; i < 3; i++)
            {
                caveTop[i] = new TextureObject(new Vector2(i * 640f + 320f, 120.0f), game.Content.Load<Texture2D>("HorizontalShooter/cave_top"), true);
                caveBottom[i] = new TextureObject(new Vector2(i * 640f + 320f, 600.0f), game.Content.Load<Texture2D>("HorizontalShooter/cave_bottom"), true);

                CollisionManager.AddCollidable(caveTop[i]);
                CollisionManager.AddCollidable(caveBottom[i]);
            }

            ufo = new TextureObject(new Vector2(640f, 360f), game.Content.Load<Texture2D>(@"HorizontalShooter/ufo"), true);
            CollisionManager.AddCollidable(ufo);

            CollisionManager.Collision += new EventHandler<CollisionEventArgs>(CollisionManager_Collision);
        }

        void CollisionManager_Collision(object sender, CollisionEventArgs e)
        {
            ICollidable2D a = e.Pair.object0 as ICollidable2D;
            ICollidable2D b = e.Pair.object1 as ICollidable2D;

            if (a != null && b != null && (a.Texture.Width < 640 || b.Texture.Width < 640))
            {
                ufo.Colliding = true;
            }
        }

        public override void Update(GameTime gameTime, KeyboardState actState, KeyboardState lastState)
        {
            Vector2 offset = new Vector2(35f * (float)gameTime.ElapsedGameTime.TotalSeconds, 0f);

            for (int i = 0; i < 3; i++)
            {
                caveTop[i].Position -= offset;
                if (caveTop[i].Position.X < -640.0f) 
                    caveTop[i].Position += new Vector2(1280.0f, 0.0f);

                caveBottom[i].Position -= offset;
                if (caveBottom[i].Position.X < -640.0f) 
                    caveBottom[i].Position += new Vector2(1280.0f, 0.0f);
            }

            if (actState.IsKeyDown(Keys.Up))
                ufo.Position -= Vector2.UnitY;

            if (actState.IsKeyDown(Keys.Down))
                ufo.Position += Vector2.UnitY;

            if (actState.IsKeyDown(Keys.Left))
                ufo.Position -= Vector2.UnitX;

            if (actState.IsKeyDown(Keys.Right))
                ufo.Position += Vector2.UnitX;

            if (actState.IsKeyDown(Keys.D) && lastState.IsKeyUp(Keys.D))
                debugDraw = !debugDraw;

            base.Update(gameTime, actState, lastState);
        }

        public override void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            spriteBatch.Begin();

            for (int i = 0; i < 3; i++)
            {
                spriteBatch.Draw(caveTop[i].Texture, caveTop[i].Position, null, Color.White, 0f, caveTop[i].Texture.CenterOrigin(), 1f, SpriteEffects.None, 1.0f);
                spriteBatch.Draw(caveBottom[i].Texture, caveBottom[i].Position, null, Color.White, 0f, caveBottom[i].Texture.CenterOrigin(), 1f, SpriteEffects.None, 1.0f);
            }

            spriteBatch.Draw(ufo.Texture, ufo.Position, null, ufo.Colliding ? Color.Red : Color.White, 0f, ufo.Texture.CenterOrigin(), 1f, SpriteEffects.None, 1.0f);

            if (debugDraw)
            {
                CollisionManager.DrawDebug(spriteBatch);
            }

            spriteBatch.End();        
        }
    }
}
