﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using starLiGHT.Components.SandboxManager;
using starLiGHT.Collision;
using starLiGHT.Collision.BroadPhase;
using starLiGHT.Collision.NarrowPhase;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

#endregion

namespace starLiGHT.Sandbox.Collision
{
    [SandboxTest("Isometric Tiles (2.5D collision)")]
    public sealed class IsometricTiles : CollisionSystemTest
    {
        Microsoft.Xna.Framework.Game game;

        Texture2D grassTile;
        Texture2D grassTileDebug;
        bool grassDebug = false;
        bool debugDraw = false;

        Texture2D house;
        HullObject houseHull;
        HullObject houseHull2;

        Knight knight;

        public override void Initialize(Game1 game)
        {
            this.game = game;

            CollisionManager = new starLiGHT.Collision.CollisionManager(BroadPhaseType.AxisAlignedRectangleBruteForce,
                                                                        NarrowPhaseType.SeparatingAxisTheorem,
                                                                        true, false, true);

            CollisionManager.Collision += new EventHandler<CollisionEventArgs>(CollisionManager_Collision);

            // load content
            grassTile = game.Content.Load<Texture2D>(@"IsometricTiles/IsoGrassTile01");
            grassTileDebug = game.Content.Load<Texture2D>(@"IsometricTiles/IsoGrassTile02");
            house = game.Content.Load<Texture2D>(@"IsometricTiles/house01_color");

            // create a knight
            knight = new Knight(game, new Vector2(200f, 200f));
            knight.Tag = "Knight";
            CollisionManager.AddCollidable(knight);

            // create a collision hull for the house
            List<Vector2> hullPoints = new List<Vector2>();
            hullPoints.Add(new Vector2(346f, 463f));
            hullPoints.Add(new Vector2(508f, 546f));
            hullPoints.Add(new Vector2(550f, 546f));
            hullPoints.Add(new Vector2(932f, 337f));
            hullPoints.Add(new Vector2(800f, 237f));
            houseHull = new HullObject(hullPoints.ToArray());
            CollisionManager.AddCollidable(houseHull);

            hullPoints.Clear();
            hullPoints.Add(new Vector2(600f, 520f));
            hullPoints.Add(new Vector2(733f, 522f));
            hullPoints.Add(new Vector2(885f, 447f));
            hullPoints.Add(new Vector2(885f, 367f));
            houseHull2 = new HullObject(hullPoints.ToArray());
            CollisionManager.AddCollidable(houseHull2);
        }

        void CollisionManager_Collision(object sender, CollisionEventArgs e)
        {
            if (e.Manifold.NarrowPhaseCollision)
            {
                Knight knightCollidable = null;

                if (e.Pair.object0.Tag != null && e.Pair.object0.Tag.ToString() == "Knight")
                {
                    knightCollidable = e.Pair.object0 as Knight;
                }
                else if (e.Pair.object0.Tag != null && e.Pair.object0.Tag.ToString() == "Knight")
                {
                    knightCollidable = e.Pair.object1 as Knight;
                }

                if (knightCollidable != null)
                {
                    knightCollidable.Position += e.Manifold.Separation.Value;
                }
            }
        }

        public override void Update(GameTime gameTime, KeyboardState actState, KeyboardState lastState)
        {
            knight.Update(gameTime);

            if (actState.IsKeyDown(Keys.G) && lastState.IsKeyUp(Keys.G))
            {
                grassDebug = !grassDebug;
            }

            if (actState.IsKeyDown(Keys.D) && lastState.IsKeyUp(Keys.D))
            {
                debugDraw = !debugDraw;
            }

            if (actState.IsKeyDown(Keys.Up) && lastState.IsKeyUp(Keys.Up))
            {
                knight.Run();
            }

            if (actState.IsKeyUp(Keys.Up) && lastState.IsKeyDown(Keys.Up))
            {
                knight.Stop();
            }

            if (actState.IsKeyDown(Keys.Left) && lastState.IsKeyUp(Keys.Left))
            {
                knight.TurnLeft();
            }

            if (actState.IsKeyDown(Keys.Right) && lastState.IsKeyUp(Keys.Right))
            {
                knight.TurnRight();
            }

            if (knight.Running)
            {
                knight.Position += knight.Direction * (float)gameTime.ElapsedGameTime.TotalSeconds * 60f;
            }

            base.Update(gameTime, actState, lastState);
        }

        public override void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            // draw grass background in own sprite batch because it is behind everything else
            // and we need depth sorting for the house and the knight.
            spriteBatch.Begin();
            Vector2 topLeft = grassTile.CenterOrigin() * -1.0f;
            for (int y = 0; y < 21; y++)
            {
                for (int x = 0; x < 10; x++)
                {
                    spriteBatch.Draw(grassDebug ? grassTileDebug : grassTile, topLeft + new Vector2((x * grassTile.Width) + (y % 2 == 0 ? grassTile.Width * 0.5f : 0f), y * (grassTile.Height * 0.5f)), Color.White);
                }
            }
            spriteBatch.End();

            // draw house and knight with depth parameter, because knight is maybe behind the house
#if XNA40
            spriteBatch.Begin(SpriteSortMode.FrontToBack, BlendState.AlphaBlend);
#else
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.FrontToBack, SaveStateMode.None);
#endif
            spriteBatch.Draw(house, new Vector2(640f, 300f), null, Color.White, 0f, house.CenterOrigin(), 1.0f, SpriteEffects.None, (knight.Position.Y < 520f) ? 1.0f : 0.0f);
            knight.Draw(spriteBatch, (knight.Position.Y < 520f) ? 0.0f : 1.0f);
            spriteBatch.End();

            // draw the debug geometry
            if (debugDraw)
            {
                spriteBatch.Begin();
                CollisionManager.DrawDebug(spriteBatch);
                spriteBatch.End();
            }
        }
    }
}
