﻿#region Using Statements
using System;
using System.Collections;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using starLiGHT.Collision;
using starLiGHT.Collision.BroadPhase;
using starLiGHT.Collision.NarrowPhase;
using starLiGHT.Components.SandboxManager;
#endregion

namespace starLiGHT.Sandbox.Collision.Tests
{
    [SandboxTest("Jump'n'Run")]
    public sealed class JumpAndRunDemo : CollisionSystemTest
    {
        #region Member Variables
        List<HullObject> groundObjects = new List<HullObject>();
        HullObject[] Pyramide;
        HullObject hero;
        Texture2D tileSet;
        float heroSpeed = 5;
        float fallingSpeed = 20;
        float jumpModifier = 2.25f;
        Game1 mainGame;
        bool debugEnabled = true;
        int screenWidth = 0;
        int screenHeight = 0;
        private Vector2 acceleration;
        bool PyramideEnabled = true;

        Vector2 mtd = Vector2.Zero;
        #endregion

        public override void Initialize(Game1 game)
        {
            mainGame = game;
            screenWidth = mainGame.GraphicsDevice.PresentationParameters.BackBufferWidth;
            screenHeight = mainGame.GraphicsDevice.PresentationParameters.BackBufferHeight;

            // setup the CollisionManager
            CollisionManager = new starLiGHT.Collision.CollisionManager(BroadPhaseType.AxisAlignedRectangleBruteForce,
                                                                        NarrowPhaseType.SeparatingAxisTheorem,
                                                                        false, false, true);
            CollisionManager.Collision += new EventHandler<CollisionEventArgs>(CollisionManager_Collision);
            CollisionManager.CollisionResolved += new EventHandler<CollisionResolvedEventArgs>(CollisionManager_CollisionResolved);



            // create a scrollable ground
            tileSet = mainGame.Content.Load<Texture2D>("JumpAndRun/Testtiles");
            groundObjects.AddRange(loadTile(tileSet, 50, 50, 2, screenWidth / 50 + 2));
            
            for (int endlessScreen = 0; endlessScreen < screenWidth + 25; endlessScreen += 50)
            {
                int idx = endlessScreen / 50;
                groundObjects[idx].Position = new Vector2(endlessScreen - 25, screenHeight - 50);
                groundObjects[idx].Tag = "Ground";
                CollisionManager.AddCollidable(groundObjects[idx]);
                
            }

            //create the Pyramide
            Vector2 startpos = new Vector2(50 * 7+25, screenHeight - 100);
            Pyramide = new HullObject[4];
            Pyramide[0] = loadTile(tileSet, 50, 50, 7, 1)[0];
            Pyramide[0].Position = startpos;
            startpos -= new Vector2(50, 0);
            Pyramide[1] = loadTile(tileSet, 50, 50, 2, 1)[0];
            Pyramide[1].Position = startpos;
            startpos -= new Vector2(50, 0);
            Pyramide[2] = loadTile(tileSet, 50, 50, 6, 1)[0];
            Pyramide[2].Position = startpos;
            startpos += new Vector2(50, -50);
            Pyramide[3] = loadTile(tileSet, 50, 50, 2, 1)[0];
            Pyramide[3].Position = startpos;
            for (int PyTileCounter = 0; PyTileCounter < Pyramide.Length; PyTileCounter++)
            {
                Pyramide[PyTileCounter].Tag = "Ground";
                CollisionManager.AddCollidable(Pyramide[PyTileCounter]);
            }


            // create user controlable hero
            Texture2D heroTexture = mainGame.Content.Load<Texture2D>("JumpAndRun/Stickman");
            hero = new HullObject(new RectangleF(20, 500, heroTexture.Width, heroTexture.Height).Corners);
            hero.Texture = heroTexture;
            hero.Tag = "Hero";
            CollisionManager.AddCollidable(hero);
        }

        void CollisionManager_CollisionResolved(object sender, CollisionResolvedEventArgs e)
        {
            e.Pair.object0.Colliding = false;
            e.Pair.object1.Colliding = false;
        }

        void CollisionManager_Collision(object sender, CollisionEventArgs e)
        {
            if (e.Manifold.NarrowPhaseCollision)
            {
                ICollidable2D groundTile = null;
                ICollidable2D collidingHero = null;

                if (e.Pair.object0.Tag.ToString() == "Hero")
                {
                    collidingHero = e.Pair.object0 as HullObject;
                    groundTile = e.Pair.object1 as HullObject;
                }
                else if (e.Pair.object1.Tag.ToString() == "Hero")
                {
                    groundTile = e.Pair.object0 as HullObject;
                    collidingHero = e.Pair.object1 as HullObject;
                }

                if (collidingHero != null && groundTile != null)
                {
                    hero.Transform.Position -= e.Manifold.Separation.Value;
                    //hero.Velocity = Vector2.Zero;
                    hero.Velocity -= e.Manifold.Separation.Value;
                }
            }
        }

        HullObject[] loadTile(Texture2D tileset, int tileWidth, int tileHeight, int tileNumber, int howOften)
        {
            int tileWidthHeight = (tileWidth * tileHeight);
            int tileHorinzontalLines = tileset.Height / tileHeight;
            int tileVerticalLines = tileset.Width / tileWidth;
            Texture2D tileTexture = new Texture2D(mainGame.GraphicsDevice, tileWidth, tileHeight);
            HullObject[] returnTileRange = new HullObject[howOften];

            Color[] tileColorData = new Color[tileWidthHeight];
            tileset.GetData<Color>(0, 
                                   new Rectangle(tileNumber % tileVerticalLines * tileWidth, tileNumber / tileVerticalLines * tileHeight, tileWidth , tileHeight),
                                   tileColorData,
                                   0,
                                   tileWidthHeight);
            tileTexture.SetData<Color>(tileColorData);

            for (int tilecount = 0; tilecount < returnTileRange.Length; tilecount++)
            {
                returnTileRange[tilecount] = new HullObject(new RectangleF(0, 0, tileWidth, tileHeight).Corners);
                returnTileRange[tilecount].Texture = tileTexture;
            }

            return returnTileRange;
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime, KeyboardState actState, KeyboardState lastState)
        {
            // Jumping
            if (actState.IsKeyDown(Keys.Space) && lastState.IsKeyUp(Keys.Space))
            {
                hero.Velocity = new Vector2(0f, -(fallingSpeed * jumpModifier));
            }

            // walking
            if (actState.IsKeyDown(Keys.Left))
            {
                hero.Velocity = new Vector2(-25f, hero.Velocity.Y);
            }
            else if (actState.IsKeyDown(Keys.Right))
            {
                hero.Velocity = new Vector2(25f, hero.Velocity.Y);
            }
            else
            {
                hero.Velocity = new Vector2(0f, hero.Velocity.Y);
            }

            // gravity
            acceleration.Y = fallingSpeed;

            // integrate
            float deltaTime = (float)gameTime.ElapsedGameTime.TotalSeconds;
            float deltaTimeSquaredDividedByTwo = (deltaTime * deltaTime) * 0.5f;
            Vector2 pos = hero.Position;
            pos.X += (hero.Velocity.X * deltaTime) + (acceleration.X * deltaTimeSquaredDividedByTwo);
            pos.Y += (hero.Velocity.Y * deltaTime) + (acceleration.Y * deltaTimeSquaredDividedByTwo);
            hero.Velocity += acceleration * deltaTime;
            hero.Position = pos;

            if (actState.IsKeyDown(Keys.D) && lastState.IsKeyUp(Keys.D))
            {
                debugEnabled = !debugEnabled;
            }

            if (actState.IsKeyDown(Keys.P) && lastState.IsKeyUp(Keys.P))
            {
                if (PyramideEnabled)
                {
                    foreach (HullObject PyTile in Pyramide)
                    {
                        CollisionManager.RemoveCollidable(PyTile);
                    }
                }
                else
                {
                    foreach (HullObject PyTile in Pyramide)
                    {
                        CollisionManager.AddCollidable(PyTile);
                    }
                }
                PyramideEnabled = !PyramideEnabled;

            }

            base.Update(gameTime, actState, lastState);
        }

        public override void Draw(Microsoft.Xna.Framework.GameTime gameTime, SpriteBatch spriteBatch)
        {
            spriteBatch.Begin();

            foreach (HullObject groundObject in groundObjects)
            {
                spriteBatch.Draw(groundObject.Texture, groundObject.Position, null, debugEnabled ? (groundObject.Colliding ? Color.Red : Color.Green) : Color.White, groundObject.Orientation, Vector2.Zero, groundObject.Scale, SpriteEffects.None, 1.0f);
            }

            if (PyramideEnabled)
            {
                foreach (HullObject PyTile in Pyramide)
                {
                    spriteBatch.Draw(PyTile.Texture, PyTile.Position, null, debugEnabled ? (PyTile.Colliding ? Color.Red : Color.Green) : Color.White, PyTile.Orientation, Vector2.Zero, PyTile.Scale, SpriteEffects.None, 1.0f);
                }
            }

            spriteBatch.Draw(hero.Texture, hero.Position, null, debugEnabled ? (hero.Colliding ? Color.Red : Color.Green) : Color.White, hero.Orientation, Vector2.Zero, hero.Scale, SpriteEffects.None, 1.0f);
            
            if (debugEnabled) CollisionManager.DrawDebug(spriteBatch);

            spriteBatch.End();  
        }

    }
}
