﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using starLiGHT.Components.SandboxManager;
using starLiGHT.Collision;
using starLiGHT.Collision.BroadPhase;
using starLiGHT.Collision.NarrowPhase;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

#endregion

namespace starLiGHT.Sandbox.Collision
{
    [SandboxTest("PerPixel Collision Optimization")]
    public sealed class PerPixelTest : CollisionSystemTest
    {
        TextureObject objectA;
        TextureObject objectB;
        Texture2D rectTexture;
        Microsoft.Xna.Framework.Game game;

        public override void Initialize(Game1 game)
        {
            this.game = game;

            CollisionManager.DebugDraw = new DefaultCollisionDebugDraw(true, false, false);
            //CollisionManager.BroadPhase = new AxisAlignedRectangleBruteForce();
            CollisionManager.NarrowPhase = new PerPixelNarrowPhase();

            rectTexture = new Texture2D(game.GraphicsDevice, 64, 64);
            Color[] colors = new Color[rectTexture.Width * rectTexture.Height];
            for (int i = 0; i < colors.Length; i++) colors[i] = Color.White;
            rectTexture.SetData<Color>(colors);

            objectA = new TextureObject(game.GraphicsDevice.Viewport.GetCenter(), rectTexture);
            CollisionManager.AddCollidable(objectA);

            objectB = new TextureObject(new Vector2(128, 128), rectTexture);
            CollisionManager.AddCollidable(objectB);
        }

        public override void Update(GameTime gameTime, KeyboardState actState, KeyboardState lastState)
        {
            objectA.Orientation += 0.01f;

            MouseState ms = Mouse.GetState();
            objectA.Position = ms.GetPosition();

            base.Update(gameTime, actState, lastState);
        }

        public override void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            spriteBatch.Begin();

            spriteBatch.Draw(objectA.Texture, objectA.Position, null, objectA.Colliding ? Color.Red : Color.White, objectA.Orientation, objectA.Texture.CenterOrigin(), 1f, SpriteEffects.None, 1.0f);
            spriteBatch.Draw(objectB.Texture, objectB.Position, null, objectB.Colliding ? Color.Red : Color.Green, objectB.Orientation, objectB.Texture.CenterOrigin(), 1f, SpriteEffects.None, 1.0f);

            CollisionManager.DrawDebug(spriteBatch);

            spriteBatch.End();        
        }
    }
}
