﻿#region Using Statements
using System;
using System.Collections;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using starLiGHT.Collision;
using starLiGHT.Collision.BroadPhase;
using starLiGHT.Collision.NarrowPhase;
using starLiGHT.Components.SandboxManager;
using starLiGHT.Collections;
#endregion

namespace starLiGHT.Sandbox.Collision.Tests
{
    [SandboxTest("Pong")]
    class Pong : CollisionSystemTest
    {
        #region Member Variables
        Game1 game;

        private HullObject player1;

        private HullObject player2;

        private HullObject ball;
        private float ballSpeed = 300f;

        private Vector2 xReflect;
        private Vector2 yReflect;

        private Texture2D middleLine;
        private Vector2 middleLinePos;

        private int screenWidth = 0;
        private int screenHeight = 0;

        private bool debugDraw = true;



        #endregion

        public override void Initialize(Game1 game)
        {
            this.game = game;

            screenWidth = game.GraphicsDevice.PresentationParameters.BackBufferWidth;
            screenHeight = game.GraphicsDevice.PresentationParameters.BackBufferHeight;

            middleLine = game.Content.Load<Texture2D>(@"Pong/Middle");
            middleLinePos = new Vector2((float)screenWidth / 2, (float)screenHeight / 2);

            CollisionManager = new starLiGHT.Collision.CollisionManager(BroadPhaseType.AxisAlignedRectangleBruteForce,
                                                                        NarrowPhaseType.SeparatingAxisTheorem,
                                                                        false, false, true);
            CollisionManager.Collision += new EventHandler<CollisionEventArgs>(CollisionManager_Collision);

            Texture2D paddlepreload = game.Content.Load<Texture2D>("Pong/Paddle");

            player1 = new HullObject(new RectangleF( 20, screenHeight / 2 - paddlepreload.Height, paddlepreload.Width, paddlepreload.Height).Corners);
            player1.Tag = "Paddle";
            player1.Texture = paddlepreload;
            CollisionManager.AddCollidable(player1);


            player2 = new HullObject(new RectangleF(screenWidth - paddlepreload.Width - 20, screenHeight / 2 - paddlepreload.Height, paddlepreload.Width, paddlepreload.Height).Corners);
            player2.Tag = "Paddle";
            player2.Texture = paddlepreload;
            CollisionManager.AddCollidable(player2);

            Texture2D ballpreload = game.Content.Load<Texture2D>("Pong/Ball");

            ball = new HullObject(new RectangleF(screenWidth/2 - ballpreload.Width, screenHeight / 2 - ballpreload.Height, ballpreload.Width, ballpreload.Height).Corners);
            ball.Tag = "Ball";
            ball.Velocity = new Vector2(0.5f, -0.5f);
            ball.Texture = ballpreload;
            CollisionManager.AddCollidable(ball);

            CreateWalls();

        }

        void CollisionManager_Collision(object sender, CollisionEventArgs e)
        {
            if (e.Manifold.NarrowPhaseCollision)
            {
                ICollidable2D a;
                ICollidable2D b;

                if (e.Manifold.Object0.Tag.ToString() == "Ball")
                {
                    a = e.Manifold.Object0 as ICollidable2D;
                    b = e.Manifold.Object1 as ICollidable2D;
                }
                else if (e.Manifold.Object1.Tag.ToString() == "Ball")
                {
                    a = e.Manifold.Object1 as ICollidable2D;
                    b = e.Manifold.Object0 as ICollidable2D;
                }
                else
                {
                    return;
                }

                if (b.Tag.ToString() == "Paddle")
                {
                    ballSpeed += 5;
                }

                if (b.Tag.ToString() == "Paddle" || b.Tag.ToString() == "Wall")
                {
                    Vector2 colNormal = e.Manifold.Separation.Value;
                    colNormal.Normalize();
                    a.Velocity = Vector2.Reflect(a.Velocity, colNormal);
                    
                }
            }
        }

        public override void Update(GameTime gameTime, KeyboardState actState, KeyboardState lastState)
        {

            player1.Position += (actState.IsKeyDown(Keys.W) ? -Vector2.UnitY : (actState.IsKeyDown(Keys.S) ? Vector2.UnitY : Vector2.Zero));
            player2.Position += (actState.IsKeyDown(Keys.Up) ? -Vector2.UnitY : (actState.IsKeyDown(Keys.Down) ? Vector2.UnitY : Vector2.Zero));

            if (ball.Position.X < 0 || ball.Position.X > screenWidth - ball.Texture.Width)
            {
                ballSpeed = 300;
                ball.Position = new Vector2(screenWidth / 2 - ball.Texture.Width, screenHeight / 2 - ball.Texture.Height);
            }

            ball.Position += ball.Velocity * (float)gameTime.ElapsedGameTime.TotalSeconds * ballSpeed;

            if (actState.IsKeyDown(Keys.D) && lastState.IsKeyUp(Keys.D))
            {
                debugDraw = !debugDraw;
            }

            base.Update(gameTime, actState, lastState);
        }

        public override void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            game.GraphicsDevice.Clear(Color.Black);
            spriteBatch.Begin();

            spriteBatch.Draw(player1.Texture, player1.Position, Color.White);
            spriteBatch.Draw(player2.Texture, player2.Position, Color.White);

            spriteBatch.Draw(ball.Texture, ball.Position, Color.White);

            spriteBatch.Draw(middleLine, middleLinePos, null, Color.White, 0f, middleLine.CenterOrigin(), 1f, SpriteEffects.None, 1);

            if (debugDraw)
            {
                CollisionManager.DrawDebug(spriteBatch);
            }
            spriteBatch.End();
        }

        private void CreateWalls()
        {
            HullObject wall = new HullObject(new RectangleF(0f, -20f, (float)screenWidth, 20f).Corners);
            wall.Static = true;
            wall.Tag = "Wall";
            CollisionManager.AddCollidable(wall);

            wall = new HullObject(new RectangleF(0f, (float)screenHeight, (float)screenWidth, 20f).Corners);
            wall.Static = true;
            wall.Tag = "Wall";
            CollisionManager.AddCollidable(wall);
        }
    }
}
