﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using starLiGHT.Components.SandboxManager;
using starLiGHT.Collision;
using starLiGHT.Collision.BroadPhase;
using starLiGHT.Collision.NarrowPhase;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

#endregion

namespace starLiGHT.Sandbox.Collision
{
    [SandboxTest("Rotating Textures")]
    public sealed class RotatingTextures : CollisionSystemTest
    {
        TextureObject objectA;
        TextureObject objectB;
        bool debugDraw = true;

        public override void Initialize(Game1 game)
        {
            CollisionManager = new starLiGHT.Collision.CollisionManager(BroadPhaseType.AxisAlignedRectangleBruteForce,
                                                                        NarrowPhaseType.PerPixel,
                                                                        true, true, false);

            Vector2 center = game.GraphicsDevice.Viewport.GetCenter();

            objectA = new TextureObject(center + new Vector2(-32, 0), game.Content.Load<Texture2D>("rectangle"), true);
            objectB = new TextureObject(center + new Vector2(+32, 0), game.Content.Load<Texture2D>("freehand"), true);

            CollisionManager.AddCollidable(objectA);
            CollisionManager.AddCollidable(objectB);
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime, KeyboardState actState, KeyboardState lastState)
        {
            objectA.Orientation += 0.01f;

            if (actState.IsKeyDown(Keys.D) && actState.IsKeyUp(Keys.D))
            {
                debugDraw = !debugDraw;
            }

            base.Update(gameTime, actState, lastState);
        }

        public override void Draw(Microsoft.Xna.Framework.GameTime gameTime, SpriteBatch spriteBatch)
        {
            spriteBatch.Begin();

            spriteBatch.Draw(objectA.Texture, objectA.Position, null, objectA.Colliding ? Color.Red : Color.Green, objectA.Orientation, objectA.Texture.CenterOrigin(), objectA.Scale, SpriteEffects.None, 1.0f);
            spriteBatch.Draw(objectB.Texture, objectB.Position, null, objectB.Colliding ? Color.Red : Color.Green, objectB.Orientation, objectB.Texture.CenterOrigin(), objectB.Scale, SpriteEffects.None, 1.0f);

            if (debugDraw)
            {
                CollisionManager.DrawDebug(spriteBatch);
            }

            spriteBatch.End();        
        }
    }
}
