﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using starLiGHT.Components.SandboxManager;
using starLiGHT.Collision;
using starLiGHT.Collision.BroadPhase;
using starLiGHT.Collision.NarrowPhase;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

#endregion

namespace starLiGHT.Sandbox.Collision
{
    [SandboxTest("TileEngine")]
    public sealed class TileEngine : CollisionSystemTest
    {
        const int WIDTH = 80;
        const int HEIGHT = 45;

        private Microsoft.Xna.Framework.Game game;
        private HullObject objectA;
        private Random rnd;

        public override void Initialize(Game1 game)
        {
            this.game = game;

            CollisionManager = new starLiGHT.Collision.CollisionManager(BroadPhaseType.QuadTree,
                                                                        NarrowPhaseType.SeparatingAxisTheorem,
                                                                        false, false, true);

            CollisionManager.Collision += new EventHandler<CollisionEventArgs>(CollisionManager_Collision);

            objectA = new HullObject(new Vector2(8f, 8f), new RectangleF(160f, 160f, 16f, 16f).Corners);
            CollisionManager.AddCollidable(objectA);

            rnd = new Random();

            for (int y = 0; y < HEIGHT; y++)
            {
                for (int x = 0; x < WIDTH; x++)
                {
                    if ((x == 0 || x == WIDTH - 1) ||
                        (y == 0 || y == HEIGHT - 1) ||
                        rnd.Next(0, 4) > 2)
                    {
                        HullObject tile = new HullObject(new Vector2(8f, 8f), new RectangleF(x * 16f, y * 16f, 15f, 15f).Corners);
                        tile.Static = true;
                        CollisionManager.AddCollidable(tile);
                    }
                }
            }

            //objectB = new HullObject(new RectangleF(256f, 128f, 64f, 64f).Corners);
            //CollisionManager.AddCollidable(objectB);
        }

        void CollisionManager_Collision(object sender, CollisionEventArgs e)
        {
            if (e.Manifold.NarrowPhaseCollision)
            {
                ICollidable2D a = e.Manifold.Object0 as ICollidable2D;
                Vector2 start = a.Hull.Center + a.Transform.Position.Value - a.Transform.Origin.Value;
                Vector2 end = e.Manifold.Separation.Value;

                ((CollisionManager)sender).DebugDraw.QueueUserLine(start, start + end, Color.Yellow);

                a.Transform.Position += end;
                a.Velocity = Vector2.Zero;
            }
        }

        public override void Update(GameTime gameTime, KeyboardState actState, KeyboardState lastState)
        {
            Vector2 offset = new Vector2(35f * (float)gameTime.ElapsedGameTime.TotalSeconds, 0f);

            if (actState.IsKeyDown(Keys.W))
                objectA.Position -= Vector2.UnitY;

            if (actState.IsKeyDown(Keys.S))
                objectA.Position += Vector2.UnitY;

            if (actState.IsKeyDown(Keys.A))
                objectA.Position -= Vector2.UnitX;

            if (actState.IsKeyDown(Keys.D))
                objectA.Position += Vector2.UnitX;

            base.Update(gameTime, actState, lastState);
        }

        public override void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            spriteBatch.Begin();
            CollisionManager.DrawDebug(spriteBatch);
            spriteBatch.End();        
        }
    }
}
