﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1482                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-07 11:53:48 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class ApplyForce : Test
    {
        const float m_ControllerAccommodation = 0.001f;
        const float m_DefaultTorque = 25.0f;
        const float m_DefaultForce = -50.0f;

        Body m_body;

        public ApplyForce(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            m_world.SetGravity(Vector2.Zero);

            const float k_restitution = 0.4f;

            RigidBody2D ground = PhysicSystem2D.Instance.createBody(0.0f, 20.0f);

            ground.addEdgeShape(new Vector2(-20.0f, -20.0f), new Vector2(-20.0f, 20.0f), 0.0f, k_restitution);
            ground.addEdgeShape(new Vector2(20.0f, -20.0f), new Vector2(20.0f, 20.0f), 0.0f, k_restitution);
            ground.addEdgeShape(new Vector2(-20.0f, 20.0f), new Vector2(20.0f, 20.0f), 0.0f, k_restitution);
            ground.addEdgeShape(new Vector2(-20.0f, -20.0f), new Vector2(20.0f, -20.0f), 0.0f, k_restitution);

		    {
			    Transform xf1 = new Transform();
                xf1.R.Set(0.3524f * (float)starLiGHT.Physic.TwoD.Math.PI);
                xf1.position = starLiGHT.Physic.TwoD.Math.Mul(xf1.R, new Vector2(1.0f, 0.0f));

			    Vector2[] vertices = new Vector2[3];
                vertices[0] = starLiGHT.Physic.TwoD.Math.Mul(xf1, new Vector2(-1.0f, 0.0f));
                vertices[1] = starLiGHT.Physic.TwoD.Math.Mul(xf1, new Vector2(1.0f, 0.0f));
                vertices[2] = starLiGHT.Physic.TwoD.Math.Mul(xf1, new Vector2(0.0f, 0.5f));
    			
			    PolygonShape poly1 = new PolygonShape();
			    poly1.Set(vertices, 3);

			    FixtureDef sd1 = new FixtureDef();
			    sd1.shape = poly1;
			    sd1.density = 4.0f;

			    Transform xf2 = new Transform();
                xf2.R.Set(-0.3524f * (float)starLiGHT.Physic.TwoD.Math.PI);
                xf2.position = starLiGHT.Physic.TwoD.Math.Mul(xf2.R, new Vector2(-1.0f, 0.0f));

                vertices[0] = starLiGHT.Physic.TwoD.Math.Mul(xf2, new Vector2(-1.0f, 0.0f));
                vertices[1] = starLiGHT.Physic.TwoD.Math.Mul(xf2, new Vector2(1.0f, 0.0f));
                vertices[2] = starLiGHT.Physic.TwoD.Math.Mul(xf2, new Vector2(0.0f, 0.5f));
    			
			    PolygonShape poly2 = new PolygonShape();
			    poly2.Set(vertices, 3);

			    FixtureDef sd2 = new FixtureDef();
			    sd2.shape = poly2;
			    sd2.density = 2.0f;

			    BodyDef bd = new BodyDef();
			    bd.type = BodyType.DYNAMIC;
			    bd.angularDamping = 5.0f;
			    bd.linearDamping = 0.1f;

			    bd.position = new Vector2(0.0f, 2.0f);
                bd.angle = (float)starLiGHT.Physic.TwoD.Math.PI;
			    m_body = m_world.CreateBody(bd);
			    m_body.CreateFixture(sd1);
			    m_body.CreateFixture(sd2);
		    }

		    {
			    PolygonShape shape = new PolygonShape();
			    shape.SetAsBox(0.5f, 0.5f);

			    FixtureDef fd = new FixtureDef();
			    fd.shape = shape;
			    fd.density = 1.0f;
			    fd.friction = 0.3f;

			    for (int i = 0; i < 10; ++i)
			    {
				    BodyDef bd = new BodyDef();
				    bd.type = BodyType.DYNAMIC;

                    bd.position = new Vector2(0.0f, 5.0f + 1.54f * i);
				    Body body = m_world.CreateBody(bd);

				    body.CreateFixture(fd);

				    float gravity = 10.0f;
				    float I = body.Inertia;
				    float mass = body.Mass;

				    // For a circle: I = 0.5 * m * r * r ==> r = sqrt(2 * I / m)
				    float radius = (float)System.Math.Sqrt(2.0f * I / mass);

				    FrictionJointDef jd = new FrictionJointDef();
				    jd.localAnchorA = Vector2.Zero;
				    jd.localAnchorB = Vector2.Zero;
				    jd.body1 = ground.Body;
				    jd.body2 = body;
				    jd.collideConnected = true;
				    jd.maxForce = mass * gravity;
				    jd.maxTorque = mass * radius * gravity;

				    m_world.CreateJoint(jd);
			    }
		    }
        }

        public override void Draw(Settings settings, Microsoft.Xna.Framework.Graphics.SpriteBatch spriteBatch, Microsoft.Xna.Framework.Graphics.SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Press w/a/d or DPad to steer.");
            m_textLine += 15;
        }

        public override void Keyboard(KeyboardDevice keyboard, InputManager input)
        {
            base.Keyboard(keyboard, input);

            if(input.IsActionHeld("STEER_FRONT"))
            {
                float force;
                //if(HaveSelectedPlayer && input.LeftThumbStick(m_SelectedPlayerID).Y > m_ControllerAccommodation) 
                //{
                //    force = input.LeftThumbStick(m_SelectedPlayerID).Y * m_DefaultForce;
                //} else 
                //{
                    force = m_DefaultForce;
                //}
                Vector2 f = m_body.GetWorldVector(new Vector2(0.0f, force));
                Vector2 v = new Vector2(0.0f, 2.0f);
                Vector2 p = m_body.GetWorldPoint(ref v);
                m_body.ApplyForce(f, p);
            }

            if(input.IsActionHeld("STEER_LEFT"))
            {
                m_body.ApplyTorque(m_DefaultTorque);
            }
            //else if(HaveSelectedPlayer && input.LeftThumbStick(m_SelectedPlayerID).X > m_ControllerAccommodation) 
            //{
            //    m_body.ApplyTorque(-input.LeftThumbStick(m_SelectedPlayerID).X * m_DefaultTorque);
            //}

            if(input.IsActionHeld("STEER_RIGHT"))
            {
                m_body.ApplyTorque(-m_DefaultTorque);
            } 
            //else if(HaveSelectedPlayer && input.LeftThumbStick(m_SelectedPlayerID).X < -m_ControllerAccommodation)
            //{
            //     m_body.ApplyTorque(input.LeftThumbStick(m_SelectedPlayerID).X*-m_DefaultTorque);
            //}

        }
    }
}
