﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using starLiGHT.Physic;
using Microsoft.Xna.Framework.Graphics;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1409                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-09-09 07:27:44 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class BodyTypes : Test
    {
        RigidBody2D m_attachment;
        RigidBody2D m_platform;
        float m_speed;

        public BodyTypes(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            RigidBody2D ground = PhysicSystem2D.Instance.createEdge(new Vector2(-20.0f, 0.0f), new Vector2(20.0f, 0.0f));

            m_attachment = PhysicSystem2D.Instance.createBox(BodyType.DYNAMIC, 0.0f, 3.0f, 0.5f, 2.0f, 0.2f, 2.0f);

            // Define platform 
            m_platform = PhysicSystem2D.Instance.createBox(BodyType.DYNAMIC, new Vector2(-4.0f, 5.0f), new Vector2(0.5f, 4.0f), new Vector2(4.0f, 0.0f), 0.5f * (float)System.Math.PI, 0.6f, 2.0f);

            RevoluteJoint rj = PhysicSystem2D.Instance.createRevoluteJoint(m_attachment, new Vector2(0.0f, 5.0f), m_platform, new Vector2(0.0f, 5.0f));
            rj.SetMaxMotorTorque(50.0f);
            rj.EnableMotor(true);

            PrismaticJoint pj = PhysicSystem2D.Instance.createPrismaticJoint(ground, m_platform, new Vector2(0.0f, 5.0f), new Vector2(1.0f, 0.0f));
            pj.SetMaxMotorForce(1000.0f);
            pj.EnableMotor(true);
            pj.SetLimits(-10.0f, 10.0f);
            pj.EnableLimit(true);

            m_speed = 3.0f;

            // Create a payload 
            PhysicSystem2D.Instance.createBox(BodyType.DYNAMIC, 0.0f, 8.0f, 0.75f, 0.75f, 0.6f, 2.0f);
        }

        public override void Keyboard(KeyboardDevice keyboard, InputManager input)
        {
            base.Keyboard(keyboard, input);

            if (input.IsActionPress("BODY_CREATE_DYNAMIC")) 
                m_platform.Body.Type = BodyType.DYNAMIC;
            if (input.IsActionPress("BODY_CREATE_STATIC"))
                m_platform.Body.Type = BodyType.STATIC;
            if (input.IsActionPress("BODY_CREATE_KINEMATIC"))
            {
                m_platform.Body.Type = BodyType.KINEMATIC;
                m_platform.Body.LinearVelocity = new Vector2(-m_speed, 0.0f);
                m_platform.Body.AngularVelocity = 0.0f;
            }
        }

        public override void Draw(Settings settings, Microsoft.Xna.Framework.Graphics.SpriteBatch spriteBatch, Microsoft.Xna.Framework.Graphics.SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Keys: (d) dynamic, (s) static, (k) kinematic");
            m_textLine += 15;
        }

        public override void Step(GraphicsDevice graphicsDevice, Settings settings)
        {
            // Drive the kinematic body.
            if (m_platform.Body.Type == BodyType.KINEMATIC)
            {
                Vector2 p = m_platform.Body.Transform.position;
                Vector2 v = m_platform.Body.LinearVelocity;

                if ((p.X < -10.0f && v.X < 0.0f) || (p.X > 10.0f && v.X > 0.0f))
                {
                    v.X = -v.X;
                    m_platform.Body.LinearVelocity = v;
                }
            }

            base.Step(graphicsDevice, settings);
        }
    }
}
