﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using starLiGHT;
using starLiGHT.Physic;
using Microsoft.Xna.Framework.Graphics;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1409                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-09-09 07:27:44 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class Breakable : Test
    {
        public Breakable(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            // Ground body
            RigidBody2D ground = PhysicSystem2D.Instance.createEdge(new Vector2(-40.0f, 0.0f), new Vector2(40.0f, 0.0f));

            //TODO: // Breakable dynamic body
            //body1 = PhysicSystem2D.Instance.createBody(0.0f, 40.0f);
            //body1.addShape(0.0f, 0.0f, 0.5f, 0.5f, new Vector2(-0.5f, 0.0f), 0.0f, 1.0f);
            //body1.addShape(0.0f, 0.0f, 0.5f, 0.5f, new Vector2(0.5f, 0.0f), 0.0f, 1.0f);

            // Breakable dynamic body
            {
                BodyDef bd = new BodyDef();
                bd.type = BodyType.DYNAMIC;
                bd.position = new Vector2(0.0f, 40.0f);
                bd.angle = 0.25f * (float)starLiGHT.Physic.TwoD.Math.PI;
                m_body1 = m_world.CreateBody(bd);

                m_shape1 = new PolygonShape();
                m_shape1.SetAsBox(0.5f, 0.5f, new Vector2(-0.5f, 0.0f), 0.0f);
                m_piece1 = m_body1.CreateFixture(m_shape1, 1.0f);

                m_shape2 = new PolygonShape();
                m_shape2.SetAsBox(0.5f, 0.5f, new Vector2(0.5f, 0.0f), 0.0f);
                m_piece2 = m_body1.CreateFixture(m_shape2, 1.0f);
            }

            m_break = false;
            m_broke = false;
        }

        public override void PostSolve(Contact contact, ContactImpulse impulse)
        {
            if (m_broke)
            {
                // The body already broke.
                return;
            }

            // Should the body break?
            Manifold manifold;
            contact.GetManifold(out manifold);
            Int32 count = manifold.pointCount;

            float maxImpulse = 0.0f;
            for (Int32 i = 0; i < count; ++i)
            {
                maxImpulse = System.Math.Max(maxImpulse, impulse.normalImpulses[i]);
            }

            if (maxImpulse > 40.0f)
            {
                // Flag the body for breaking.
                m_break = true;
            }
        }

        void Break()
        {
            // Create two bodies from one.
            Body body1 = m_piece1.GetBody();
            Vector2 center = body1.WorldCenter;

            body1.DestroyFixture(m_piece2);
            m_piece2 = null;

            BodyDef bd = new BodyDef();
            bd.type = BodyType.DYNAMIC;
            bd.position = body1.Position;
            bd.angle = body1.Angle;

            Body body2 = m_world.CreateBody(bd);
            m_piece2 = body2.CreateFixture(m_shape2, 1.0f);

            // Compute consistent velocities for new bodies based on
            // cached velocity.
            Vector2 center1 = body1.WorldCenter;
            Vector2 center2 = body2.WorldCenter;

            Vector2 velocity1 = m_velocity + Vector2Extensions.Cross(m_angularVelocity, center1 - center);
            Vector2 velocity2 = m_velocity + Vector2Extensions.Cross(m_angularVelocity, center2 - center);

            body1.AngularVelocity = m_angularVelocity;
            body1.LinearVelocity = velocity1;

            body2.AngularVelocity = m_angularVelocity;
            body2.LinearVelocity = velocity2;
        }

        public override void Step(GraphicsDevice graphicsDevice, Settings settings)
        {
            if (m_break)
            {
                Break();
                m_broke = true;
                m_break = false;
            }

            // Cache velocities to improve movement on breakage.
            if (m_broke == false)
            {
                m_velocity = m_body1.LinearVelocity;
                m_angularVelocity = m_body1.AngularVelocity;
            }

            base.Step(graphicsDevice, settings);
        }

        Body m_body1;
        Vector2 m_velocity;
        float m_angularVelocity;
        PolygonShape m_shape1;
        PolygonShape m_shape2;
        Fixture m_piece1;
        Fixture m_piece2;

        bool m_broke;
        bool m_break;
    }
}
