﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 971                   $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-06-21 18:13:19 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class BulletTest : Test
    {
        RigidBody2D stick;
        RigidBody2D bullet;
        float m_x;

        public BulletTest(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            RigidBody2D body = PhysicSystem2D.Instance.createEdge(new Vector2(-10.0f, 0.0f), new Vector2(10.0f, 0.0f));
            body.addBox(0.0f, 0.0f, 0.2f, 1.0f, new Vector2(0.5f, 1.0f), 0.0f, 0.0f);

            stick = PhysicSystem2D.Instance.createBox(BodyType.DYNAMIC, 0.0f, 4.0f, 2.0f, 0.1f, 0.2f, 1.0f);
            bullet = PhysicSystem2D.Instance.createBox(BodyType.DYNAMIC, m_x, 10.0f, 0.25f, 0.25f, 0.2f, 100.0f);
            bullet.Bullet = true;
            bullet.LinearVelocity = new Vector2(0.0f, -50.0f);
        }

        void Launch()
        {
                stick.SetTransform(new Vector2(0.0f, 4.0f), 0.0f);
                stick.LinearVelocity = Vector2.Zero;
                stick.AngularVelocity = 0.0f;

                m_x = RandomFloat(-1.0f, 1.0f);
                bullet.SetTransform(new Vector2(m_x, 10.0f), 0.0f);
                bullet.LinearVelocity = new Vector2(0.0f, -50.0f);
                bullet.AngularVelocity = 0.0f;

                starLiGHT.Physic.TwoD.Collision.gjkCalls = 0;
                starLiGHT.Physic.TwoD.Collision.gjkIters = 0;
                starLiGHT.Physic.TwoD.Collision.gjkMaxIters = 0;

                starLiGHT.Physic.TwoD.Collision.ToiCalls = 0;
                starLiGHT.Physic.TwoD.Collision.ToiIters = 0;
                starLiGHT.Physic.TwoD.Collision.MaxToiIters = 0;
                starLiGHT.Physic.TwoD.Collision.ToiRootIters = 0;
                starLiGHT.Physic.TwoD.Collision.MaxToiRootIters = 0;
        }

        public override void Step(GraphicsDevice graphicsDevice, Settings settings)
        {
            if (m_stepCount == 12)
            {
                m_stepCount += 0;
            }

            base.Step(graphicsDevice, settings);
        }

        public override void Draw(Settings settings, Microsoft.Xna.Framework.Graphics.SpriteBatch spriteBatch, Microsoft.Xna.Framework.Graphics.SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            if (starLiGHT.Physic.TwoD.Collision.gjkCalls > 0)
            {
                m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, String.Format("gjk calls = {0}, ave gjk iters = {1}, max gjk iters = {2}",
                                                                                       starLiGHT.Physic.TwoD.Collision.gjkCalls, starLiGHT.Physic.TwoD.Collision.gjkIters / starLiGHT.Physic.TwoD.Collision.gjkCalls, starLiGHT.Physic.TwoD.Collision.gjkMaxIters));
                m_textLine += 15;
            }

            if (starLiGHT.Physic.TwoD.Collision.ToiCalls > 0)
            {
                m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine,
                                       String.Format("toi calls = {0}, ave toi iters = {1}, max toi iters = {2}",
                                                     starLiGHT.Physic.TwoD.Collision.ToiCalls,
                                                     starLiGHT.Physic.TwoD.Collision.ToiIters / (float)starLiGHT.Physic.TwoD.Collision.ToiCalls,
                                                     starLiGHT.Physic.TwoD.Collision.MaxToiRootIters));
                m_textLine += 15;

                m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine,
                                       String.Format("ave toi root iters = {0}, max toi root iters = {1}",
                                                     starLiGHT.Physic.TwoD.Collision.ToiRootIters / (float)starLiGHT.Physic.TwoD.Collision.ToiCalls,
                                                     starLiGHT.Physic.TwoD.Collision.MaxToiRootIters));
                m_textLine += 15;

            }

            if (m_stepCount % 60 == 0)
            {
                Launch(); 
            }
        }
    }
}
