﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using starLiGHT;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 7607                  $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2011-03-25 03:47:10 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class Car : Test
    {
        public Car(Microsoft.Xna.Framework.Game game)
            : base(game, 2.0f)
        {
		    m_hz = 4.0f;
 	        m_zeta = 0.7f;
            m_speed = 20.0f;

            Body ground = null;
            {
                BodyDef bd = new BodyDef();
                ground = m_world.CreateBody(bd);

                EdgeShape shape = new EdgeShape();

                FixtureDef fd = new FixtureDef();
                fd.shape = shape;
                fd.density = 0.0f;
                fd.friction = 0.6f;

                shape.Set(new Vector2(-20.0f, 0.0f), new Vector2(20.0f, 0.0f));
                ground.CreateFixture(fd);

                float[] hs = new float[] {0.25f, 1.0f, 4.0f, 0.0f, 0.0f, -1.0f, -2.0f, -2.0f, -1.25f, 0.0f};

                float x = 20.0f, y1 = 0.0f, dx = 5.0f;

                for (int i = 0; i < 10; ++i)
                {
                    float y2 = hs[i];
                    shape.Set(new Vector2(x, y1), new Vector2(x + dx, y2));
                    ground.CreateFixture(fd);
                    y1 = y2;
                    x += dx;
                }

                for (int i = 0; i < 10; ++i)
                {
                    float y2 = hs[i];
                    shape.Set(new Vector2(x, y1), new Vector2(x + dx, y2));
                    ground.CreateFixture(fd);
                    y1 = y2;
                    x += dx;
                }

                shape.Set(new Vector2(x, 0.0f), new Vector2(x + 40.0f, 0.0f));
                ground.CreateFixture(fd);

                x += 80.0f;
                shape.Set(new Vector2(x, 0.0f), new Vector2(x + 40.0f, 0.0f));
                ground.CreateFixture(fd);

                x += 40.0f;
                shape.Set(new Vector2(x, 0.0f), new Vector2(x + 10.0f, 5.0f));
                ground.CreateFixture(fd);

                x += 20.0f;
                shape.Set(new Vector2(x, 0.0f), new Vector2(x + 40.0f, 0.0f));
                ground.CreateFixture(fd);

                x += 40.0f;
                shape.Set(new Vector2(x, 0.0f), new Vector2(x, 20.0f));
                ground.CreateFixture(fd);
            }

            // Teeter
            {
                BodyDef bd = new BodyDef();
                bd.position = new Vector2(140.0f, 1.0f);
                bd.type = BodyType.DYNAMIC;
                Body body = m_world.CreateBody(bd);

                PolygonShape box = new PolygonShape();
                box.SetAsBox(10.0f, 0.25f);
                body.CreateFixture(box, 1.0f);

                RevoluteJointDef jd = new RevoluteJointDef();
                jd.Initialize(ground, body, body.Position);
                jd.lowerAngle = -8.0f * (float)System.Math.PI / 180.0f;
                jd.upperAngle = 8.0f * (float)System.Math.PI / 180.0f;
                jd.enableLimit = true;
                m_world.CreateJoint(jd);

                body.ApplyAngularImpulse(100.0f);
            }

            // Bridge
            {
                int N = 20;
                PolygonShape shape = new PolygonShape();
                shape.SetAsBox(1.0f, 0.125f);

                FixtureDef fd = new FixtureDef();
                fd.shape = shape;
                fd.density = 1.0f;
                fd.friction = 0.6f;

                RevoluteJointDef jd = new RevoluteJointDef();

                Body prevBody = ground;
                for (int i = 0; i < N; ++i)
                {
                    BodyDef bd = new BodyDef();
                    bd.type = BodyType.DYNAMIC;
                    bd.position = new Vector2(161.0f + 2.0f * i, -0.125f);
                    Body body = m_world.CreateBody(bd);
                    body.CreateFixture(fd);

                    Vector2 anchor = new Vector2(160.0f + 2.0f * i, -0.125f);
                    jd.Initialize(prevBody, body, anchor);
                    m_world.CreateJoint(jd);

                    prevBody = body;
                }

                Vector2 _anchor = new Vector2(160.0f + 2.0f * N, -0.125f);
                jd.Initialize(prevBody, ground, _anchor);
                m_world.CreateJoint(jd);
            }

            // Boxes
            {
                PolygonShape box = new PolygonShape();
                box.SetAsBox(0.5f, 0.5f);

                Body body = null;
                BodyDef bd = new BodyDef();
                bd.type = BodyType.DYNAMIC;

                bd.position = new Vector2(230.0f, 0.5f);
                body = m_world.CreateBody(bd);
                body.CreateFixture(box, 0.5f);

                bd.position = new Vector2(230.0f, 1.5f);
                body = m_world.CreateBody(bd);
                body.CreateFixture(box, 0.5f);

                bd.position = new Vector2(230.0f, 2.5f);
                body = m_world.CreateBody(bd);
                body.CreateFixture(box, 0.5f);

                bd.position = new Vector2(230.0f, 3.5f);
                body = m_world.CreateBody(bd);
                body.CreateFixture(box, 0.5f);

                bd.position = new Vector2(230.0f, 4.5f);
                body = m_world.CreateBody(bd);
                body.CreateFixture(box, 0.5f);
            }

            // Car
            {
                PolygonShape chassis = new PolygonShape();
                Vector2[] vertices = new Vector2[8];
                vertices[0] = new Vector2(-1.5f, -0.5f);
                vertices[1] = new Vector2(1.5f, -0.5f);
                vertices[2] = new Vector2(1.5f, 0.0f);
                vertices[3] = new Vector2(0.0f, 0.9f);
                vertices[4] = new Vector2(-1.15f, 0.9f);
                vertices[5] = new Vector2(-1.5f, 0.2f);
                chassis.Set(vertices, 6);

                CircleShape circle = new CircleShape();
                circle.m_radius = 0.4f;

                BodyDef bd = new BodyDef();
                bd.type = BodyType.DYNAMIC;
                bd.position = new Vector2(0.0f, 1.0f);
                m_car = m_world.CreateBody(bd);
                m_car.CreateFixture(chassis, 1.0f);

                FixtureDef fd = new FixtureDef();
                fd.shape = circle;
                fd.density = 1.0f;
                fd.friction = 0.9f;

                bd.position = new Vector2(-1.0f, 0.35f);
                m_wheel1 = m_world.CreateBody(bd);
                m_wheel1.CreateFixture(fd);

                bd.position = new Vector2(1.0f, 0.4f);
                m_wheel2 = m_world.CreateBody(bd);
                m_wheel2.CreateFixture(fd);

                LineJointDef jd = new LineJointDef(); 
                Vector2 axis = new Vector2(0.0f, 1.0f);

                jd.Initialize(m_car, m_wheel1, m_wheel1.Position, axis);
                jd.motorSpeed = 0.0f;
                jd.maxMotorTorque = 20.0f;
                jd.enableMotor = true;
                jd.frequencyHz = m_hz;
                jd.dampingRatio = m_zeta;
                m_spring1 = (starLiGHT.Physic.TwoD.LineJoint)m_world.CreateJoint(jd);

                jd.Initialize(m_car, m_wheel2, m_wheel2.Position, axis);
                jd.motorSpeed = 0.0f;
                jd.maxMotorTorque = 10.0f;
                jd.enableMotor = false;
                jd.frequencyHz = m_hz;
                jd.dampingRatio = m_zeta;
                m_spring2 = (starLiGHT.Physic.TwoD.LineJoint)m_world.CreateJoint(jd);
            }
        }

        public override void Keyboard(KeyboardDevice keyboard, InputManager input)
        {
            if (keyboard.GetKey(Keys.A).Pressed)
            {
                m_spring1.SetMotorSpeed(m_speed);
            }

            if (keyboard.GetKey(Keys.S).Pressed)
            {
                m_spring1.SetMotorSpeed(0.0f);
            }

            if (keyboard.GetKey(Keys.D).Pressed)
            {
                m_spring1.SetMotorSpeed(-m_speed);
            }

            if (keyboard.GetKey(Keys.Q).Pressed)
            {
                m_hz = System.Math.Max(0.0f, m_hz - 1.0f);
                m_spring1.SpringFrequencyHz = m_hz;
                m_spring2.SpringFrequencyHz = m_hz;
            }

            if (keyboard.GetKey(Keys.E).Pressed)
            {
                m_hz += 1.0f;
                m_spring1.SpringFrequencyHz = m_hz;
                m_spring2.SpringFrequencyHz = m_hz;
            }

            base.Keyboard(keyboard, input);
        }

        public override void Draw(Settings settings, Microsoft.Xna.Framework.Graphics.SpriteBatch spriteBatch, Microsoft.Xna.Framework.Graphics.SpriteFont font)
        {
            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Keys: left = a, brake = s, right = d, hz_down = q, hz_up = e");
            m_textLine += 15;
            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, String.Format("frequency = {0} hz, damping ratio = {1}", m_hz, m_zeta));
            m_textLine += 15;

            base.camera.Center = new Vector2((m_car.Position.X + 1.0f) + 200f, base.camera.Center.Y);

            base.Draw(settings, spriteBatch, font);
        }

	    Body m_car;
 	    Body m_wheel1;
 	    Body m_wheel2;

        float m_hz;
 	    float m_zeta;
        float m_speed;
        starLiGHT.Physic.TwoD.LineJoint m_spring1;
        starLiGHT.Physic.TwoD.LineJoint m_spring2;
    }
}
