﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using starLiGHT;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1482                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-07 11:53:48 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class CharacterCollision : Test
    {
        public CharacterCollision(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            // Ground body
            RigidBody2D ground = PhysicSystem2D.Instance.createEdge(new Vector2(-20.0f, 0.0f), new Vector2(20.0f, 0.0f)); 

            // Collinear edges with no adjacency information.
            // This shows the problematic case where a box shape can hit
            // an internal vertex.
            {
                RigidBody2D body = PhysicSystem2D.Instance.createBody(Vector2.Zero);
                body.addEdgeShape(new Vector2(-8.0f, 1.0f), new Vector2(-6.0f, 1.0f));
                body.addEdgeShape(new Vector2(-6.0f, 1.0f), new Vector2(-4.0f, 1.0f));
                body.addEdgeShape(new Vector2(-4.0f, 1.0f), new Vector2(-2.0f, 1.0f));
            }

            // Square tiles. This shows that adjacency shapes may
            // have non-smooth collision. There is no solution
            // to this problem.
            {
                RigidBody2D body = PhysicSystem2D.Instance.createBody(Vector2.Zero);
                body.addBox(0.0f, 0.0f, 1.0f, 1.0f, new Vector2(4.0f, 3.0f), 0.0f, 1.0f);
                body.addBox(0.0f, 0.0f, 1.0f, 1.0f, new Vector2(6.0f, 3.0f), 0.0f, 1.0f);
                body.addBox(0.0f, 0.0f, 1.0f, 1.0f, new Vector2(8.0f, 3.0f), 0.0f, 1.0f);
            }

            // Square made from an edge loop. Collision should be smooth.
            {
                BodyDef bd = new BodyDef();
                Body body = m_world.CreateBody(bd);

                Vector2[] vs = { new Vector2(-1.0f, 3.0f),
                                 new Vector2(1.0f, 3.0f),
                                 new Vector2(1.0f, 5.0f),
                                 new Vector2(-1.0f, 5.0f) };
                LoopShape shape = new LoopShape();
                shape.Create(vs, 4);
                body.CreateFixture(shape, 0.0f);
            }

            // Edge loop. Collision should be smooth.
            {
                BodyDef bd = new BodyDef();
                bd.position = new Vector2(-10.0f, 4.0f);

                Body body = m_world.CreateBody(bd);

                Vector2[] vs = new Vector2[10];
                vs[0] = new Vector2(0.0f, 0.0f);

                vs[0] = new Vector2(6.0f, 0.0f);
                vs[0] = new Vector2(6.0f, 2.0f);
                vs[0] = new Vector2(4.0f, 1.0f);
                vs[0] = new Vector2(2.0f, 2.0f);
                vs[0] = new Vector2(0.0f, 2.0f);
                vs[0] = new Vector2(-2.0f, 2.0f);
                vs[0] = new Vector2(-4.0f, 3.0f);
                vs[0] = new Vector2(-6.0f, 2.0f);
                vs[0] = new Vector2(-6.0f, 0.0f);
                LoopShape shape = new LoopShape();
                shape.Create(vs, 10);
                body.CreateFixture(shape, 0.0f);
            }

            // Square character 1
            {
                BodyDef bd = new BodyDef();
                bd.position = new Vector2(-3.0f, 8.0f);
                bd.type = BodyType.DYNAMIC;
                bd.fixedRotation = true;
                bd.allowSleep = false;

                Body body = m_world.CreateBody(bd);

                PolygonShape shape = new PolygonShape();
                shape.SetAsBox(0.5f, 0.5f);

                FixtureDef fd = new FixtureDef();
                fd.shape = shape;
                fd.density = 20.0f;
                body.CreateFixture(fd);
            }

            // Square character 2
            {
                BodyDef bd = new BodyDef();
                bd.position = new Vector2(-5.0f, 5.0f);
                bd.type = BodyType.DYNAMIC;
                bd.fixedRotation = true;
                bd.allowSleep = false;

                Body body = m_world.CreateBody(bd);

                PolygonShape shape = new PolygonShape();
                shape.SetAsBox(0.25f, 0.25f);

                FixtureDef fd = new FixtureDef();
                fd.shape = shape;
                fd.density = 20.0f;
                body.CreateFixture(fd);
            }

            // Hexagon character
            {
                BodyDef bd = new BodyDef();
                bd.position = new Vector2(-5.0f, 8.0f);
                bd.type = BodyType.DYNAMIC;
                bd.fixedRotation = true;
                bd.allowSleep = false;

                Body body = m_world.CreateBody(bd);

                float angle = 0.0f;
                float delta = (float)System.Math.PI / 3.0f;
                Vector2[] vertices = new Vector2[6];
                for (int i = 0; i < 6; ++i)
                {
                        vertices[i] = new Vector2(0.5f *  (float)System.Math.Cos(angle), 0.5f * (float)System.Math.Sin(angle));
                        angle += delta;
                }

                PolygonShape shape = new PolygonShape();
                shape.Set(vertices, 6);

                FixtureDef fd = new FixtureDef();
                fd.shape = shape;
                fd.density = 20.0f;
                body.CreateFixture(fd);
            }

            // Circle character
            {
                BodyDef bd = new BodyDef();
                bd.position = new Vector2(3.0f, 5.0f);
                bd.type = BodyType.DYNAMIC;
                bd.fixedRotation = true;
                bd.allowSleep = false;

                Body body = m_world.CreateBody(bd);

                CircleShape shape = new CircleShape();
                shape.m_radius = 0.5f;

                FixtureDef fd = new FixtureDef();
                fd.shape = shape;
                fd.density = 20.0f;
                body.CreateFixture(fd);
            }
        }

        public override void Draw(Settings settings, Microsoft.Xna.Framework.Graphics.SpriteBatch spriteBatch, Microsoft.Xna.Framework.Graphics.SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "This tests various character collision shapes.");
            m_textLine += 15;
            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Limitation: square and hexagon can snag on aligned boxes.");
            m_textLine += 15;
            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Feature: loops have smooth collision inside and out.");
            m_textLine += 15;
        }
    }
}
