﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using starLiGHT;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1297                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-08-17 13:56:34 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class CollisionFiltering : Test
    {
        // This is a test of collision filtering. 
        // There is a triangle, a box, and a circle. 
        // There are 6 shapes. 3 large and 3 small. 
        // The 3 small ones always collide. 
        // The 3 large ones never collide. 
        // The boxes don't collide with triangles (except if both are small). 
        const short k_smallGroup = 1;
        const short k_largeGroup = -1;

        const ushort k_defaultCategory = 0x0001;
        const ushort k_triangleCategory = 0x0002;
        const ushort k_boxCategory = 0x0004;
        const ushort k_circleCategory = 0x0008;

        const ushort k_triangleMask = 0xFFFF;
        const ushort k_boxMask = 0xFFFF ^ k_triangleCategory;
        const ushort k_circleMask = 0xFFFF; 
 
        public CollisionFiltering(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            PhysicSystem2D.Instance.createEdge(new Vector2(-40.0f, 0.0f), new Vector2(40.0f, 0.0f));
  
            // Small triangle 
            Vector2[] vertices = new Vector2[3];
            vertices[0]= new Vector2(-1.0f, 0.0f); 
            vertices[1]= new Vector2(1.0f, 0.0f); 
            vertices[2]= new Vector2(0.0f, 2.0f);

            RigidBody2D body1 = PhysicSystem2D.Instance.createBody(BodyType.DYNAMIC, -5.0f, 2.0f);
            Fixture triangleShape = body1.addPolygon(vertices);
            triangleShape.SetFilterData(new Filter(k_smallGroup, k_triangleCategory, k_triangleMask));

            // Large triangle (recycle definitions) 
            vertices[0] *= 2.0f; 
            vertices[1] *= 2.0f; 
            vertices[2] *= 2.0f;

            RigidBody2D body2 = PhysicSystem2D.Instance.createBody(BodyType.DYNAMIC, -5.0f, 6.0f);
            Fixture triangleShape2 = body1.addPolygon(vertices);
            triangleShape2.SetFilterData(new Filter(k_largeGroup, k_triangleCategory, k_triangleMask));

            RigidBody2D body = PhysicSystem2D.Instance.createBox(-5.0f, 10.0f, 0.5f, 1.0f);
            PrismaticJoint jd = PhysicSystem2D.Instance.createPrismaticJoint(body2, body, new Vector2(0.0f, 4.0f), Vector2.Zero, new Vector2(0.0f, 1.0f));
            jd.EnableLimit(true);
            jd.SetLimits(-1.0f, 1.0f);

            // Small box
            RigidBody2D body3 = PhysicSystem2D.Instance.createBody(BodyType.DYNAMIC, 0.0f, 2.0f);
            Fixture boxShape = body3.addBox(0.0f, 0.0f, 1.0f, 0.5f);
            boxShape.SetRestitution(0.1f);
            boxShape.SetFilterData(new Filter(k_smallGroup, k_boxCategory, k_boxMask));
  
            // Large box (recycle definitions) 
            RigidBody2D body4 = PhysicSystem2D.Instance.createBody(BodyType.DYNAMIC, 0.0f, 6.0f);
            Fixture boxShape4 = body4.addBox(0.0f, 0.0f, 2.0f, 1.0f);
            boxShape4.SetRestitution(0.1f);
            boxShape4.SetFilterData(new Filter(k_largeGroup, k_boxCategory, k_boxMask));

  
            // Small circle 
            RigidBody2D body5 = PhysicSystem2D.Instance.createBody(BodyType.DYNAMIC, 5.0f, 2.0f);
            Fixture circle = body5.addCircle(0.0f, 0.0f, 1.0f);
            circle.SetFilterData(new Filter(k_smallGroup, k_circleCategory, k_circleMask));

            // Large circle 
            RigidBody2D body6 = PhysicSystem2D.Instance.createBody(BodyType.DYNAMIC, 5.0f, 6.0f);
            Fixture circle2 = body6.addCircle(0.0f, 0.0f, 2.0f);
            circle2.SetFilterData(new Filter(k_largeGroup, k_circleCategory, k_circleMask));
         }
    }
}
