﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using starLiGHT;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1482                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-07 11:53:48 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class CollisionProcessing : Test
    {
        public CollisionProcessing(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            // Ground body 
            RigidBody2D ground = PhysicSystem2D.Instance.createEdge(new Vector2(-50.0f, 0.0f), new Vector2(50.0f, 0.0f)); 
  
            float xLo = -5.0f, xHi = 5.0f; 
            float yLo = 2.0f, yHi = 35.0f; 

            // Small triangle 
            Vector2[] vertices = new Vector2[3];
            vertices[0] = new Vector2(-1.0f, 0.0f);
            vertices[1] = new Vector2(1.0f, 0.0f);
            vertices[2] = new Vector2(0.0f, 2.0f); 

            PolygonShape polygon = new PolygonShape(); 
            polygon.Set(vertices, 3); 

            FixtureDef triangleShapeDef = new FixtureDef(); 
            triangleShapeDef.shape = polygon; 
            triangleShapeDef.density = 1.0f; 

            BodyDef triangleBodyDef = new BodyDef(); 
            triangleBodyDef.type = BodyType.DYNAMIC;
            triangleBodyDef.position = new Vector2(RandomFloat(xLo, xHi), RandomFloat(yLo, yHi)); 

            Body body1 = m_world.CreateBody(triangleBodyDef); 
            body1.CreateFixture(triangleShapeDef); 

            // Large triangle (recycle definitions) 
            vertices[0] *= 2.0f; 
            vertices[1] *= 2.0f; 
            vertices[2] *= 2.0f; 
            polygon.Set(vertices, 3);

            triangleBodyDef.position = new Vector2(RandomFloat(xLo, xHi), RandomFloat(yLo, yHi)); 

            Body body2 = m_world.CreateBody(triangleBodyDef); 
            body2.CreateFixture(triangleShapeDef); 

            // Small box 
            polygon.SetAsBox(1.0f, 0.5f); 

            FixtureDef boxShapeDef = new FixtureDef(); 
            boxShapeDef.shape = polygon; 
            boxShapeDef.density = 1.0f; 

            BodyDef boxBodyDef = new BodyDef(); 
            boxBodyDef.type = BodyType.DYNAMIC;
            boxBodyDef.position = new Vector2(RandomFloat(xLo, xHi), RandomFloat(yLo, yHi)); 

            Body body3 = m_world.CreateBody(boxBodyDef); 
            body3.CreateFixture(boxShapeDef); 

            // Large box (recycle definitions) 
            polygon.SetAsBox(2.0f, 1.0f);
            boxBodyDef.position = new Vector2(RandomFloat(xLo, xHi), RandomFloat(yLo, yHi)); 

            Body body4 = m_world.CreateBody(boxBodyDef); 
            body4.CreateFixture(boxShapeDef); 

            // Small circle 
            CircleShape circle = new CircleShape(); 
            circle.m_radius = 1.0f; 

            FixtureDef circleShapeDef = new FixtureDef(); 
            circleShapeDef.shape = circle; 
            circleShapeDef.density = 1.0f; 

            BodyDef circleBodyDef = new BodyDef(); 
            circleBodyDef.type = BodyType.DYNAMIC;
            circleBodyDef.position = new Vector2(RandomFloat(xLo, xHi), RandomFloat(yLo, yHi)); 

            Body body5 = m_world.CreateBody(circleBodyDef); 
            body5.CreateFixture(circleShapeDef); 

            // Large circle 
            circle.m_radius *= 2.0f;
            circleBodyDef.position = new Vector2(RandomFloat(xLo, xHi), RandomFloat(yLo, yHi)); 

            Body body6 = m_world.CreateBody(circleBodyDef); 
            body6.CreateFixture(circleShapeDef); 
        }

        public override void Step(GraphicsDevice graphicsDevice, Settings settings)
        {
            base.Step(graphicsDevice, settings);

                 // We are going to destroy some bodies according to contact 
                 // points. We must buffer the bodies that should be destroyed 
                 // because they may belong to multiple contact points. 
                 const int k_maxNuke = 6; 
                 Body[] nuke = new Body[k_maxNuke]; 
                 int nukeCount = 0; 
  
                 // Traverse the contact results. Destroy bodies that 
                 // are touching heavier bodies. 
                 for (int i = 0; i < m_pointCount; ++i) 
                 { 
                         ContactPoint point = m_points[i]; 
  
                         Body body1 = point.fixtureA.GetBody(); 
                         Body body2 = point.fixtureB.GetBody(); 
                         float mass1 = body1.Mass; 
                         float mass2 = body2.Mass; 
  
                         if (mass1 > 0.0f && mass2 > 0.0f) 
                         { 
                                 if (mass2 > mass1) 
                                 { 
                                         nuke[nukeCount++] = body1; 
                                 } 
                                 else 
                                 { 
                                         nuke[nukeCount++] = body2; 
                                 } 
  
                                 if (nukeCount == k_maxNuke) 
                                 { 
                                         break; 
                                 } 
                         } 
                 } 
  
                 // Sort the nuke array to group duplicates. 
                 //TODO: Array.Sort(nuke, 0, nukeCount); 
  
                 // Destroy the bodies, skipping duplicates. 
                 int j = 0; 
                 while (j < nukeCount) 
                 { 
                     Body b = nuke[j++]; 
                     while (j < nukeCount && nuke[j] == b) 
                     { 
                         ++j; 
                     } 
                        
                     if (b != m_bomb)
                         m_world.DestroyBody(b);
                 }
         }
    }
}
