﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using starLiGHT;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1482                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-07 11:53:48 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class Confined : Test
    {
        static int e_columnCount = 0;
        static int e_rowCount = 0;

        public Confined(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            {
                RigidBody2D ground = PhysicSystem2D.Instance.createBody(Vector2.Zero);
                // Floor
                ground.addEdgeShape(new Vector2(-10.0f, 0.0f), new Vector2(10.0f, 0.0f));

                // Left wall
                ground.addEdgeShape(new Vector2(-10.0f, 0.0f), new Vector2(-10.0f, 20.0f));

                // Right wall
                ground.addEdgeShape(new Vector2(10.0f, 0.0f), new Vector2(10.0f, 20.0f));

                // Roof
                ground.addEdgeShape(new Vector2(-10.0f, 20.0f), new Vector2(10.0f, 20.0f));
            }

            {
                float radius = 0.5f;
                CircleShape shape = new CircleShape();
                shape.m_p = Vector2.Zero;
                shape.m_radius = radius;

                FixtureDef fd = new FixtureDef();
                fd.shape = shape;
                fd.density = 1.0f;
                fd.friction = 0.1f;

                for (Int32 j = 0; j < e_columnCount; ++j)
                {
                    for (Int32 i = 0; i < e_rowCount; ++i)
                    {
                        BodyDef bd = new BodyDef();
                        bd.type = BodyType.DYNAMIC;
                        bd.position = new Vector2(-10.0f + (2.1f * j + 1.0f + 0.01f * i) * radius, (2.0f * i + 1.0f) * radius);
                        Body body = m_world.CreateBody(bd);

                        body.CreateFixture(fd);
                    }
                }

                m_world.SetGravity(Vector2.Zero);
            }
        }

        void CreateCircle()
        {
            float radius = 2.0f;
            CircleShape shape = new CircleShape();
            shape.m_p = Vector2.Zero;
            shape.m_radius = radius;

            FixtureDef fd = new FixtureDef();
            fd.shape = shape;
            fd.density = 1.0f;
            fd.friction = 0.0f;

            Vector2 p = new Vector2(RandomFloat(), 3.0f + RandomFloat());
            BodyDef bd = new BodyDef();
            bd.type = BodyType.DYNAMIC;
            bd.position = p;
            //bd.allowSleep = false;
            Body body = m_world.CreateBody(bd);

            body.CreateFixture(fd);
        }

        public override void Keyboard(KeyboardDevice keyboard, starLiGHT.Input.InputManager input)
        {
            base.Keyboard(keyboard, input);

            if (input.IsActionPress("BODY_CREATE_CIRCLE"))
            {
                CreateCircle();
            }
        }

        public override void Draw(Settings settings, Microsoft.Xna.Framework.Graphics.SpriteBatch spriteBatch, Microsoft.Xna.Framework.Graphics.SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Press 'c' to create a circle.");
            m_textLine += 15;
        }

        public override void Step(GraphicsDevice graphicsDevice, Settings settings)
        {
            Boolean sleeping = true;
            for (Body b = m_world.GetBodyList(); b != null; b = b.Next)
            {
                if (b.Type != BodyType.DYNAMIC)
                {
                    continue;
                }

                if (b.Awake)
                {
                    sleeping = false;
                }
            }

            if (m_stepCount == 180)
            {
                m_stepCount += 0;
            }

            //if (sleeping)
            //{
            //CreateCircle(;
            //}

            base.Step(graphicsDevice, settings);

            for (Body b = m_world.GetBodyList(); b != null; b = b.Next)
            {
                if (b.Type != BodyType.DYNAMIC)
                {
                    continue;
                }

                Vector2 p = b.Position;
                if (p.X <= -10.0f || 10.0f <= p.X || p.Y <= 0.0f || 20.0f <= p.Y)
                {
                    p.X += 0.0f;
                }
            }
        }
    }
}
