﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using starLiGHT;
using Microsoft.Xna.Framework.Graphics;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1482                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-07 11:53:48 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class ContinuousTest : Test
    {
        Body m_body;
        float m_angularVelocity;

        public ContinuousTest(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            {
                BodyDef bd = new BodyDef(); ; 
                bd.position = new Vector2(0.0f, 0.0f); 
                Body body = m_world.CreateBody(bd); 

                EdgeShape edge = new EdgeShape(); 

                edge.Set(new Vector2(-10.0f, 0.0f), new Vector2(10.0f, 0.0f));
                body.CreateFixture(edge, 0.0f);

                PolygonShape shape = new PolygonShape();
                shape.SetAsBox(0.2f, 1.0f, new Vector2(0.5f, 1.0f), 0.0f);
                body.CreateFixture(shape, 0.0f); 
            } 
  
 #if true 
            { 
                BodyDef bd = new BodyDef(); 
                bd.type = BodyType.DYNAMIC; 
                bd.position = new Vector2(0.0f, 20.0f); 
                //bd.angle = 0.1f; 

                PolygonShape shape = new PolygonShape(); 
                shape.SetAsBox(2.0f, 0.1f); 

                m_body = m_world.CreateBody(bd); 
                m_body.CreateFixture(shape, 1.0f); 

                m_angularVelocity = RandomFloat(-50.0f, 50.0f);
                //m_angularVelocity = 46.661274f; 
                m_body.LinearVelocity = new Vector2(0.0f, -100.0f);
                m_body.AngularVelocity = m_angularVelocity; 
            } 
 #else 
                 { 
                         BodyDef bd; 
                         bd.type = BodyType.DYNAMIC; 
                         bd.position.Set(0.0f, 2.0f); 
                         Body* body = m_world.CreateBody(&bd); 
  
                         b2CircleShape shape; 
                         shape.m_p.SetZero(); 
                         shape.m_radius = 0.5f; 
                         body.CreateFixture(&shape, 1.0f); 
  
                         bd.bullet = true; 
                         bd.position.Set(0.0f, 10.0f); 
                         body = m_world.CreateBody(&bd); 
                         body.CreateFixture(&shape, 1.0f); 
                         body.SetLinearVelocity(Vector2(0.0f, -100.0f)); 
                 } 
 #endif 
 
        }

        void Launch() 
        { 
            m_body.SetTransform(new Vector2(0.0f, 20.0f), 0.0f); 
            m_angularVelocity = RandomFloat(-50.0f, 50.0f); 
            m_body.LinearVelocity = new Vector2(0.0f, -100.0f);
            m_body.AngularVelocity = m_angularVelocity;
        }

        public override void Step(GraphicsDevice graphicsDevice, Settings settings)
        {
            if (m_stepCount == 12) 
            { 
                m_stepCount += 0; 
            } 
 
            base.Step(graphicsDevice, settings);
        }

        public override void Draw(Settings settings, Microsoft.Xna.Framework.Graphics.SpriteBatch spriteBatch, Microsoft.Xna.Framework.Graphics.SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            if (starLiGHT.Physic.TwoD.Collision.gjkCalls > 0) 
            { 
                m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, String.Format("gjk calls = {0}, ave gjk iters = {1}, max gjk iters = {2}",
                                                                                       starLiGHT.Physic.TwoD.Collision.gjkCalls, starLiGHT.Physic.TwoD.Collision.gjkIters / starLiGHT.Physic.TwoD.Collision.gjkCalls, starLiGHT.Physic.TwoD.Collision.gjkMaxIters)); 
                m_textLine += 15; 
            }

            if (starLiGHT.Physic.TwoD.Collision.ToiCalls > 0) 
            { 
                m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, 
                                       String.Format("toi calls = {0}, ave toi iters = {1}, max toi iters = {2}",
                                                     starLiGHT.Physic.TwoD.Collision.ToiCalls,
                                                     starLiGHT.Physic.TwoD.Collision.ToiIters / (float)starLiGHT.Physic.TwoD.Collision.ToiCalls,
                                                     starLiGHT.Physic.TwoD.Collision.MaxToiRootIters)); 
                m_textLine += 15;

                m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, 
                                       String.Format("ave toi root iters = {0}, max toi root iters = {1}",
                                                     starLiGHT.Physic.TwoD.Collision.ToiRootIters / (float)starLiGHT.Physic.TwoD.Collision.ToiCalls,
                                                     starLiGHT.Physic.TwoD.Collision.MaxToiRootIters)); 
                m_textLine += 15; 

            } 

            if (m_stepCount % 60 == 0) 
            { 
                //Launch(); 
            } 
        }

    }
}
