﻿#region Using Statements
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1409                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-09-09 07:27:44 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    class Actor
    {
        public AABB aabb;
        public float fraction;
        public System.Boolean overlap;
        public int proxyId;
    }

    public class QueryCallbackClass : WorldQueryWrapper
    {
        public AABB m_queryAABB;
        public DynamicTree m_tree;

        public override System.Boolean QueryCallback(int proxyId)
        {
            Actor actor = (Actor)m_tree.GetUserData(proxyId);
            actor.overlap = starLiGHT.Physic.TwoD.Collision.TestOverlap(ref m_queryAABB, ref actor.aabb);
            return true;
        }

    }

    public class DynamicTreeTest : Test
    {
        const int e_actorCount = 128;

        float m_worldExtent; 
        float m_proxyExtent;

        System.Random rand;
        DynamicTree m_tree; 
        AABB m_queryAABB; 
        RayCastInput m_rayCastInput; 
        RayCastOutput m_rayCastOutput; 
        Actor m_rayActor; 
        Actor[] m_actors = new Actor[e_actorCount]; 
        int m_stepCount; 
        System.Boolean m_automated; 

        public DynamicTreeTest(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            m_worldExtent = 15.0f;
            m_proxyExtent = 0.5f;

            rand = new System.Random(888);
            //srand(888);

            m_tree = new DynamicTree();

            for (int i = 0; i < e_actorCount; ++i)
            {
                if (m_actors[i] == null) m_actors[i] = new Actor();
                Actor actor = m_actors[i];
                GetRandomAABB(ref actor.aabb);
                actor.proxyId = m_tree.CreateProxy(actor.aabb, actor);
            }

            m_stepCount = 0;

            float h = m_worldExtent;
            m_queryAABB.lowerBound = new Vector2(-3.0f, -4.0f + h);
            m_queryAABB.upperBound = new Vector2(5.0f, 6.0f + h);

            m_rayCastInput.p1 = new Vector2(-5.0f, 5.0f + h);
            m_rayCastInput.p2 = new Vector2(7.0f, -4.0f + h);
            //m_rayCastInput.p1.Set(0.0f, 2.0f + h); 
            //m_rayCastInput.p2.Set(0.0f, -2.0f + h); 
            m_rayCastInput.maxFraction = 1.0f;

            m_automated = false; 
        }

        public override void Step(GraphicsDevice graphicsDevice, Settings settings)
        {
            base.Step(graphicsDevice, settings);

            m_rayActor = null;
            for (int i = 0; i < e_actorCount; ++i)
            {
                m_actors[i].fraction = 1.0f;
                m_actors[i].overlap = false;
            }

            if (m_automated == true)
            {
                int actionCount = System.Math.Max(1, e_actorCount >> 2);

                for (int i = 0; i < actionCount; ++i)
                {
                    Action();
                }
            }

            Query();
            RayCast();

            ++m_stepCount;
        }

        public override void Draw(Settings settings, SpriteBatch spriteBatch, SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            for (int i = 0; i < e_actorCount; ++i)
            {
                Actor actor = m_actors[i];
                if (actor.proxyId == DynamicTree.nullNode)
                    continue;

                Color c = new Color(0.9f, 0.9f, 0.9f);
                if (actor == m_rayActor && actor.overlap)
                {
                    c = new Color(0.9f, 0.6f, 0.6f);
                }
                else if (actor == m_rayActor)
                {
                    c = new Color(0.6f, 0.9f, 0.6f);
                }
                else if (actor.overlap)
                {
                    c = new Color(0.6f, 0.6f, 0.9f);
                }

                m_debugDraw.DrawAABB(ref actor.aabb, c);
            }

            {
                Color c = new Color(0.7f, 0.7f, 0.7f);
                m_debugDraw.DrawAABB(ref m_queryAABB, c);

                m_debugDraw.DrawSegment(m_rayCastInput.p1, m_rayCastInput.p2, c);
            }

            Color c1 = new Color(0.2f, 0.9f, 0.2f);
            Color c2 = new Color(0.9f, 0.2f, 0.2f);
            m_debugDraw.DrawPoint(m_rayCastInput.p1, 6.0f, c1);
            m_debugDraw.DrawPoint(m_rayCastInput.p2, 6.0f, c2);

            if (m_rayActor != null)
            {
                Color cr = new Color(0.2f, 0.2f, 0.9f);
                Vector2 p = m_rayCastInput.p1 + m_rayActor.fraction * (m_rayCastInput.p2 - m_rayCastInput.p1);
                m_debugDraw.DrawPoint(p, 6.0f, cr);
            }

            {
                int height = m_tree.ComputeHeight();
                m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, System.String.Format("dynamic tree height = {0}", height));
                m_textLine += 15;
            }
        }

        public override void Keyboard(KeyboardDevice keyboard, InputManager input)
        {
            base.Keyboard(keyboard, input);

            if (keyboard.GetKey(Keys.A).Pressed) m_automated = !m_automated;
            if (keyboard.GetKey(Keys.C).Pressed) CreateProxy();
            if (keyboard.GetKey(Keys.D).Pressed) DestroyProxy();
            if (keyboard.GetKey(Keys.M).Pressed) MoveProxy();
        }

        bool QueryCallback(int proxyId)
        {
            Actor actor = (Actor)m_tree.GetUserData(proxyId);
            actor.overlap = starLiGHT.Physic.TwoD.Collision.TestOverlap(ref m_queryAABB, ref actor.aabb);
            return true;
        }

        float RayCastCallback(ref RayCastInput input, int proxyId)
        {
            Actor actor = (Actor)m_tree.GetUserData(proxyId);

            RayCastOutput output;
            bool hit = actor.aabb.RayCast(out output, ref input);

            if (hit)
            {
                m_rayCastOutput = output;
                m_rayActor = actor;
                m_rayActor.fraction = output.fraction;
                return output.fraction;
            }

            return input.maxFraction; 
        }

        void GetRandomAABB(ref AABB aabb)
        {
            Vector2 w = new Vector2(2.0f * m_proxyExtent, 2.0f * m_proxyExtent);
            //aabb.lowerBound.x = -m_proxyExtent; 
            //aabb.lowerBound.y = -m_proxyExtent + m_worldExtent; 
            aabb.lowerBound.X = RandomFloat(-m_worldExtent, m_worldExtent);
            aabb.lowerBound.Y = RandomFloat(0.0f, 2.0f * m_worldExtent);
            aabb.upperBound = aabb.lowerBound + w;
        }

        void MoveAABB(ref AABB aabb)
        {
            Vector2 d = new Vector2();
            d.X = RandomFloat(-0.5f, 0.5f);
            d.Y = RandomFloat(-0.5f, 0.5f);
            //d.x = 2.0f; 
            //d.y = 0.0f; 
            aabb.lowerBound += d;
            aabb.upperBound += d;

            Vector2 c0 = 0.5f * (aabb.lowerBound + aabb.upperBound);
            Vector2 min = new Vector2(-m_worldExtent, 0.0f);
            Vector2 max = new Vector2(m_worldExtent, 2.0f * m_worldExtent);
            Vector2 c = Vector2.Clamp(c0, min, max);

            aabb.lowerBound += c - c0;
            aabb.upperBound += c - c0;
        }

        void CreateProxy()
        {
            for (int i = 0; i < e_actorCount; ++i)
            {
                int j = rand.Next() % e_actorCount;
                Actor actor = m_actors[j];
                if (actor.proxyId == DynamicTree.nullNode)
                {
                    GetRandomAABB(ref actor.aabb);
                    actor.proxyId = m_tree.CreateProxy(actor.aabb, actor);
                    return;
                }
            }
        }

        void DestroyProxy()
        {
            for (int i = 0; i < e_actorCount; ++i)
            {
                int j = rand.Next() % e_actorCount;
                Actor actor = m_actors[j];
                if (actor.proxyId != DynamicTree.nullNode)
                {
                    m_tree.DestroyProxy(actor.proxyId);
                    actor.proxyId = DynamicTree.nullNode;
                    return;
                }
            }
        }

        void MoveProxy()
        {
            for (int i = 0; i < e_actorCount; ++i)
            {
                int j = rand.Next() % e_actorCount;
                Actor actor = m_actors[j];
                if (actor.proxyId == DynamicTree.nullNode)
                {
                    continue;
                }

                AABB aabb0 = actor.aabb;
                MoveAABB(ref actor.aabb);
                Vector2 displacement = actor.aabb.GetCenter() - aabb0.GetCenter();
                m_tree.MoveProxy(actor.proxyId, ref actor.aabb, displacement);
                return;
            }
        }

        void Action()
        {
            int choice = rand.Next() % 20;

            switch (choice)
            {
                case 0:
                    CreateProxy();
                    break;

                case 1:
                    DestroyProxy();
                    break;

                default:
                    MoveProxy();
                    break;
            }
        }

        void Query()
        {
            QueryCallbackClass callback = new QueryCallbackClass();
            callback.m_tree = m_tree;
            callback.m_queryAABB = m_queryAABB;

            m_tree.Query(callback, ref m_queryAABB);

            for (int i = 0; i < e_actorCount; ++i)
            {
                if (m_actors[i].proxyId == DynamicTree.nullNode)
                {
                    continue;
                }

                bool overlap = starLiGHT.Physic.TwoD.Collision.TestOverlap(ref m_queryAABB, ref m_actors[i].aabb);
                //B2_NOT_USED(overlap);
                System.Diagnostics.Debug.Assert(overlap == m_actors[i].overlap);
            }
        }

        void RayCast()
        {
            m_rayActor = null;

            RayCastInput input = m_rayCastInput;

            // Ray cast against the dynamic tree. 
            m_tree.RayCast(RayCastCallback, ref input);

            // Brute force ray cast. 
            Actor bruteActor = null;
            RayCastOutput bruteOutput = new RayCastOutput();
            for (int i = 0; i < e_actorCount; ++i)
            {
                if (m_actors[i].proxyId == DynamicTree.nullNode)
                {
                    continue;
                }

                RayCastOutput output;
                bool hit = m_actors[i].aabb.RayCast(out output, ref input);
                if (hit)
                {
                    bruteActor = m_actors[i];
                    bruteOutput = output;
                    input.maxFraction = output.fraction;
                }
            }

            if (bruteActor != null)
            {
                System.Diagnostics.Debug.Assert(bruteOutput.fraction == m_rayCastOutput.fraction);
            }
        } 
 
    }
}
