﻿#region Using Statements
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1482                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-07 11:53:48 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class OneSidedPlatform : Test
    {
        float m_radius, m_top, m_bottom;
        State m_state;
        Fixture m_platform;
        Fixture m_character; 
 
        enum State
        {
            Unknown,
            Above,
            Below,
        }; 
 
        public OneSidedPlatform(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            // Ground 
            RigidBody2D ground = PhysicSystem2D.Instance.createEdge(new Vector2(-20.0f, 0.0f), new Vector2(20.0f, 0.0f)); 

            // Platform 
            {
                BodyDef bd = new BodyDef();
                bd.position = new Vector2(0.0f, 10.0f);
                Body body = m_world.CreateBody(bd);

                PolygonShape shape = new PolygonShape();
                shape.SetAsBox(3.0f, 0.5f);
                m_platform = body.CreateFixture(shape, 0.0f);

                m_bottom = 10.0f - 0.5f;
                m_top = 10.0f + 0.5f;
            }

            // Actor 
            {
                BodyDef bd = new BodyDef();
                bd.type = BodyType.DYNAMIC;
                bd.position = new Vector2(0.0f, 12.0f);
                Body body = m_world.CreateBody(bd);

                m_radius = 0.5f;
                CircleShape shape = new CircleShape();
                shape.m_radius = m_radius;
                m_character = body.CreateFixture(shape, 20.0f);

                body.LinearVelocity = new Vector2(0.0f, -50.0f);

                m_state = State.Unknown;
            } 
        }

        public override void PreSolve(Contact contact, Manifold oldManifold)
        {
            base.PreSolve(contact, oldManifold);

            Fixture fixtureA = contact.GetFixtureA();
            Fixture fixtureB = contact.GetFixtureB();

            if (fixtureA != m_platform && fixtureA != m_character)
            {
                return;
            }

            if (fixtureB != m_platform && fixtureB != m_character)
            {
                return;
            }

            Vector2 position = m_character.GetBody().Position;

            if (position.Y < m_top + m_radius - 3.0f * PhysicSettings.LinearSlop)
            {
                contact.SetEnabled(false);
            }
        }

        //public override void Step(Settings settings)
        //{
        //    base.Step(settings);
        //}

        public override void Draw(Settings settings, SpriteBatch spriteBatch, SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Press: (c) create a shape, (d) destroy a shape.");
            m_textLine += 15;
        }
    }
}
