﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 971                   $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-06-21 18:13:19 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{

    public class PinballTest : Test
    {
        public PinballTest(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
                // Ground body
                Body ground = null;
                {
                        BodyDef bd = new BodyDef();
                        ground = m_world.CreateBody(bd);

                        Vector2[] vs = new Vector2[5];
                        vs[0]= new Vector2(0.0f, -2.0f);
                        vs[1]= new Vector2(8.0f, 6.0f);
                        vs[2]= new Vector2(8.0f, 20.0f);
                        vs[3]= new Vector2(-8.0f, 20.0f);
                        vs[4]= new Vector2(-8.0f, 6.0f);

                        LoopShape loop = new LoopShape();
                        loop.Create(vs, 5);
                        FixtureDef fd = new FixtureDef();
                        fd.shape = loop.Clone();
                        fd.density = 0.0f;
                        ground.CreateFixture(fd);
                }

                // Flippers
                {
                        Vector2 p1 = new Vector2(-2.0f, 0.0f), p2 = new Vector2(2.0f, 0.0f);

                        BodyDef bd = new BodyDef();
                        bd.type = BodyType.DYNAMIC;

                        bd.position = p1;
                        Body leftFlipper = m_world.CreateBody(bd);

                        bd.position = p2;
                        Body rightFlipper = m_world.CreateBody(bd);

                        PolygonShape box = new PolygonShape();
                        box.SetAsBox(1.75f, 0.1f);

                        FixtureDef fd = new FixtureDef(); 
                        fd.shape = box.Clone();
                        fd.density = 1.0f;

                        leftFlipper.CreateFixture(fd);
                        rightFlipper.CreateFixture(fd);

                        RevoluteJointDef jd = new RevoluteJointDef();
                        jd.body1 = ground;
                        jd.localAnchor2 = Vector2.Zero;
                        jd.enableMotor = true;
                        jd.maxMotorTorque = 1000.0f;
                        jd.enableLimit = true;

                        jd.motorSpeed = 0.0f;
                        jd.localAnchor1 = p1;
                        jd.body2 = leftFlipper;
                        jd.lowerAngle = -30.0f * (float)System.Math.PI / 180.0f;
                        jd.upperAngle = 5.0f * (float)System.Math.PI / 180.0f;
                        m_leftJoint = (RevoluteJoint)m_world.CreateJoint(jd);

                        jd.motorSpeed = 0.0f;
                        jd.localAnchor1 = p2;
                        jd.body2 = rightFlipper;
                        jd.lowerAngle = -5.0f * (float)System.Math.PI / 180.0f;
                        jd.upperAngle = 30.0f * (float)System.Math.PI / 180.0f;
                        m_rightJoint = (RevoluteJoint)m_world.CreateJoint(jd);
                }

                // Circle character
                {
                        BodyDef bd = new BodyDef();
                        bd.position = new Vector2(1.0f, 15.0f);
                        bd.type = BodyType.DYNAMIC;
                        bd.bullet = true;

                        m_ball = m_world.CreateBody(bd);

                        CircleShape shape = new CircleShape();
                        shape.m_radius = 0.2f;

                        FixtureDef fd = new FixtureDef();
                        fd.shape = shape;
                        fd.density = 1.0f;
                        m_ball.CreateFixture(fd);
                }

                m_button = false;
        }

        public override void Draw(Settings settings, SpriteBatch spriteBatch, SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Press 'a' to control the flippers");
            m_textLine += 15;
       
        }

        public override void Step(GraphicsDevice graphicsDevice, Settings settings)
        {
            if (m_button)
            {
                m_leftJoint.SetMotorSpeed(20.0f);
                m_rightJoint.SetMotorSpeed(-20.0f);
            }
            else
            {
                m_leftJoint.SetMotorSpeed(-10.0f);
                m_rightJoint.SetMotorSpeed(10.0f);
            }

            base.Step(graphicsDevice, settings);
        }

        public override void Keyboard(KeyboardDevice keyboard, InputManager input)
        {
            base.Keyboard(keyboard, input);

            if (keyboard.GetKey(Keys.A).Pressed)
            {
                m_button = true;
            }

            if (keyboard.GetKey(Keys.A).Released)
            {
                m_button = false;
            }
        }

        RevoluteJoint m_leftJoint;
        RevoluteJoint m_rightJoint;
        Body m_ball;
        Boolean m_button;
    }
}
