﻿#region Using Statements
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1482                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-07 11:53:48 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class PolyCollision : Test
    {
        PolygonShape m_polygonA = new PolygonShape();
        PolygonShape m_polygonB = new PolygonShape();

        Transform m_transformA = new Transform();
        Transform m_transformB = new Transform();

        Vector2 m_positionB;
        float m_angleB; 
 
        public PolyCollision(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            {
                m_polygonA.SetAsBox(0.2f, 0.4f);
                m_transformA.Set(new Vector2(0.0f, 0.0f), 0.0f);
            }

            {
                m_polygonB.SetAsBox(0.5f, 0.5f);
                m_positionB = new Vector2(19.345284f, 1.5632932f);
                m_angleB = 1.9160721f;
                m_transformB.Set(m_positionB, m_angleB);
            } 
 
        }

        System.String debugString;

        //public override void Step(Settings settings)
        //{
        //    base.Step(settings);
     
        //}

        public override void Keyboard(KeyboardDevice keyboard, InputManager input)
        {
            base.Keyboard(keyboard, input);

            if (keyboard.GetKey(Keys.A).Pressed) m_positionB.X -= 0.1f;
            if (keyboard.GetKey(Keys.D).Pressed) m_positionB.X += 0.1f;
            if (keyboard.GetKey(Keys.S).Pressed) m_positionB.Y -= 0.1f;
            if (keyboard.GetKey(Keys.W).Pressed) m_positionB.Y += 0.1f;

            if (keyboard.GetKey(Keys.Q).Pressed) m_angleB += 0.1f * (float)Math.PI;
            if (keyboard.GetKey(Keys.E).Pressed) m_angleB -= 0.1f * (float)Math.PI;

            m_transformB.Set(m_positionB, m_angleB);
        }

        public override void Draw(Settings settings, SpriteBatch spriteBatch, SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, debugString);
            m_textLine += 15;

            Manifold manifold;
            starLiGHT.Physic.TwoD.Collision.CollidePolygons(out manifold, m_polygonA, ref m_transformA, m_polygonB, ref m_transformB);

            WorldManifold worldManifold = new WorldManifold();
            worldManifold.Initialize(manifold, m_transformA, m_polygonA.m_radius, m_transformB, m_polygonB.m_radius);

            debugString = System.String.Format("point count = {0}", manifold.pointCount);

            {
                Color color = new Color(0.9f, 0.9f, 0.9f);
                Vector2[] v = new Vector2[PhysicSettings.MaxPolygonVertices];
                for (int i = 0; i < m_polygonA.m_vertexCount; ++i)
                {
                    v[i] = Math.Mul(m_transformA, m_polygonA.m_vertices[i]);
                }
                m_debugDraw.DrawPolygon(v, m_polygonA.m_vertexCount, color);

                for (int i = 0; i < m_polygonB.m_vertexCount; ++i)
                {
                    v[i] = Math.Mul(m_transformB, m_polygonB.m_vertices[i]);
                }
                m_debugDraw.DrawPolygon(v, m_polygonB.m_vertexCount, color);
            }

            for (int i = 0; i < manifold.pointCount; ++i)
            {
                m_debugDraw.DrawPoint(worldManifold.points[i], 4.0f, new Color(0.9f, 0.3f, 0.3f));
            } 

        }
    }
}
