﻿#region Using Statements
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1482                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-07 11:53:48 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    /// This callback is called by b2World::QueryAABB. We find all the fixtures 
    /// that overlap an AABB. Of those, we use b2TestOverlap to determine which fixtures 
    /// overlap a circle. Up to 4 overlapped fixtures will be highlighted with a yellow border. 
    class PolyShapesCallback : QueryCallback 
    { 
        static int e_maxCount = 4;

        public PolyShapesCallback() 
        { 
            m_count = 0; 
        } 

        void DrawFixture(Fixture fixture) 
        { 
            Color color = new Color(0.95f, 0.95f, 0.6f); 
            Transform xf = fixture.GetBody().Transform; 

            switch (fixture.GetType()) 
            { 
                case ShapeType.CIRCLE: 
                { 
                    CircleShape circle = (CircleShape)fixture.GetShape(); 

                    Vector2 center = starLiGHT.Physic.TwoD.Math.Mul(xf, circle.m_p); 
                    float radius = circle.m_radius; 

                    m_debugDraw.DrawCircle(center, radius, color); 
                } 
                break; 

                case ShapeType.POLYGON: 
                { 
                    PolygonShape poly = (PolygonShape)fixture.GetShape(); 
                    int vertexCount = poly.m_vertexCount; 
                    System.Diagnostics.Debug.Assert(vertexCount <= PhysicSettings.MaxPolygonVertices); 
                    Vector2[] vertices = new Vector2[PhysicSettings.MaxPolygonVertices]; 

                    for (int i = 0; i < vertexCount; ++i) 
                    {
                        vertices[i] = starLiGHT.Physic.TwoD.Math.Mul(xf, poly.m_vertices[i]); 
                    } 

                    m_debugDraw.DrawPolygon(vertices, vertexCount, color); 
                } 
                break; 
            } 
        } 

        /// Called for each fixture found in the query AABB. 
        /// @return false to terminate the query. 
        public override System.Boolean ReportFixture(Fixture fixture) 
        { 
            if (m_count == e_maxCount) 
            { 
                return false; 
            } 

            Body body = fixture.GetBody(); 
            Shape shape = fixture.GetShape();

            bool overlap = starLiGHT.Physic.TwoD.Collision.TestOverlap(shape, 0, m_circle, 0, body.Transform, m_transform); 

            if (overlap) 
            { 
                DrawFixture(fixture); 
                ++m_count; 
            } 

            return true; 
        } 

        public CircleShape m_circle = new CircleShape(); 
        public Transform m_transform = new Transform(); 
        public DebugDraw m_debugDraw; 
        int m_count; 
    }; 
 
    public class PolyShapes : Test
    {
        static int k_maxBodies = 256;
        int m_bodyIndex;
        Body[] m_bodies = new Body[k_maxBodies];
        PolygonShape[] m_polygons = new PolygonShape[4];
        CircleShape m_circle = new CircleShape();

        public PolyShapes(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            // Ground body
            RigidBody2D ground = PhysicSystem2D.Instance.createEdge(new Vector2(-40.0f, 0.0f), new Vector2(40.0f, 0.0f)); 

            { 
                Vector2[] vertices = new Vector2[3]; 
                vertices[0] = new Vector2(-0.5f, 0.0f); 
                vertices[1] = new Vector2(0.5f, 0.0f); 
                vertices[2] = new Vector2(0.0f, 1.5f);
                m_polygons[0] = new PolygonShape();
                m_polygons[0].Set(vertices, 3); 
            } 

            { 
                Vector2[] vertices = new Vector2[3]; 
                vertices[0] = new Vector2(-0.1f, 0.0f); 
                vertices[1] = new Vector2(0.1f, 0.0f); 
                vertices[2] = new Vector2(0.0f, 1.5f);
                m_polygons[1] = new PolygonShape();
                m_polygons[1].Set(vertices, 3); 
            } 

            { 
                float w = 1.0f; 
                float b = w / (2.0f + (float)System.Math.Sqrt(2.0f)); 
                float s = (float)System.Math.Sqrt(2.0f) * b;

                Vector2[] vertices = new Vector2[8];
                vertices[0] = new Vector2(0.5f * s, 0.0f);
                vertices[1] = new Vector2(0.5f * w, b);
                vertices[2] = new Vector2(0.5f * w, b + s);
                vertices[3] = new Vector2(0.5f * s, w);
                vertices[4] = new Vector2(-0.5f * s, w);
                vertices[5] = new Vector2(-0.5f * w, b + s);
                vertices[6] = new Vector2(-0.5f * w, b);
                vertices[7] = new Vector2(-0.5f * s, 0.0f);

                m_polygons[2] = new PolygonShape();
                m_polygons[2].Set(vertices, 8); 
            } 

            {
                m_polygons[3] = new PolygonShape();
                m_polygons[3].SetAsBox(0.5f, 0.5f); 
            } 

            { 
                m_circle.m_radius = 0.5f; 
            } 

            m_bodyIndex = 0;
            m_bodies.Initialize();
            //memset(m_bodies, 0, sizeof(m_bodies)); 
        }

        void Create(int index) 
        { 
            if (m_bodies[m_bodyIndex] != null) 
            { 
                m_world.DestroyBody(m_bodies[m_bodyIndex]); 
                m_bodies[m_bodyIndex] = null; 
            } 

            BodyDef bd = new BodyDef(); 
            bd.type = BodyType.DYNAMIC; 

            float x = RandomFloat(-2.0f, 2.0f); 
            bd.position = new Vector2(x, 10.0f);
            bd.angle = RandomFloat(-(float)System.Math.PI, (float)System.Math.PI); 

            if (index == 4) 
            { 
                bd.angularDamping = 0.02f; 
            } 

            m_bodies[m_bodyIndex] = m_world.CreateBody(bd); 

            if (index < 4) 
            { 
                FixtureDef fd = new FixtureDef(); 
                fd.shape = m_polygons[index]; 
                fd.density = 1.0f; 
                fd.friction = 0.3f; 
                m_bodies[m_bodyIndex].CreateFixture(fd); 
            } 
            else 
            { 
                FixtureDef fd = new FixtureDef(); 
                fd.shape = m_circle; 
                fd.density = 1.0f; 
                fd.friction = 0.3f; 

                m_bodies[m_bodyIndex].CreateFixture(fd); 
            } 

            m_bodyIndex = (m_bodyIndex + 1) % k_maxBodies; 
        } 
  
        void DestroyBody() 
        { 
            for (int i = 0; i < k_maxBodies; ++i) 
            { 
                if (m_bodies[i] != null) 
                { 
                    m_world.DestroyBody(m_bodies[i]); 
                    m_bodies[i] = null; 
                    return; 
                } 
            } 
        }

        public override void Keyboard(KeyboardDevice keyboard, InputManager input)
        {
 	         base.Keyboard(keyboard, input);

            if (keyboard.GetKey(Keys.D1).Pressed) Create(0);
            if (keyboard.GetKey(Keys.D2).Pressed) Create(1);
            if (keyboard.GetKey(Keys.D3).Pressed) Create(2);
            if (keyboard.GetKey(Keys.D4).Pressed) Create(3);
            if (keyboard.GetKey(Keys.D5).Pressed) Create(4);
            if (keyboard.GetKey(Keys.D).Pressed) DestroyBody();
            if (keyboard.GetKey(Keys.A).Pressed)
            {
                for (int i = 0; i < k_maxBodies; i+= 2)
                {
                    if (m_bodies[i] != null)
                    {
                        bool active = m_bodies[i].Active;
                        m_bodies[i].Active = !active;
                    }
                }
            }
        }

        public override void  Draw(Settings settings, SpriteBatch spriteBatch, SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            PolyShapesCallback callback = new PolyShapesCallback(); 
            callback.m_circle.m_radius = 2.0f; 
            callback.m_circle.m_p = new Vector2(0.0f, 2.1f); 
            callback.m_transform.SetIdentity(); 
            callback.m_debugDraw = m_debugDraw; 

            AABB aabb; 
            callback.m_circle.ComputeAABB(out aabb, ref callback.m_transform, 0); 

            m_world.QueryAABB(callback, ref aabb); 

            Color color = new Color(0.4f, 0.7f, 0.8f); 
            m_debugDraw.DrawCircle(callback.m_circle.m_p, callback.m_circle.m_radius, color); 

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Press 1-5 to drop stuff"); 
            m_textLine += 15;
            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Press 'a' to (de)activate some bodies"); 
            m_textLine += 15;
            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Press 'd' to destroy a body"); 
            m_textLine += 15; 
         }
    }
}
