﻿#region Using Statements
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1482                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-07 11:53:48 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class Prismatic : Test
    {
        PrismaticJoint m_joint;

        public Prismatic(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            // Ground body
            RigidBody2D ground = PhysicSystem2D.Instance.createEdge(new Vector2(-40.0f, 0.0f), new Vector2(40.0f, 0.0f)); 

            { 
                PolygonShape shape = new PolygonShape(); 
                shape.SetAsBox(2.0f, 0.5f); 

                BodyDef bd = new BodyDef(); 
                bd.type = BodyType.DYNAMIC; 
                bd.position = new Vector2(-10.0f, 10.0f); 
                bd.angle = 0.5f * (float)System.Math.PI; 
                bd.allowSleep = false; 
                Body body = m_world.CreateBody(bd); 
                body.CreateFixture(shape, 5.0f); 

                PrismaticJointDef pjd = new PrismaticJointDef(); 

                // Bouncy limit 
                Vector2 axis = new Vector2(2.0f, 1.0f); 
                axis.Normalize(); 
                pjd.Initialize(ground.Body, body, Vector2.Zero, axis); 

                // Non-bouncy limit 
                //pjd.Initialize(ground, body, b2Vec2(-10.0f, 10.0f), b2Vec2(1.0f, 0.0f)); 

                pjd.motorSpeed = 10.0f; 
                pjd.maxMotorForce = 10000.0f; 
                pjd.enableMotor = true; 
                pjd.lowerTranslation = 0.0f; 
                pjd.upperTranslation = 20.0f; 
                pjd.enableLimit = true; 

                m_joint = (PrismaticJoint)m_world.CreateJoint(pjd); 
            } 
        }

        public override void Keyboard(KeyboardDevice keyboard, InputManager input)
        {
            base.Keyboard(keyboard, input);

            if (keyboard.GetKey(Keys.L).Pressed) m_joint.EnableLimit(!m_joint.IsLimitEnabled());
            if (keyboard.GetKey(Keys.M).Pressed) m_joint.EnableMotor(!m_joint.IsMotorEnabled());
            if (keyboard.GetKey(Keys.S).Pressed) m_joint.SetMotorSpeed(-m_joint.GetMotorSpeed());
        }

        public override void Draw(Settings settings, SpriteBatch spriteBatch, SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Keys: (l) limits, (m) motors, (s) speed");
            m_textLine += 15;
            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, System.String.Format("Motor Force = {0}", m_joint.GetMotorForce(settings.hz)));
            m_textLine += 15; 
        }
    }
}
