﻿#region Using Statements
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1482                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-07 11:53:48 #$:  Date of last commit     *
// ************************************************************
#endregion

// This test demonstrates how to use the world ray-cast feature.
// NOTE: we are intentionally filtering one of the polygons, therefore
// the ray will always miss one type of polygon.

namespace starLiGHT.Sandbox.Physic2D
{
    public class RayCast : Test
    {
        static int e_maxBodies = 256;

        enum Mode
        {
            Closest,
            Any,
            Multiple,
        }

        int m_bodyIndex; 
        Body[] m_bodies = new Body[e_maxBodies];
        int[] m_userData = new int[e_maxBodies];
        PolygonShape[] m_polygons = new PolygonShape[4]; 
        CircleShape m_circle = new CircleShape(); 
 
        float m_angle;

        Mode m_mode;

        public RayCast(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            // Ground body
            RigidBody2D ground = PhysicSystem2D.Instance.createEdge(new Vector2(-40.0f, 0.0f), new Vector2(40.0f, 0.0f)); 

            { 
                Vector2[] vertices = new Vector2[3]; 
                vertices[0] = new Vector2(-0.5f, 0.0f); 
                vertices[1] = new Vector2(0.5f, 0.0f); 
                vertices[2] = new Vector2(0.0f, 1.5f);
                m_polygons[0] = new PolygonShape();
                m_polygons[0].Set(vertices, 3); 
            } 

            { 
                Vector2[] vertices = new Vector2[3]; 
                vertices[0] = new Vector2(-0.1f, 0.0f); 
                vertices[1] = new Vector2(0.1f, 0.0f); 
                vertices[2] = new Vector2(0.0f, 1.5f);
                m_polygons[1] = new PolygonShape();
                m_polygons[1].Set(vertices, 3); 
            } 

            { 
                float w = 1.0f; 
                float b = w / (2.0f + (float)System.Math.Sqrt(2.0f)); 
                float s = (float)System.Math.Sqrt(2.0f) * b;

                Vector2[] vertices = new Vector2[8];
                vertices[0] = new Vector2(0.5f * s, 0.0f);
                vertices[1] = new Vector2(0.5f * w, b);
                vertices[2] = new Vector2(0.5f * w, b + s);
                vertices[3] = new Vector2(0.5f * s, w);
                vertices[4] = new Vector2(-0.5f * s, w);
                vertices[5] = new Vector2(-0.5f * w, b + s);
                vertices[6] = new Vector2(-0.5f * w, b);
                vertices[7] = new Vector2(-0.5f * s, 0.0f);

                m_polygons[2] = new PolygonShape();
                m_polygons[2].Set(vertices, 8); 
            } 

            {
                m_polygons[3] = new PolygonShape();
                m_polygons[3].SetAsBox(0.5f, 0.5f); 
            } 

            { 
                m_circle.m_radius = 0.5f; 
            } 

            m_bodyIndex = 0;
            m_bodies.Initialize();
            //memset(m_bodies, 0, sizeof(m_bodies)); 

            m_angle = 0.0f;

            m_mode = Mode.Closest;
        }

        void Create(int index)
        {
            if (m_bodies[m_bodyIndex] != null)
            {
                m_world.DestroyBody(m_bodies[m_bodyIndex]);
                m_bodies[m_bodyIndex] = null;
            }

            BodyDef bd = new BodyDef();

            float x = RandomFloat(-10.0f, 10.0f);
            float y = RandomFloat(0.0f, 20.0f);
            bd.position = new Vector2(x, y);
            bd.angle = RandomFloat(-(float)System.Math.PI, (float)System.Math.PI);

            m_userData[m_bodyIndex] = index;
            bd.userData = m_userData[m_bodyIndex];

            if (index == 4)
            {
                bd.angularDamping = 0.02f;
            }

            m_bodies[m_bodyIndex] = m_world.CreateBody(bd);

            if (index < 4)
            {
                FixtureDef fd = new FixtureDef();
                fd.shape = m_polygons[index].Clone();
                fd.friction = 0.3f;
                m_bodies[m_bodyIndex].CreateFixture(fd);
            }
            else
            {
                FixtureDef fd = new FixtureDef();
                fd.shape = m_circle.Clone();
                fd.friction = 0.3f;

                m_bodies[m_bodyIndex].CreateFixture(fd);
            }

            m_bodyIndex = (m_bodyIndex + 1) % e_maxBodies;
        }

        void DestroyBody()
        {
            for (int i = 0; i < e_maxBodies; ++i)
            {
                if (m_bodies[i] != null)
                {
                    m_world.DestroyBody(m_bodies[i]);
                    m_bodies[i] = null;
                    return;
                }
            }
        }

        public override void Keyboard(KeyboardDevice keyboard, InputManager input)
        {
            base.Keyboard(keyboard, input);

            if (keyboard.GetKey(Keys.D1).Pressed) Create(0);
            if (keyboard.GetKey(Keys.D2).Pressed) Create(1);
            if (keyboard.GetKey(Keys.D3).Pressed) Create(2);
            if (keyboard.GetKey(Keys.D4).Pressed) Create(3);
            if (keyboard.GetKey(Keys.D5).Pressed) Create(4);
            if (keyboard.GetKey(Keys.D).Pressed) DestroyBody();

            if (keyboard.GetKey(Keys.M).Pressed)
            {
                if (m_mode == Mode.Closest)
                    m_mode = Mode.Any;
                else if (m_mode == Mode.Any)
                    m_mode = Mode.Multiple;
                else if (m_mode == Mode.Multiple)
                    m_mode = Mode.Closest;
            }
        }

        //public override void Step(Settings settings)
        //{
        //    base.Step(settings);
        //}

        public override void Draw(Settings settings, SpriteBatch spriteBatch, SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Press 1-5 to drop stuff");
            m_textLine += 15;

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, System.String.Format("Mode = {0}", m_mode));
            m_textLine += 15;

            bool advanceRay = settings.pause == 0 || settings.singleStep > 0;

            float L = 11.0f;
            Vector2 point1 = new Vector2(0.0f, 10.0f);
            Vector2 d = new Vector2(L * (float)System.Math.Cos(m_angle), L * (float)System.Math.Sin(m_angle));
            Vector2 point2 = point1 + d;

            Vector2 m_point = Vector2.Zero;
            Vector2 m_normal = Vector2.Zero;

            if (m_mode == Mode.Closest)
            {
                bool m_hit = false;
                m_world.RayCast((fixture, p, n, fr) =>
                {
                    Body body = fixture.GetBody();
                    object userData = body.UserData;

                    if (userData != null)
                    {
                        int index = (int)userData;
                        if (index == 0)
                        {
                            // filter
                            return -1.0f;
                        }
                    }

                    m_hit = true;
                    m_point = p;
                    m_normal = n;
                    return fr;
                }
                                , point1, point2);

                if (m_hit)
                {
                    m_debugDraw.DrawPoint(m_point, 5.0f, new Color(0.4f, 0.9f, 0.4f));
                    m_debugDraw.DrawSegment(point1, m_point, new Color(0.8f, 0.8f, 0.8f));
                    Vector2 head = m_point + 0.5f * m_normal;
                    m_debugDraw.DrawSegment(m_point, head, new Color(0.9f, 0.9f, 0.4f));
                }
                else
                {
                    m_debugDraw.DrawSegment(point1, point2, new Color(0.8f, 0.8f, 0.8f));
                }
            }
            else if (m_mode == Mode.Any)
            {
                bool m_hit = false;

                m_world.RayCast((fixture, point, normal, fraction) =>
                    {
                        Body body = fixture.GetBody();
                        object userData = body.UserData;

                        if (userData != null)
                        {
                            int index = (int)userData;
                            if (index == 0)
                            {
                                // filter
                                return -1.0f;
                            }
                        }

                        m_hit = true;
                        m_point = point;
                        m_normal = normal;
                        return 0.0f;
                    }
                    , point1, point2);

                if (m_hit)
                {
                    m_debugDraw.DrawPoint(m_point, 5.0f, new Color(0.4f, 0.9f, 0.4f));
                    m_debugDraw.DrawSegment(point1, m_point, new Color(0.8f, 0.8f, 0.8f));
                    Vector2 head = m_point + 0.5f * m_normal;
                    m_debugDraw.DrawSegment(m_point, head, new Color(0.9f, 0.9f, 0.4f));
                }
                else
                {
                    m_debugDraw.DrawSegment(point1, point2, new Color(0.8f, 0.8f, 0.8f));
                }
            }
            else if (m_mode == Mode.Multiple)
            {
                bool m_hit = false;
                int maxCount = 3;
                Vector2[] m_points = new Vector2[maxCount];
                Vector2[] m_normals = new Vector2[maxCount];
                int m_count = 0;

                m_world.RayCast((fixture, point, normal, fraction) =>
                    {
                        Body body = fixture.GetBody();
                        object userData = body.UserData;

                        if (userData != null)
                        {
                            if ((int)userData == 0)
                            {
                                // filter
                                return -1.0f;
                            }
                        }

                        System.Diagnostics.Debug.Assert(m_count < maxCount);

                        m_points[m_count] = point;
                        m_normals[m_count] = normal;
                        ++m_count;

                        if (m_count == maxCount)
                        {
                            return 0.0f;
                        }

                        return 1.0f;
                    }
                    , point1, point2);

                m_debugDraw.DrawSegment(point1, point2, new Color(0.8f, 0.8f, 0.8f));

                for (int i = 0; i < m_count; ++i)
                {
                    Vector2 p = m_points[i];
                    Vector2 n = m_normals[i];
                    m_debugDraw.DrawPoint(p, 5.0f, new Color(0.4f, 0.9f, 0.4f));
                    m_debugDraw.DrawSegment(point1, p, new Color(0.8f, 0.8f, 0.8f));
                    Vector2 head = p + 0.5f * n;
                    m_debugDraw.DrawSegment(p, head, new Color(0.9f, 0.9f, 0.4f));
                }
            }

            if (advanceRay)
            {
                m_angle += 0.25f * (float)System.Math.PI / 180.0f;
            }
        }

    }
}
