﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using starLiGHT;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 7607                  $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2011-03-25 03:47:10 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class RopeTest : Test
    {
        public RopeTest(Microsoft.Xna.Framework.Game game)
            : base(game, 4.0f)
        {
            int N = 40;
            Vector2[] vertices = new Vector2[N];
            float[] masses = new float[N];

            for (int i = 0; i < N; ++i)
            {
                vertices[i] = new Vector2(0f, 20f - 0.25f * i);
                masses[i] = 1f;
            }
            masses[0] = 0f;
            masses[1] = 0f;

            RopeDef def = new RopeDef();
            def.vertices = vertices;
            def.count = N;
            def.gravity = new Vector2(0f, -10f);
            def.masses = masses;
            def.damping = 0.1f;
            def.k2 = 1.0f;
            def.k3 = 0.5f;

            m_rope.Initialize(def);

            m_angle = 0f;
            m_rope.SetAngle(m_angle);
        }

        public override void Keyboard(KeyboardDevice keyboard, InputManager input)
        {
            base.Keyboard(keyboard, input);

            if (keyboard.GetKey(Keys.Q).Pressed)
            {
                m_angle = (float)System.Math.Max(-System.Math.PI, m_angle - 0.05f * System.Math.PI);
                m_rope.SetAngle(m_angle);
            }

            if (keyboard.GetKey(Keys.E).Pressed)
            {
                m_angle = (float)System.Math.Max(System.Math.PI, m_angle + 0.05f * System.Math.PI);
                m_rope.SetAngle(m_angle);
            }
        }

        public override void Step(Microsoft.Xna.Framework.Graphics.GraphicsDevice graphicsDevice, Settings settings)
        {
            float dt = settings.hz > 0f ? 1f / settings.hz : 0f;

            if (settings.pause == 1 && settings.singleStep == 0)
            {
                dt = 0f;
            }

            m_rope.Step(dt, 1);

            base.Step(graphicsDevice, settings);
        }

        public override void Draw(Settings settings, Microsoft.Xna.Framework.Graphics.SpriteBatch spriteBatch, Microsoft.Xna.Framework.Graphics.SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_rope.Draw(m_debugDraw);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Press (q,e) to adjust target angle");
            m_textLine += 15;
            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, String.Format("Target angle = {0} degrees", MathHelper.ToDegrees(m_angle)));
            m_textLine += 15;
        }

        private Rope m_rope;
        private float m_angle;
    }
}

