﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using starLiGHT;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 5003                  $:  Revision of last commit *
// * $Author:: unknown            $:  Author of last commit   *
// * $Date:: 2011-02-04 16:53:43 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class RopeJoint : Test
    {
        public RopeJoint(Microsoft.Xna.Framework.Game game)
            : base(game, 4.0f)
        {
            RigidBody2D ground = PhysicSystem2D.Instance.createEdge(new Vector2(-40.0f, 0.0f), new Vector2(40.0f, 0.0f));

            PolygonShape shape = new PolygonShape();
            shape.SetAsBox(0.5f, 0.125f);

            FixtureDef fd = new FixtureDef();
            fd.shape = shape;
            fd.density = 20.0f;
            fd.friction = 0.2f;
            fd.filter.categoryBits = 0x0001;
            fd.filter.maskBits = 0xFFFF & ~0x0002;

            RevoluteJointDef jd = new RevoluteJointDef();
            jd.collideConnected = false;

            const int N = 10;
            const float y = 15.0f;
            m_ropeDef.localAnchorA = new Vector2(0.0f, y);

            Body prevBody = ground.Body;
            for (int i = 0; i < N; ++i)
            {
                BodyDef bd = new BodyDef();
                bd.type = BodyType.DYNAMIC;
                bd.position = new Vector2(0.5f + 1.0f * i, y);
                if (i == N - 1)
                {
                    shape.SetAsBox(1.5f, 1.5f);
                    fd.density = 100.0f;
                    fd.filter.categoryBits = 0x0002;
                    bd.position = new Vector2(1.0f * 1, y);
                    bd.angularDamping = 0.4f;
                }

                Body body = m_world.CreateBody(bd);

                body.CreateFixture(fd);

                Vector2 anchor = new Vector2(i, y);
                jd.Initialize(prevBody, body, anchor);
                m_world.CreateJoint(jd);

                prevBody = body;
            }

            m_ropeDef.localAnchorB = Vector2.Zero;

            float extraLength = 0.01f;
            m_ropeDef.maxLength = N - 1.0f + extraLength;
            m_ropeDef.body1 = prevBody;

            m_ropeDef.body2 = ground.Body;
            m_rope = m_world.CreateJoint(m_ropeDef);
        }

        public override void Keyboard(KeyboardDevice keyboard, InputManager input)
        {
            base.Keyboard(keyboard, input);

            if (keyboard.GetKey(Keys.J).Pressed)
            {
                if (m_rope != null)
                {
                    m_world.DestroyJoint(m_rope);
                    m_rope = null;
                }
                else
                {
                    m_rope = m_world.CreateJoint(m_ropeDef);
                }
            }
        }

        public override void Step(Microsoft.Xna.Framework.Graphics.GraphicsDevice graphicsDevice, Settings settings)
        {
            base.Step(graphicsDevice, settings);
        }

        public override void Draw(Settings settings, Microsoft.Xna.Framework.Graphics.SpriteBatch spriteBatch, Microsoft.Xna.Framework.Graphics.SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Press (j) to toggle the rope joint.");
            m_textLine += 15;

            if (m_rope != null)
            {
                m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Rope ON");
            }
            else
            {
                m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Rope OFF");
            }
        }

        RopeJointDef m_ropeDef = new RopeJointDef();
        Joint m_rope;
    }
}

