﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1409                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-09-09 07:27:44 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class SensorTest : Test
    {
        static int e_count = 7;
        Fixture m_sensor; 
        Body[] m_bodies = new Body[e_count]; 
        Boolean[] m_touching = new Boolean[e_count]; 
 
        public SensorTest(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            {
                RigidBody2D ground = PhysicSystem2D.Instance.createEdge(new Vector2(-40.0f, 0.0f), new Vector2(40.0f, 0.0f));

#if false 
                { 
                    b2FixtureDef sd; 
                    sd.SetAsBox(10.0f, 2.0f, b2Vec2(0.0f, 20.0f), 0.0f); 
                    sd.isSensor = true; 
                    m_sensor = ground->CreateFixture(&sd); 
                } 
#else 
                m_sensor = ground.addCircle(0.0f, 10.0f, 5.0f);
                m_sensor.SetSensor(true);
#endif 
            } 

            { 
                CircleShape shape = new CircleShape(); 
                shape.m_radius = 1.0f; 

                for (int i = 0; i < e_count; ++i) 
                { 
                    BodyDef bd = new BodyDef(); 
                    bd.type = BodyType.DYNAMIC;
                    bd.position = new Vector2(-10.0f + 3.0f * i, 20.0f); 
                    bd.userData = m_touching[i]; 

                    m_touching[i] = false; 
                    m_bodies[i] = m_world.CreateBody(bd); 

                    m_bodies[i].CreateFixture(shape, 1.0f); 
                } 
            } 
        }

        public override void Step(GraphicsDevice graphicsDevice, Settings settings)
        {
            base.Step(graphicsDevice, settings);

            // Traverse the contact results. Apply a force on shapes 
            // that overlap the sensor. 
            for (int i = 0; i < e_count; ++i)
            {
                Body body = m_bodies[i];
                if (((Boolean)body.UserData) == false) continue;
                Body ground = m_sensor.GetBody();

                CircleShape circle = (CircleShape)m_sensor.GetShape();
                Vector2 center = ground.GetWorldPoint(ref circle.m_p);

                Vector2 position = body.Position;

                Vector2 d = center - position;
                if (d.LengthSquared() < PhysicSettings.FLT_EPSILON_SQUARED)
                {
                    continue;
                }

                d.Normalize();
                Vector2 F = 100.0f * d;
                body.ApplyForce(F, position);
            } 
        }

        public override void BeginContact(Contact contact)
        {
            base.BeginContact(contact);

            Fixture fixtureA = contact.GetFixtureA();
            Fixture fixtureB = contact.GetFixtureB();

            if (fixtureA == m_sensor)
            {
                fixtureB.GetBody().UserData = true;
            }

            if (fixtureB == m_sensor)
            {
                fixtureA.GetBody().UserData = true;
            }
        }

        public override void EndContact(Contact contact)
        {
            base.EndContact(contact);

            Fixture fixtureA = contact.GetFixtureA();
            Fixture fixtureB = contact.GetFixtureB();

            if (fixtureA == m_sensor)
            {
                fixtureB.GetBody().UserData = false;
            }

            if (fixtureB == m_sensor)
            {
                fixtureA.GetBody().UserData = false;
            }
        }
    }
}
