﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1430                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-09-18 15:46:54 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class SliderCrank : Test
    {
        RevoluteJoint m_joint1;
        PrismaticJoint m_joint2; 
         
        public SliderCrank(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            RigidBody2D ground = PhysicSystem2D.Instance.createEdge(new Vector2(-40.0f, 0.0f), new Vector2(40.0f, 0.0f));

            RigidBody2D prevBody = ground;

            // Define crank.
            RigidBody2D body = PhysicSystem2D.Instance.createBox(0.0f, 7.0f, 0.5f, 2.0f, 0.2f, 2.0f);
            m_joint1 = PhysicSystem2D.Instance.createRevoluteJoint(prevBody, body, new Vector2(0.0f, 5.0f));
            m_joint1.SetMotorSpeed(1.0f * (float)System.Math.PI);
            m_joint1.SetMaxMotorTorque(10000.0f);
            m_joint1.EnableMotor(true);
            prevBody = body;

            // Define follower. 
            body = PhysicSystem2D.Instance.createBox(0.0f, 13.0f, 0.5f, 4.0f, 0.2f, 2.0f);
            PhysicSystem2D.Instance.createRevoluteJoint(prevBody, body, new Vector2(0.0f, 9.0f));
            prevBody = body;

            // Define piston 
            body = PhysicSystem2D.Instance.createBox(0.0f, 17.0f, 1.5f, 1.5f, 0.2f, 2.0f);
            body.Body.FixedRotation = true;
            PhysicSystem2D.Instance.createRevoluteJoint(prevBody, body, new Vector2(0.0f, 17.0f));

            PrismaticJoint m_joint2 = PhysicSystem2D.Instance.createPrismaticJoint(ground, body, new Vector2(0.0f, 17.0f), new Vector2(0.0f, 1.0f));
            m_joint2.SetMaxMotorForce(1000.0f);
            m_joint2.EnableLimit(false);
            m_joint2.EnableMotor(true);

            // Create a payload 
            PhysicSystem2D.Instance.createBox(0.0f, 23.0f, 1.5f, 1.5f);
        }

        public override void Keyboard(KeyboardDevice keyboard, InputManager input)
        {
            base.Keyboard(keyboard, input);

            if (keyboard.GetKey(Keys.F).Pressed)
            {
                m_joint2.EnableMotor(!m_joint2.IsMotorEnabled());
                m_joint2.GetBodyB().Awake = true;
            }

            if (keyboard.GetKey(Keys.M).Pressed)
            {
                m_joint1.EnableMotor(!m_joint1.IsMotorEnabled());
                m_joint1.GetBodyB().Awake = true;
            }
        }

        public override void Draw(Settings settings, SpriteBatch spriteBatch, SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Keys: (f) toggle friction, (m) toggle motor");
            m_textLine += 15;
            float torque = m_joint1.GetMotorTorque(settings.hz);
            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, String.Format("Motor Torque = {0}", torque));
            m_textLine += 15; 
         }
    }
}
