﻿#region Using Statements
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1482                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-10-07 11:53:48 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class TheoJansen : Test
    {
        Vector2 m_offset;
        Body m_chassis;
        Body m_wheel;
        RevoluteJoint m_motorJoint;
        bool m_motorOn;
        float m_motorSpeed;


	    public TheoJansen(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            m_offset = new Vector2(0.0f, 8.0f); 
            m_motorSpeed = 2.0f; 
            m_motorOn = true;
            Vector2 pivot = new Vector2(0.0f, 0.8f); 

            // Ground 
            { 
                BodyDef bd = new BodyDef(); 
                Body ground = m_world.CreateBody(bd); 

                EdgeShape shape = new EdgeShape();
                shape.Set(new Vector2(-50.0f, 0.0f), new Vector2(50.0f, 0.0f)); 
                ground.CreateFixture(shape, 0.0f); 

                shape.Set(new Vector2(-50.0f, 0.0f), new Vector2(-50.0f, 10.0f));
                ground.CreateFixture(shape, 0.0f); 

                shape.Set(new Vector2(50.0f, 0.0f), new Vector2(50.0f, 10.0f));
                ground.CreateFixture(shape, 0.0f); 
            }

            // Balls 
            for (int i = 0; i < 40; ++i) 
            { 
                CircleShape shape = new CircleShape(); 
                shape.m_radius = 0.25f; 

                BodyDef bd = new BodyDef(); 
                bd.type = BodyType.DYNAMIC; 
                bd.position = new Vector2(-40.0f + 2.0f * i, 0.5f); 

                Body body = m_world.CreateBody(bd); 
                body.CreateFixture(shape, 1.0f); 
            } 

            // Chassis 
            { 
                PolygonShape shape = new PolygonShape();
                shape.SetAsBox(2.5f, 1.0f); 

                FixtureDef sd = new FixtureDef();
                sd.density = 1.0f;
                sd.shape = shape; 
                sd.filter.groupIndex = -1; 
                BodyDef bd = new BodyDef();
                bd.type = BodyType.DYNAMIC;
                bd.position = pivot + m_offset;
                m_chassis = m_world.CreateBody(bd);
                m_chassis.CreateFixture(sd);
            } 

            { 
                CircleShape shape = new CircleShape();
                shape.m_radius = 1.6f;

                FixtureDef sd = new FixtureDef();
                sd.density = 1.0f;
                sd.shape = shape;
                sd.filter.groupIndex = -1;
                BodyDef bd = new BodyDef();
                bd.type = BodyType.DYNAMIC;
                bd.position = pivot + m_offset;
                m_wheel = m_world.CreateBody(bd); 
                m_wheel.CreateFixture(sd); 
            } 

            { 
                RevoluteJointDef jd = new RevoluteJointDef();
                jd.Initialize(m_wheel, m_chassis, pivot + m_offset);
                jd.collideConnected = false;
                jd.motorSpeed = m_motorSpeed;
                jd.maxMotorTorque = 400.0f;
                jd.enableMotor = m_motorOn;
                m_motorJoint = (RevoluteJoint)m_world.CreateJoint(jd);
            } 

            Vector2 wheelAnchor; 

            wheelAnchor = pivot + new Vector2(0.0f, -0.8f); 

            CreateLeg(-1.0f, wheelAnchor); 
            CreateLeg(1.0f, wheelAnchor); 

            m_wheel.SetTransform(m_wheel.Position, 120.0f * (float)System.Math.PI / 180.0f); 
            CreateLeg(-1.0f, wheelAnchor); 
            CreateLeg(1.0f, wheelAnchor); 

            m_wheel.SetTransform(m_wheel.Position, -120.0f * (float)System.Math.PI / 180.0f); 
            CreateLeg(-1.0f, wheelAnchor); 
            CreateLeg(1.0f, wheelAnchor); 
        }

        void CreateLeg(float s, Vector2 wheelAnchor) 
        { 
            Vector2 p1 = new Vector2(5.4f * s, -6.1f); 
            Vector2 p2 = new Vector2(7.2f * s, -1.2f); 
            Vector2 p3 = new Vector2(4.3f * s, -1.9f); 
            Vector2 p4 = new Vector2(3.1f * s, 0.8f); 
            Vector2 p5 = new Vector2(6.0f * s, 1.5f); 
            Vector2 p6 = new Vector2(2.5f * s, 3.7f); 

            FixtureDef fd1 = new FixtureDef(), fd2 = new FixtureDef(); 
            fd1.filter.groupIndex = -1; 
            fd2.filter.groupIndex = -1; 
            fd1.density = 1.0f; 
            fd2.density = 1.0f; 

            PolygonShape poly1 = new PolygonShape(), poly2 = new PolygonShape(); 

            if (s > 0.0f) 
            { 
                Vector2[] vertices = new Vector2[3]; 

                vertices[0] = p1; 
                vertices[1] = p2; 
                vertices[2] = p3; 
                poly1.Set(vertices, 3); 

                vertices[0] = Vector2.Zero; 
                vertices[1] = p5 - p4; 
                vertices[2] = p6 - p4; 
                poly2.Set(vertices, 3); 
            } 
            else 
            {
                Vector2[] vertices = new Vector2[3]; 

                vertices[0] = p1; 
                vertices[1] = p3; 
                vertices[2] = p2; 
                poly1.Set(vertices, 3); 

                vertices[0] = Vector2.Zero;
                vertices[1] = p6 - p4;
                vertices[2] = p5 - p4;
                poly2.Set(vertices, 3);
            } 

            fd1.shape = poly1; 
            fd2.shape = poly2; 

            BodyDef bd1 = new BodyDef(), bd2 = new BodyDef();
            bd1.type = BodyType.DYNAMIC; 
            bd2.type = BodyType.DYNAMIC; 
            bd1.position = m_offset;
            bd2.position = p4 + m_offset;

            bd1.angularDamping = 10.0f;
            bd2.angularDamping = 10.0f; 

            Body body1 = m_world.CreateBody(bd1); 
            Body body2 = m_world.CreateBody(bd2); 

            body1.CreateFixture(fd1); 
            body2.CreateFixture(fd2); 

            DistanceJointDef djd = new DistanceJointDef();

            // Using a soft distance constraint can reduce some jitter. 
            // It also makes the structure seem a bit more fluid by 
            // acting like a suspension system. 
            djd.dampingRatio = 0.5f;
            djd.frequencyHz = 10.0f;

            djd.Initialize(body1, body2, p2 + m_offset, p5 + m_offset); 
            m_world.CreateJoint(djd);

            djd.Initialize(body1, body2, p3 + m_offset, p4 + m_offset);
            m_world.CreateJoint(djd);

            djd.Initialize(body1, m_wheel, p3 + m_offset, wheelAnchor + m_offset);
            m_world.CreateJoint(djd);

            djd.Initialize(body2, m_wheel, p6 + m_offset, wheelAnchor + m_offset);
            m_world.CreateJoint(djd);

            RevoluteJointDef rjd = new RevoluteJointDef();

            rjd.Initialize(body2, m_chassis, p4 + m_offset);
            m_world.CreateJoint(rjd);
        }

        //public override void Step(Settings settings)
        //{
        //    base.Step(settings);
        //}

        public override void Draw(Settings settings, Microsoft.Xna.Framework.Graphics.SpriteBatch spriteBatch, Microsoft.Xna.Framework.Graphics.SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Keys: left = a, brake = s, right = d, toggle motor = m");
            m_textLine += 15;

        }

        public override void Keyboard(KeyboardDevice keyboard, starLiGHT.Input.InputManager input)
        {
            base.Keyboard(keyboard, input);

            if (keyboard.GetKey(Keys.A).Pressed)
            {
                m_motorJoint.SetMotorSpeed(-m_motorSpeed);
            }

            if (keyboard.GetKey(Keys.S).Pressed)
            {
                m_motorJoint.SetMotorSpeed(0.0f);
            }

            if (keyboard.GetKey(Keys.D).Pressed)
            {
                m_motorJoint.SetMotorSpeed(m_motorSpeed);
            }

            if (keyboard.GetKey(Keys.M).Pressed)
            {
                m_motorJoint.EnableMotor(!m_motorJoint.IsMotorEnabled());
            }
        }
    }
}
