﻿#region Using Statements
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT;
using System.Diagnostics;
using System;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1409                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-09-09 07:27:44 #$:  Date of last commit     *
// ************************************************************
#endregion

/// This stress tests the dynamic tree broad-phase. This also shows that tile 
/// based collision is _not_ smooth due to Box2D not knowing about adjacency. 
namespace starLiGHT.Sandbox.Physic2D
{
    public class Tiles : Test
    {
        public static int e_count = 20;

        public Tiles(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            m_fixtureCount = 0;
            Stopwatch timer = Stopwatch.StartNew();

            { 
                float a = 0.5f; 
                BodyDef bd = new BodyDef(); 
                bd.position.Y = -a; 
                Body ground = m_world.CreateBody(bd); 

                int N = 200; 
                int M = 10; 
                Vector2 position = new Vector2(); 
                position.Y = 0.0f; 
                for (int j = 0; j < M; ++j) 
                { 
                     position.X = -N * a; 
                     for (int i = 0; i < N; ++i) 
                     { 
                             PolygonShape shape = new PolygonShape(); 
                             shape.SetAsBox(a, a, position, 0.0f); 
                             ground.CreateFixture(shape, 0.0f);
                             ++m_fixtureCount;
                             position.X += 2.0f * a; 
                     } 
                     position.Y -= 2.0f * a; 
                } 
            } 
  
            { 
                float a = 0.5f; 
                PolygonShape shape = new PolygonShape(); 
                shape.SetAsBox(a, a); 

                Vector2 x = new Vector2(-7.0f, 0.75f); 
                Vector2 y; 
                Vector2 deltaX = new Vector2(0.5625f, 1.25f); 
                Vector2 deltaY = new Vector2(1.125f, 0.0f); 

                for (int i = 0; i < e_count; ++i) 
                { 
                     y = x; 

                     for (int j = i; j < e_count; ++j) 
                     { 
                             BodyDef bd = new BodyDef(); 
                             bd.type = BodyType.DYNAMIC; 
                             bd.position = y; 
                             Body body = m_world.CreateBody(bd); 
                             body.CreateFixture(shape, 5.0f);
                             ++m_fixtureCount;
                             y += deltaY; 
                     } 

                     x += deltaX; 
                } 
            } 

            timer.Stop();
            m_createTime = timer.Elapsed.TotalMilliseconds;
        }

        //public override void Step(Settings settings)
        //{
        //    base.Step(settings);
        //}

        public override void Draw(Settings settings, SpriteBatch spriteBatch, SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            ContactManager cm = m_world.GetContactManager(); 
            int height = cm.BroadPhase.ComputeHeight(); 
            int leafCount = cm.BroadPhase.GetProxyCount(); 
            int minimumNodeCount = 2 * leafCount - 1; 
            float minimumHeight = (float)System.Math.Ceiling(System.Math.Log(minimumNodeCount) / System.Math.Log(2.0f)); 
            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, String.Format("dynamic tree height = {0}, min = {1}", height, (int)minimumHeight)); 
            m_textLine += 15; 
            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, String.Format("create time = {0}, fixture count = {1}", m_createTime, m_fixtureCount));
            m_textLine += 15;
        }

        private int m_fixtureCount;
        private double m_createTime;
    }
}
