﻿#region Using Statements
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1409                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-09-09 07:27:44 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class TimeOfImpact : Test
    {
        PolygonShape m_shapeA = new PolygonShape();
        PolygonShape m_shapeB = new PolygonShape();
        System.String debug1;
        System.String debug2;

        public TimeOfImpact(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            m_shapeA.SetAsBox(25.0f, 5.0f);
            m_shapeB.SetAsBox(2.5f, 2.5f);
        }

        //public override void Step(Settings settings)
        //{
        //    base.Step(settings);

 
        //}

        public override void  Draw(Settings settings, SpriteBatch spriteBatch, SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, debug1); 
            m_textLine += 15; 

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, debug2); 
            m_textLine += 15;

            Sweep sweepA = new Sweep();
            sweepA.c0 = new Vector2(24.0f, -60.0f);
            sweepA.a0 = 2.95f;
            sweepA.c = sweepA.c0;
            sweepA.a = sweepA.a0;
            sweepA.localCenter = Vector2.Zero;

            Sweep sweepB = new Sweep();
            sweepB.c0 = new Vector2(53.474274f, -50.252514f);
            sweepB.a0 = 513.36676f; // - 162.0f * b2_pi;
            sweepB.c = new Vector2(54.595478f, -51.083473f);
            sweepB.a = 513.62781f; //  - 162.0f * b2_pi;
            sweepB.localCenter = Vector2.Zero;

            //sweepB.a0 -= 300.0f * b2_pi;
            //sweepB.a -= 300.0f * b2_pi;

            TOIInput input = new TOIInput();
            input.proxyA.Set(m_shapeA, 0);
            input.proxyB.Set(m_shapeB, 0);
            input.sweepA = sweepA;
            input.sweepB = sweepB;
            input.tMax = 1.0f;

            TOIOutput output = new TOIOutput();

            starLiGHT.Physic.TwoD.Collision.TimeOfImpact(out output, input);

            debug1 = System.String.Format("toi = {0}", output.t);
            //TODO: debug2 = System.String.Format("max toi iters = {0}, max root iters = {1}", b2_toiMaxIters, b2_toiMaxRootIters); 

            Vector2[] vertices = new Vector2[PhysicSettings.MaxPolygonVertices];

            Transform transformA;
            sweepA.GetTransform(out transformA, 0.0f);
            for (int i = 0; i < m_shapeA.m_vertexCount; ++i)
            {
                vertices[i] = Math.Mul(transformA, m_shapeA.m_vertices[i]);
            }
            m_debugDraw.DrawPolygon(vertices, m_shapeA.m_vertexCount, new Color(0.9f, 0.9f, 0.9f));

            Transform transformB;
            sweepB.GetTransform(out transformB, 0.0f);

            Vector2 localPoint = new Vector2(2.0f, -0.1f);
            Vector2 rB = Math.Mul(transformB, localPoint) - sweepB.c0;
            float wB = sweepB.a - sweepB.a0;
            Vector2 vB = sweepB.c - sweepB.c0;
            Vector2 v = vB + Vector2Extensions.Cross(wB, rB);

            for (int i = 0; i < m_shapeB.m_vertexCount; ++i)
            {
                vertices[i] = Math.Mul(transformB, m_shapeB.m_vertices[i]);
            }
            m_debugDraw.DrawPolygon(vertices, m_shapeB.m_vertexCount, new Color(0.5f, 0.9f, 0.5f));

            sweepB.GetTransform(out transformB, output.t);
            for (int i = 0; i < m_shapeB.m_vertexCount; ++i)
            {
                vertices[i] = Math.Mul(transformB, m_shapeB.m_vertices[i]);
            }
            m_debugDraw.DrawPolygon(vertices, m_shapeB.m_vertexCount, new Color(0.5f, 0.7f, 0.9f));

            sweepB.GetTransform(out transformB, 1.0f);
            for (int i = 0; i < m_shapeB.m_vertexCount; ++i)
            {
                vertices[i] = Math.Mul(transformB, m_shapeB.m_vertices[i]);
            }
            m_debugDraw.DrawPolygon(vertices, m_shapeB.m_vertexCount, new Color(0.9f, 0.5f, 0.5f));

#if false 
            for (float32 t = 0.0f; t < 1.0f; t += 0.1f) 
            { 
            sweepB.GetTransform(&transformB, t); 
            for (int32 i = 0; i < m_shapeB.m_vertexCount; ++i) 
            { 
                 vertices[i] = b2Mul(transformB, m_shapeB.m_vertices[i]); 
            } 
            m_debugDraw.DrawPolygon(vertices, m_shapeB.m_vertexCount, b2Color(0.9f, 0.5f, 0.5f)); 
            } 
#endif 

        }
    }
}
