﻿#region Using Statements
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using starLiGHT.Physic.TwoD;
using starLiGHT.Input;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using starLiGHT;
using starLiGHT.Physic;

#endregion

#region LICENSE

/*
    * This file is part of starLiGHT.Collision.
    * Copyright (c) 2007-2010 Roland Rosenkranz http://www.starLiGHT-Engine.de

    * This software is provided 'as-is', without any express or implied
    * warranty.  In no event will the authors be held liable for any damages
    * arising from the use of this software.
    * Permission is granted to anyone to use this software for any purpose,
    * including commercial applications, and to alter it and redistribute it
    * freely, subject to the following restrictions:
    * 1. The origin of this software must not be misrepresented; you must not
    * claim that you wrote the original software. If you use this software
    * in a product, an acknowledgment in the product documentation would be
    * appreciated but is not required.
    * 2. Altered source versions must be plainly marked as such, and must not be
    * misrepresented as being the original software.
    * 3. This notice may not be removed or altered from any source distribution.
    
    * ADDITIONAL (commercial) LICENSES for starLiGHT.Collision are available on request.
*/

#endregion

#region Original Box2D License

/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#endregion

#region Version Stuff
// **************[ starLiGHT.Engine SVN ]**********************
// * $Rev:: 1409                  $:  Revision of last commit *
// * $Author:: glatzemann         $:  Author of last commit   *
// * $Date:: 2010-09-09 07:27:44 #$:  Date of last commit     *
// ************************************************************
#endregion

namespace starLiGHT.Sandbox.Physic2D
{
    public class Web : Test
    {
	    RigidBody2D[] m_bodies = new RigidBody2D[4];
	    Joint[] m_joints = new Joint[8];

        public Web(Microsoft.Xna.Framework.Game game)
            : base(game, 3.0f)
        {
            RigidBody2D ground = PhysicSystem2D.Instance.createEdge(new Vector2(-40.0f, 0.0f), new Vector2(40.0f, 0.0f));

            m_bodies[0] = PhysicSystem2D.Instance.createBox(-5.0f,  5.0f, 0.5f, 0.5f, 0.2f, 5.0f);
            m_bodies[1] = PhysicSystem2D.Instance.createBox( 5.0f,  5.0f, 0.5f, 0.5f, 0.2f, 5.0f);
            m_bodies[2] = PhysicSystem2D.Instance.createBox( 5.0f, 15.0f, 0.5f, 0.5f, 0.2f, 5.0f);
            m_bodies[3] = PhysicSystem2D.Instance.createBox(-5.0f, 15.0f, 0.5f, 0.5f, 0.2f, 5.0f);

            m_joints[0] = PhysicSystem2D.Instance.createDistanceJoint(ground, new Vector2(-10.0f,  0.0f), m_bodies[0], new Vector2(-0.5f, -0.5f));
            m_joints[1] = PhysicSystem2D.Instance.createDistanceJoint(ground, new Vector2( 10.0f,  0.0f), m_bodies[1], new Vector2( 0.5f, -0.5f));
            m_joints[2] = PhysicSystem2D.Instance.createDistanceJoint(ground, new Vector2( 10.0f, 20.0f), m_bodies[2], new Vector2( 0.5f,  0.5f));
            m_joints[3] = PhysicSystem2D.Instance.createDistanceJoint(ground, new Vector2(-10.0f, 20.0f), m_bodies[3], new Vector2(-0.5f,  0.5f));
            m_joints[4] = PhysicSystem2D.Instance.createDistanceJoint(m_bodies[0], new Vector2( 0.5f,  0.0f), m_bodies[1], new Vector2(-0.5f,  0.0f));
            m_joints[5] = PhysicSystem2D.Instance.createDistanceJoint(m_bodies[1], new Vector2( 0.0f,  0.5f), m_bodies[2], new Vector2( 0.0f, -0.5f));
            m_joints[6] = PhysicSystem2D.Instance.createDistanceJoint(m_bodies[2], new Vector2(-0.5f,  0.0f), m_bodies[3], new Vector2( 0.5f,  0.0f));
            m_joints[7] = PhysicSystem2D.Instance.createDistanceJoint(m_bodies[3], new Vector2( 0.0f, -0.5f), m_bodies[0], new Vector2( 0.0f,  0.5f));

            for (int i = 0; i < m_joints.Length; i++)
            {
                ((DistanceJoint)m_joints[i]).SetFrequency(2.0f);
                ((DistanceJoint)m_joints[i]).SetDampingRatio(0.0f);
            }
        }

        public override void Keyboard(KeyboardDevice keyboard, starLiGHT.Input.InputManager input)
        {
            base.Keyboard(keyboard, input);

            if (keyboard.GetKey(Keys.B).Pressed)
            {
                for (int i = 0; i < 4; i++)
                {
                    if (m_bodies[i] != null)
                    {
                        JointEdge jn = m_bodies[i].Body.JointList;
                        while (jn != null)
                        {
                            for (int j = 0; j < 8; j++)
                            {
                                if (m_joints[j] == jn.joint)
                                    m_joints[j] = null;
                            }
                            jn = jn.next;
                        }

                        m_world.DestroyBody(m_bodies[i].Body);
                        m_bodies[i] = null;
                        break;
                    }
                }
            }

            if (keyboard.GetKey(Keys.J).Pressed)
            {
                for (int i = 0; i < 8; i++)
                {
                    if (m_joints[i] != null)
                    {
                        m_world.DestroyJoint(m_joints[i]);
                        m_joints[i] = null;
                        break;
                    }
                }
            }
        }

        public override void Step(GraphicsDevice graphicsDevice, Settings settings)
        {
            settings.drawJoints = 1;
            base.Step(graphicsDevice, settings);
        }

        public override void Draw(Settings settings, SpriteBatch spriteBatch, SpriteFont font)
        {
            base.Draw(settings, spriteBatch, font);

            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "This demonstrates a soft distance joint.");
            m_textLine += 15;
            m_debugDraw.DrawString(spriteBatch, font, 5, m_textLine, "Press: (b) to delete a body, (j) to delete a joint.");
            m_textLine += 15;
        }

        public override void JointDestroyed(Joint joint)
        {
            base.JointDestroyed(joint);

            for (int i = 0; i < 8; ++i)
            {
                if (m_joints[i] == joint)
                {
                    m_joints[i] = null;
                    break;
                }
            }
        }
    }
}
